/*******************************************************************************
* Copyright (c) 2007-2013 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFile;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFolder;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.core.utils.BzrTeamStatus;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.local.BzrLocalFile;
import org.vcs.bazaar.eclipse.internal.core.model.local.BzrLocalFolder;

/**
 * This idea for a repository specific WorkspaceRoot has been taken form
 * Subclipse's SVNWorkspaceRoot. This class provides static methods for common
 * operations between a project and it's repository (i.e: converting IResources
 * into Bazaar Resources)
 * <p>
 * Instances of this class represent a local workspace root (i.e. a project).
 * <br>
 * Here is how you can get a BzrWorkspaceRoot from an IProject : <br>
 * <code>
 * BazaarTeamProvider prov = RepositoryProvider.getProvider(project); <br>
 * BzrWorkspaceRoot bzrProject = prov.getBzrWorkspaceRoot();
 * </code>
 * </p>
 */
public class BzrWorkspaceRoot {

	private static final Map<IBzrBranch, IBazaarClient> CLIENTS = new ConcurrentHashMap<IBzrBranch, IBazaarClient>();
	private static final Map<IBzrBranch, IBazaarClient> NO_LOGS_CLIENTS = new ConcurrentHashMap<IBzrBranch, IBazaarClient>();
	private IBzrLocalBranch branch;

	public BzrWorkspaceRoot(IProject project) {
		this.branch = BranchFactory.findOrCreateFor(project);
		try {
			ensureMetaFolderIsTeamPrivate(project);
		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
		}
	}

	/**
	 * Returns a resource path to the given local location. Returns null if it
	 * is not under a project's location.
	 *
	 * @see FileSystemResourceManager.pathForLocation(org.eclipse.core.runtime.IPath)
	 */
	public static IPath pathForLocation(IPath location) {
		if (Platform.getLocation().equals(location))
			return Path.ROOT;
		IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (int i = 0; i < projects.length; i++) {
			IProject project = projects[i];
			IPath projectLocation = project.getLocation();
			if (projectLocation != null && projectLocation.isPrefixOf(location)) {
				int segmentsToRemove = projectLocation.segmentCount();
				return project.getFullPath().append(location.removeFirstSegments(segmentsToRemove));
			}
		}
		return null;
	}

	/**
	 * get the IBzrLocalFile for the given resource
	 * @param resource
	 * @return IBzrLocalFile
	 */
	public static IBzrLocalFile getBzrFileFor(IResource resource) {
		IBzrBranch branch = BranchFactory.findOrCreateFor(resource.getProject());
		return new BzrLocalFile(resource, branch);
	}

	/**
	 * get the IBzrLocalFolder for the given resource
	 * @param resource
	 * @return IBzrLocalFolder
	 */
	public static IBzrLocalFolder getBzrFolderFor(IContainer resource) {
		IBzrBranch branch = BranchFactory.findOrCreateFor(resource.getProject());
		return new BzrLocalFolder(resource, branch);
	}

	/**
	 * get the IBzrLocalResource for the given resource
	 * @param resource
	 * @return IBzrLocalResource
	 */
	public static IBzrLocalResource getBzrResourceFor(IResource resource) {
		if (resource.getType() == IResource.FILE) {
			return getBzrFileFor(resource);
		} else {
			return getBzrFolderFor((IContainer) resource);
		}
	}

	/**
	 * get the IBzrLocalResource's for the given resources
	 *
	 * @param resources
	 * @return IBzrLocalResource[]
	 */
	public static IBzrLocalResource[] getBzrResourcesFor(IResource resources[]) {
		IBzrLocalResource[] bzrResources = new IBzrLocalResource[resources.length];
		for (int i = 0; i < resources.length; i++) {
			bzrResources[i] = getBzrResourceFor(resources[i]);
		}
		return bzrResources;
	}

	/**
	 * get the branch for this project
	 * @return IBzrLocalBranch
	 */
	public IBzrLocalBranch getBranch() {
		return this.branch;
	}

	public static BzrWorkspaceRoot getBzrWorkspaceRootFor(IProject project) {
		BazaarTeamProvider provider = getProviderFor(project);
		if(provider == null)
			return null;
		return provider.getBzrWorkspaceRoot();
	}

	public static BazaarTeamProvider getProviderFor(IProject project) {
		BazaarTeamProvider provider = (BazaarTeamProvider) RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID());
		return provider;
	}

	public static IBzrLocalBranch getBranchFor(IProject project) {
		BazaarTeamProvider provider = getProviderFor(project);
		if(provider == null)
			return null;
		return provider.getBzrWorkspaceRoot().getBranch();
	}

	public IBazaarClient getClient() throws BazaarException {
		return getClient(this.branch);
	}

	public IBazaarClient getClient(boolean disableLogs) throws BazaarException {
		return getClient(this.branch, disableLogs);
	}

	public static IBazaarClient getClient(IBzrBranch branch) throws BazaarException {
		return getClient(branch, false);
	}

	public static IBazaarClient getClient(IBzrBranch branch, boolean disableLogs) throws BazaarException {
		Map<IBzrBranch, IBazaarClient> clients = disableLogs ? NO_LOGS_CLIENTS : CLIENTS;
		IBazaarClient client = clients.get(branch);
		if (client == null) {
			synchronized (clients) {
				client = clients.get(branch);
				if (client == null) {
					try {
						client = EclipseBazaarCore.getDefault().createClient();
						if(branch.isLocal()) {
							client.setWorkDir(branch.getFile());
						}
						if (disableLogs) {
							client.getNotificationHandler().disableLog();
						}
						clients.put(branch, client);
						return client;
					} catch (BazaarException e) {
						if(!EclipseBazaarCore.getDefault().isClientAvailable()) {
							throw new BazaarException(new BzrTeamStatus(BzrTeamStatus.ERROR, "Bazaar client not available." + System.getProperty("line.separator") + " Please check the bazaar and bzr-xmloutput plugin installation", e));
						}
						throw e;
					}
				}
			}
		}
		return client;
	}

	/**
	 * This check if the project contains a .bzr folder, if true set as teamPrivateMember
	 * @param project
	 * @throws CoreException
	 */
	private void ensureMetaFolderIsTeamPrivate(IProject project) throws CoreException {
		if (project != null) {
			IResource metaFolder = project.findMember(new Path(IBzrBranch.DIR_META_FOLDER));
			if (metaFolder != null && metaFolder.exists())
				metaFolder.setTeamPrivateMember(true);
			metaFolder = project.findMember(new Path(IBzrBranch.DIR_META_FOLDER+".backup"));
			if (metaFolder != null && metaFolder.exists())
				metaFolder.setTeamPrivateMember(true);
		}
	}

	public static IProject[] findProjectsInBranch(IResource resource) {
		IProject project = resource.getProject();
		IBzrBranch branch = getProviderFor(project).getBranch();
		return findProjectsInBranch(branch);
	}

	public static IProject[] findProjectsInBranch(IBzrBranch branch) {
		List<IProject> projects = new ArrayList<IProject>();
		for (IProject p : ResourcesPlugin.getWorkspace().getRoot().getProjects()) {
			if (p.isAccessible()) {
				BazaarTeamProvider provider = getProviderFor(p);
				if (provider != null) {
					if (branch.equals(provider.getBranch())) {
						projects.add(p);
					}
				}
			}
		}
		return projects.toArray(new IProject[projects.size()]);
	}

	public static void purgeClients() {
		CLIENTS.clear();
		NO_LOGS_CLIENTS.clear();
	}
	
	public static boolean isLightweightCheckout(final IProject project) {
		IPath location = ResourceUtil.findRootLocation(project);
		boolean lightweight = false;
		if (location != null) {
			File metaFolder = new File(location.toFile(), IBzrBranch.DIR_META_FOLDER);
			File branchFolder = new File(metaFolder, IBzrBranch.BRANCH_FOLDER);
			File locationFile = new File(branchFolder, IBzrBranch.LOCATION_FILE);
			if (locationFile.exists()) {
				lightweight = true;
			}
		}
		return lightweight;
	}
}
