/*******************************************************************************
* Copyright (c) 2007-2013 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.ISafeRunnable;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.Plugin;
import org.eclipse.core.runtime.SafeRunner;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.BazaarClientPreferences;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarNotifyListener;
import org.vcs.bazaar.client.IBazaarPromptUserPassword;
import org.vcs.bazaar.client.IPlugin;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BazaarClientException.BazaarUncheckedException;
import org.vcs.bazaar.client.xmlrpc.XmlRpcClient;
import org.vcs.bazaar.eclipse.core.BzrSaveParticipant;
import org.vcs.bazaar.eclipse.core.client.DelegatingConsoleListener;
import org.vcs.bazaar.eclipse.core.credentials.DelegatingCredentialsProvider;
import org.vcs.bazaar.eclipse.core.credentials.SecureCredentialsStore;
import org.vcs.bazaar.eclipse.core.listeners.AbstractResourceChangeListener;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.FileModificationManager;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.listeners.SyncFileChangeListener;
import org.vcs.bazaar.eclipse.core.log.LogCacheManager;
import org.vcs.bazaar.eclipse.core.notifications.DelegatingNotificationListener;
import org.vcs.bazaar.eclipse.core.notifications.Notification;
import org.vcs.bazaar.eclipse.core.notifications.NotificationListener;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.status.StatusCacheManager;
import org.vcs.bazaar.eclipse.core.utils.PreferenceHelper;
import org.vcs.bazaar.eclipse.core.utils.TracingUtils;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * The main plugin class to be used in the desktop.
 *
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail DOT com>
 */
@SuppressWarnings("deprecation")
public class EclipseBazaarCore extends Plugin {

	public static final String ID = "org.vcs.bazaar.eclipse.core";
	
	// The shared instance.
	private static EclipseBazaarCore plugin;
	private static final List<IResourceStateListener> stateListeners = new ArrayList<IResourceStateListener>();

	private StatusCacheManager statusCacheManager;
	private LogCacheManager logCacheManager;
	/** this listener will listen to modifications to files */
	private final AbstractResourceChangeListener fileModificationManager = new FileModificationManager();
	private final AbstractResourceChangeListener metaDirChangeListener = new SyncFileChangeListener();
	private boolean clientAvailable = false;
	private Set<IPlugin> bzrPlugins;
	private SecureCredentialsStore secureCredentialsStore;
	private DelegatingCredentialsProvider credentialsProvider = new DelegatingCredentialsProvider();
	private DelegatingConsoleListener consoleListener = new DelegatingConsoleListener();
	private DelegatingNotificationListener notificationListener = new DelegatingNotificationListener();
	/**
	 * The constructor.
	 */
	public EclipseBazaarCore() {
		plugin = this;
	}

	@Override
    public void start(BundleContext context) throws Exception {
		super.start(context);
		statusCacheManager = new StatusCacheManager();
		logCacheManager = new LogCacheManager();
		secureCredentialsStore = new SecureCredentialsStore();
		ResourcesPlugin.getWorkspace().addResourceChangeListener(fileModificationManager, IResourceChangeEvent.POST_CHANGE);
		ResourcesPlugin.getWorkspace().addResourceChangeListener(metaDirChangeListener, IResourceChangeEvent.POST_CHANGE);
		// TODO: register as save participant
		// fileModificationManager.registerSaveParticipant();
		ResourcesPlugin.getWorkspace().addSaveParticipant(this, new BzrSaveParticipant());
		try {
			checkClient();
		} catch (BazaarException e) {
			log(e);
		}
	}

	@Override
    public void stop(BundleContext context) throws Exception {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(fileModificationManager);
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(metaDirChangeListener);
		synchronized (stateListeners) {
			stateListeners.clear();
		}
		ResourcesPlugin.getWorkspace().removeSaveParticipant(this);
		super.stop(context);
		plugin = null;
	}

	/**
	 * @return plugin Returns the shared instance.
	 */
	public static EclipseBazaarCore getDefault() {
		return plugin;
	}

	public static final String getPluginId() {
		return ID;
	}

	public static String getProviderID() {
		return BazaarTeamProvider.ID;
	}

	/**
	 * Create and returns a new IBazaarClient instance.<br>
	 * If the client can't be created, a exception is thrown.
	 *
	 * @return
	 * @throws BazaarException
	 */
	public IBazaarClient createClient() throws BazaarException {
		boolean checkPlugins = bzrPlugins == null || !clientAvailable;
		checkClient();
		try {
			IBazaarClient client = BazaarClientFactory.createClient(BazaarClientFactory.getPreferredClientType());
			if (client != null) {
				if (checkPlugins) {
					try {
						bzrPlugins = client.plugins();
					} catch (Exception e) { // Do nothing, we can do without the list of plugins 
					}
				}
				// to prevent adding a null listener (this happen when the core
				// plugin is initialized before the ui plugin)
				client.addNotifyListener(consoleListener);
				client.addPasswordCallback(credentialsProvider);
			}
			return client;
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} catch (BazaarUncheckedException e) {
			log(e.getMessage(), e);
			throw BazaarException.wrapException(e);
		}
	}

	public void checkClient() throws BazaarException {
		if (!clientAvailable) {
			checkClient(getClientPropertiesFromPreferences());
		}
	}

	public void checkClient(ClientProperties props) throws BazaarException {
		updateClientPreferences(props);
		try {
			BazaarClientFactory.setupBestAvailableBackend(true);
			clientAvailable = true;
		} catch (BazaarClientException e) {
			clientAvailable = false;
			throw BazaarException.wrapException(e);
		}
	}

	public String[] getAvailableClientTypes() {
		 return BazaarClientFactory.getClientTypes();
	}

	public boolean isClientAvailable() {
		return clientAvailable;
	}

	public void setConsoleListener(IBazaarNotifyListener listener) {
		consoleListener.setConsoleListener(listener);
	}

	public static void addResourceStateListener(final IResourceStateListener listener) {
		synchronized (stateListeners) {
			if(!stateListeners.contains(listener)){
				stateListeners.add(listener);
			}
		}
	}

	public static void removeResourceStateListener(final IResourceStateListener listener) {
		synchronized (stateListeners) {
			stateListeners.remove(listener);
		}
	}

	/**
	 * get the resource status cache
	 * @return statusCacheManager the {@link StatusCacheManager} singleton
	 */
	public StatusCacheManager getStatusCacheManager() {
		return statusCacheManager;
	}

	public LogCacheManager getLogCacheManager() {
		return logCacheManager;
	}

	public static void broadcastModificationStateChanges(final BzrChangeStateEvent event) {
		EclipseBazaarCore.trace("", "broadcastModificationStateChanges: ", event);
		final IResourceStateListener[] toNotify;
		synchronized (stateListeners) {
			toNotify = stateListeners.toArray(new IResourceStateListener[stateListeners.size()]);
		}

		for (int i = 0; i < toNotify.length; ++i) {
			final IResourceStateListener listener = toNotify[i];
			ISafeRunnable code = new ISafeRunnable() {
				public void run() throws Exception {
					listener.resourceStateChanged(event);
				}

				public void handleException(Throwable e) {
					// don't log the exception....it is already being logged in
					// Platform#run
				}
			};
			SafeRunner.run(code);
		}
	}


	/**
	 * Progress monitor helpers
	 * @param monitor
	 */
	public static void checkCanceled(IProgressMonitor monitor) {
		if (monitor != null && monitor.isCanceled()) {
            throw new OperationCanceledException();
        }
	}
	public static IProgressMonitor monitorFor(IProgressMonitor monitor) {
		if (monitor == null) {
            return new NullProgressMonitor();
        }
		return monitor;
	}

	/**
	 * creates a {@link SubProgressMonitor} with 1000 ticks
	 * @param monitor
	 * @return
	 */
	public static IProgressMonitor subProgressMonitorFor(IProgressMonitor monitor) {
		if (monitor == null) {
            return new NullProgressMonitor();
        }
		if (monitor instanceof NullProgressMonitor) {
            return monitor;
        }
		return new SubProgressMonitor(monitor, 1000);
	}

	// logging and tracing (taken from EclipseDarcsCore - @author The mighty PDE
	// wizard dskñsa)
	// ////////////////////

	public static void log(final String message, final int severity) {
		Status status = new Status(severity, getPluginId(), IStatus.OK, message, null);
		getDefault().getLog().log(status);
	}

	public static void log(final String message, final Throwable thr) {
		Status status = new Status(IStatus.ERROR, getPluginId(), IStatus.OK, message, thr);
		getDefault().getLog().log(status);
	}

	public static void log(CoreException ce) {
		getDefault().getLog().log(ce.getStatus());
	}

	public static boolean isTracing(final String optionId) {
		String option = getPluginId() + "/trace/" + optionId; //$NON-NLS-1$
		String value = Platform.getDebugOption(option);
		return "true".equals(value); //$NON-NLS-1$
	}

	public static void trace(final String optionId, final String what, Object...objects) {
		if (isTracing(optionId)) {
			TracingUtils.trace(what, objects);
		}
	}

	public static void traceException(final String optionId, final Exception ex) {
		if (isTracing(optionId)) {
			TracingUtils.trace("", ex);
		}
	}

	public IPlugin getPlugin(String pluginName) {
		if (clientAvailable && bzrPlugins != null) {
			for (IPlugin plugin : bzrPlugins) {
				if (pluginName.equals(plugin.getName())) {
					return plugin;
				}
			}
		}
		return null;
	}

	public SecureCredentialsStore getSecureCredentialsStore() {
		return secureCredentialsStore;
	}

	public void setCredentialsProvider(IBazaarPromptUserPassword provider) {
		credentialsProvider.setCredentialsProvider(provider);
	}

	public void setNotificationListener(NotificationListener listener) {
		notificationListener.setNotificationListener(listener);
	}

	protected ClientProperties getCurrentClientProperties() {
		BazaarClientPreferences clientPreferences = getClientPreferences();
		String executable = clientPreferences.getString(BazaarPreference.EXECUTABLE);
		String email = clientPreferences.getString(BazaarPreference.BZR_EMAIL);
		String port = clientPreferences.getString(BazaarPreference.BZR_XMLRPC_PORT);
		String pluginPath = clientPreferences.getString(BazaarPreference.BZR_PLUGIN_PATH);
		return new ClientProperties(executable, email, port, pluginPath);
	}

	protected ClientProperties getClientPropertiesFromPreferences() {
		String executable = PreferenceHelper.get(BazaarPreference.EXECUTABLE.name());
		String email = PreferenceHelper.get(BazaarPreference.BZR_EMAIL.name());
		String port = PreferenceHelper.get(BazaarPreference.BZR_XMLRPC_PORT.name());
		return getClientProperties(executable, email, port);
	}

	public ClientProperties getClientProperties(String executable, String email, String port) {
		if (executable == null || executable.trim().isEmpty()) {
			executable = getClientPreferences().getAutodetectedExecutable();
		}
		String bundledPluginPath = getBundledPluginsPath();
		return new ClientProperties(executable, email, port, bundledPluginPath);
	}

	
	/**
	 * Set bazaar java library preferences from Eclipse preferences.
	 */
	private void updateClientPreferences(ClientProperties properties) {
		if (!getCurrentClientProperties().equals(properties)) {
			stopClient();
			BazaarClientPreferences clientPreferences = getClientPreferences();
			clientPreferences.set(BazaarPreference.EXECUTABLE, properties.getExecutable());
			if (properties.getEmail() != null && ! properties.getEmail().trim().isEmpty()) {
				clientPreferences.set(BazaarPreference.BZR_EMAIL, properties.getEmail());
			} else {
				clientPreferences.unset(BazaarPreference.BZR_EMAIL);
			}
			clientPreferences.set(BazaarPreference.BZR_XMLRPC_PORT, properties.getPort());
			if (properties.getPluginPath() != null) {
				clientPreferences.set(BazaarPreference.BZR_PLUGIN_PATH, properties.getPluginPath());
			} else {
				clientPreferences.unset(BazaarPreference.BZR_PLUGIN_PATH);
			}
		}
	}

	private void stopClient() {
		if (clientAvailable) {
			try {
				final IBazaarClient client = BazaarClientFactory.createClient(BazaarClientFactory.getPreferredClientType());
				if (client instanceof XmlRpcClient) {
					((XmlRpcClient)client).stopXmlRpcService();
				}
			} catch (BazaarClientException e) {
				log(e.getMessage(), e);
			} catch (BazaarUncheckedException e) {
				log(e.getMessage(), e);
			}
			BzrWorkspaceRoot.purgeClients();
			clientAvailable = false;
		}
	}

	private BazaarClientPreferences getClientPreferences() {
		return BazaarClientPreferences.getInstance();
	}

	private String getBundledPluginsPath() {
		Bundle clientBundle = null;
		for (Bundle bundle : this.getBundle().getBundleContext().getBundles()) {
			if ("org.vcs.bazaar.eclipse.client".equals(bundle.getSymbolicName().trim())) {
				clientBundle = bundle;
				break;
			}
		}
		try {
			URL entry = FileLocator.resolve(FileLocator.findEntries(clientBundle, new Path("plugins"))[0]);
			return new File(entry.getPath()).getPath();
		} catch (IOException e) {
			IStatus status = new Status(IStatus.ERROR, this.getClass().getPackage().getName(), e.getMessage(), e);
			this.getLog().log(status);
			return null;
		}
	}

	public void sendNotification(Notification notification) {
		notificationListener.message(notification);
	}
}
