/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public abstract class BazaarCommand implements IBazaarCommand {

	/**
	 * Create and return a Map of File's per project
	 *
	 * @param resources IBzrResource[]
	 * @return Map<IProject,List<File>>
	 */
	protected static final Map<IProject, List<File>> getFilesByProjectMap(IBzrLocalResource[] resources) {
		Map<IProject, List<File>> map = new HashMap<IProject, List<File>>();
		for (IBzrLocalResource bzrResource : resources) {
			IProject project = bzrResource.getIResource().getProject();
			List<File> list = map.get(project);
			if (list == null) {
				list = new ArrayList<File>();
				map.put(project, list);
			}
			list.add(bzrResource.getIResource().getLocation().toFile());
		}
		return map;
	}

	/**
	 * Create and return a Map of File's per branch
	 *
	 * @param resources List<IBzrResource>
	 * @return Map<IBzrBranch, List<File>>
	 */
	protected static final Map<IBzrBranch, List<File>> getFilesByBranchMap(List<IBzrLocalResource> resources) {
		return getFilesByBranchMap(resources.toArray(new IBzrLocalResource[0]));
	}

	protected static final Map<IBzrBranch, List<File>> getFilesByBranchMap(IBzrLocalResource[] resources) {
		Map<IBzrBranch, List<File>> map = new HashMap<IBzrBranch, List<File>>();
		for (IBzrLocalResource bzrResource : resources) {
			IBzrBranch branch = bzrResource.getBranch();
			List<File> list = map.get(branch);
			if (list == null) {
				list = new ArrayList<File>();
				map.put(branch, list);
			}
			list.add(bzrResource.getFile());
		}
		return map;
	}

	protected static Map<IBzrBranch, List<IProject>> getProjectsByBranch(IProject[] projects) {
		Map<IBzrBranch, List<IProject>> map = new HashMap<IBzrBranch, List<IProject>>();
		for (IProject project : projects) {
			IBzrBranch branch = BzrWorkspaceRoot.getBranchFor(project);
			List<IProject> list = map.get(branch);
			if (list == null) {
				list = new ArrayList<IProject>();
				map.put(branch, list);
			}
			list.add(project);
		}
		return map;
	}

	protected static final IBazaarClient getClient(IProject project) throws BazaarException {
		return BzrWorkspaceRoot.getBzrWorkspaceRootFor(project).getClient();
	}

	protected static final IBazaarClient getClient(IProject project, boolean disableLogs) throws BazaarException {
		return BzrWorkspaceRoot.getBzrWorkspaceRootFor(project).getClient(disableLogs);
	}

	protected static final IBazaarClient getClient(final IBzrBranch branch) throws BazaarException {
		return BzrWorkspaceRoot.getClient(branch);
	}

	protected static final IBazaarClient getClient(final IBzrBranch branch, boolean disableLogs) throws BazaarException {
		return BzrWorkspaceRoot.getClient(branch, disableLogs);
	}

	protected static final List<File> getFiles(Collection<IBzrLocalResource> resources) {
		ArrayList<File> list = new ArrayList<File>();
		for (IBzrLocalResource bzrResource : resources) {
			list.add(bzrResource.getFile());
		}
		return list;
	}
	
	public abstract void run(IProgressMonitor monitor) throws BazaarException;

}
