/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;

import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BazaarClientException.BazaarUncheckedException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez TODO: add revision support and write a testcase
 */
public class BranchCommand implements IBazaarCommand {

	private final IBzrBranch remoteLocation;
	private BazaarRevision revision;
	private final File toLocation;

	public BranchCommand(final IBzrBranch sourceLocation, final File toLocation, String revision) {
		this.remoteLocation = sourceLocation;
		this.toLocation = toLocation;
		if (revision != null && revision.trim().length() != 0) {
			try {
				this.setRevision(BazaarRevision.getRevision(BazaarRevision.Prefix.REVNO, revision));
			} catch (BazaarUncheckedException e) {
				// TODO Handle bad revision
				EclipseBazaarCore.log(e.getMessage(), e);
				this.setRevision(null);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.vcs.bazaar.eclipse.core.operations.IBazaarOperation#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	public void run(IProgressMonitor monitor) throws BazaarException {
		try {
			monitor.beginTask("", 100);
			final IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			client.setWorkDir(getToLocation());
			runOnClient(client, new CommandProgressListener(monitor));
		} catch (BazaarClientException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
			// Clean up any mess
			ResourceUtil.deleteRecursively(getToLocation());
			throw BazaarException.wrapException(e);
		} finally {
			monitor.setTaskName("");
			monitor.done();
		}
	}

	/**
	 * Run the command on the given client, within the context of a progress monitor.
	 */
	protected void runOnClient(IBazaarClient client, CommandProgressListener cpl) 
		throws BazaarClientException {
		client.branch(getRemoteLocation(), getToLocation(), getRevision(), cpl);
	}

	/**
	 * @return the toLocation
	 */
	protected File getToLocation() {
		return toLocation;
	}

	/**
	 * @param revision the revision to set
	 */
	protected void setRevision(BazaarRevision revision) {
		this.revision = revision;
	}

	/**
	 * @return the revision
	 */
	protected BazaarRevision getRevision() {
		return revision;
	}

	/**
	 * @return the remoteLocation
	 */
	protected BranchLocation getRemoteLocation() {
		return remoteLocation.getBranchLocation();
	}

}
