/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class CommitCommand extends StateChangeCommand {

	private String commitMessage;
	private List<IBzrLocalResource> resources;
	final private List<Option> options;
	private boolean fullCommit;
	private boolean fullRefresh;

	/**
	 *
	 * @param filesToCommit
	 * 					a list of IBzrResource that can belongs to different branches
	 * @param commitMessage
	 * @param localCommit
	 */
    public CommitCommand(List<IBzrLocalResource> filesToCommit, String commitMessage, List<Option> options) {
		this.resources = filesToCommit;
		this.commitMessage = commitMessage;
		this.options = options;
	}

    /**
     *
     * @param filesToCommit
     * @param commitMessage
     * @param options
     * @param fullCommit if tru, the filesToCommit 'll be ignored and a full commit 'll be executed (commit with 0 arguments)
     */
    public CommitCommand(List<IBzrLocalResource> filesToCommit, String commitMessage, List<Option> options, 
    		boolean fullCommit, boolean fullRefresh) {
		this(filesToCommit, commitMessage, options);
		this.fullCommit = fullCommit;
		this.fullRefresh = fullRefresh;
	}

	/**
	 * execute the commit.
	 * Actually this method call client.commit(..) for each group of IBzrResource that belong
	 * to the same branch
	 *
	 */
	@Override
    public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		Map<IBzrBranch, List<File>> map = getFilesByBranchMap(resources);
		try {
			monitor.beginTask(CoreTexts.CommitCommand_commit, resources.size());
			monitor.setTaskName(CoreTexts.CommitCommand_commit);
			for (IBzrBranch branch : map.keySet()) {
				IBazaarClient client = BzrWorkspaceRoot.getClient(branch);
                if(fullCommit) {
                	client.commit(new File[]{branch.getFile()}, commitMessage, options.toArray(new Option[options.size()]));
                } else {
                	client.commit(map.get(branch).toArray(new File[0]), commitMessage, options.toArray(new Option[options.size()]));
                }
			}
			if (fullCommit || fullRefresh) {
				List<IProject> projects = new ArrayList<IProject>();
				for (IBzrBranch branch : map.keySet()) {
					projects.addAll(Arrays.asList(BzrWorkspaceRoot.findProjectsInBranch(branch)));
				}
				return projects.toArray(new IProject[0]);
			}
			monitor.worked(resources.size());
			return getIResources(resources);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

}
