/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ILogOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResource;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class LogCommand extends BazaarCommand {

	private IBzrResource resource;
	private List<IBazaarLogMessage> logMessages;
	private Integer limit;
	private BazaarRevision revision;
	private boolean verbose;

	/**
	 * limit the log size using the value of {@link IBazaarCoreConstants#LOG_REVISION_LIMIT}
	 * in the plugin preferences.
	 */

	/**
	 *
	 * @param bzrResource - the {@link IBzrResource} to fetch the log
	 * @param revision - revision to get the log for
	 */
	public LogCommand(IBzrResource bzrResource, BazaarRevision revision, boolean verbose) {
		this.resource = bzrResource;
		this.revision = revision;
		this.verbose = verbose;
	}

	/**
	 *
	 * @param bzrResource - the {@link IBzrResource} to fetch the log
	 * @param limit - the number of revisions to fetch
	 */
	public LogCommand(IBzrResource bzrResource, Integer limit) {
		this.resource = bzrResource;
		this.limit = limit;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.vcs.bazaar.eclipse.core.commands.IBazaarCommand#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
    public void run(IProgressMonitor monitor) throws BazaarException {
		try {
			if(resource == null)
				return;
			monitor.beginTask(CoreTexts.LogCommand_log, 1000);
			monitor.setTaskName(CoreTexts.LogCommand_log);
			IBazaarClient client = getClient(resource.getBranch(), true);
			List<Option> options = new ArrayList<Option>();
			if (limit != null) {
				options.add(ILogOptions.LIMIT.setArgument(limit.toString()));
			} 
			if (revision != null) {
				options.add(ILogOptions.REVISION.setArgument(revision.toString()));
			}
			if (verbose) {
				options.add(ILogOptions.VERBOSE);
			}
			if(resource instanceof IBzrBranch) {
				logMessages = client.log(((IBzrBranch)resource).getBranchLocation(), options.toArray(new Option[0]));
			} else if (resource instanceof IBzrLocalResource) {
				IBzrLocalResource localResource = ((IBzrLocalResource)resource);
				logMessages = client.log(localResource.getFile(), options.toArray(new Option[0]));
			} else {
				logMessages = client.log(resource.getLocation(), options.toArray(new Option[0]));
			}
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

	public List<IBazaarLogMessage> getLogMessages() {
		if(logMessages == null) {
			logMessages = Collections.emptyList();
		}
		return logMessages;
	}

}
