/*******************************************************************************
* Copyright (c) 2007-2013 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRunnable;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.SubProgressMonitor;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.BzrTeamStatus;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public abstract class MetaModifyCommand extends BazaarCommand {

	protected static final Pattern CONFLICT_PATTERN = Pattern.compile("conflict.*encountered");
	protected static final Pattern CONFLICT_LINE_PATTERN = Pattern.compile(".*conflict.*", Pattern.CASE_INSENSITIVE);

	protected BazaarException postCommandError;

	protected boolean isTouchRequired() {
		return false;
	}

	@Override
    public void run(IProgressMonitor monitor) throws BazaarException {
		monitor.beginTask("", 100);
		IProgressMonitor subMonitor = new SubProgressMonitor(monitor, 90);
		final IResource[] resources = execute(subMonitor);
		subMonitor = new SubProgressMonitor(monitor, 10);
		try {
			ResourcesPlugin.getWorkspace().run(new IWorkspaceRunnable() {
				public void run(IProgressMonitor monitor) throws CoreException {
					try {
						monitor.beginTask("", resources.length);
						monitor.setTaskName(CoreTexts.MetaModifyCommad_refreshing);
						for (IResource resource : resources) {
							if(resource.exists()) {
								if (monitor.isCanceled()) {
									break;
								}
								if (isTouchRequired() && resource.isSynchronized(IResource.DEPTH_INFINITE)) {
									resource.touch(new SubProgressMonitor(monitor, 1));
								} else {
									resource.refreshLocal(IResource.DEPTH_INFINITE, new SubProgressMonitor(monitor, 1));
								}
								ResourceUtil.markAsDirty(resource);
							}
							monitor.worked(1);
						}
					} finally {
						monitor.done();
					}
				}
			}, subMonitor);
			List<IResource> resourceToRefresh = Arrays.asList(resources);
			EclipseBazaarCore.getDefault().getStatusCacheManager().scheduleRefresh(resourceToRefresh, new NullProgressMonitor(), getChangeType());
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
		if (getPostCommandError() != null) {
			throw getPostCommandError();
		}
	}

	public abstract IResource[] execute(IProgressMonitor monitor) throws BazaarException;

	protected abstract int getChangeType();

	protected BazaarException getPostCommandError() {
		return postCommandError;
	}
	
	// helpers
	protected static IResource[] getIResources(IBzrLocalResource[] bzrResources) {
		List<IResource> resources = new ArrayList<IResource>(bzrResources.length);
		for (IBzrLocalResource resource : bzrResources) {
			resources.add(resource.getIResource());
		}
		return resources.toArray(new IResource[0]);
	}

	protected static IResource[] getIResources(List<IBzrLocalResource> bzrResources) {
		return getIResources(bzrResources.toArray(new IBzrLocalResource[0]));
	}

	protected boolean setPostCommandErrorOnConflicts(String message) {
		if (message != null && CONFLICT_PATTERN.matcher(message).find()) {
			Matcher m = CONFLICT_LINE_PATTERN.matcher(message);
			StringBuilder sb = new StringBuilder();
			while (m.find()) {
				if (sb.length() > 0) {
					sb.append("\n");
				}
				sb.append(m.group());
			}
			postCommandError = new BazaarException(BzrTeamStatus.INFO, IStatus.OK, sb.toString());
			return true;
		}
		return false;
	}

}
