/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Piotr Piastucki
 */
public class ReconfigureCommand extends BazaarCommand {

	private final IBzrBranch branch;

	public ReconfigureCommand(IBzrBranch branch) {
		this.branch = branch;
	}

	private BranchLocation getBranchLocation() {
		IPath location = branch.getPath();
		if (location != null) {
			File metaFolder = new File(location.toFile(), IBzrBranch.DIR_META_FOLDER);
			File branchFolder = new File(metaFolder, IBzrBranch.BRANCH_FOLDER);
			File locationFile = new File(branchFolder, IBzrBranch.LOCATION_FILE);
			if (locationFile.exists()) {
				BufferedReader reader = null;
				try {
					reader = new BufferedReader(new FileReader(locationFile));
					return new BranchLocation(reader.readLine());
				} catch (Exception e) {
				} finally {
					if (reader != null) {
						try {
							reader.close();
						} catch (IOException e) {
						}
					}
				}
			}
		}
		return null;
	}
	
	@Override
	public void run(IProgressMonitor monitor) throws BazaarException {
		try {
			monitor.beginTask(CoreTexts.bind(CoreTexts.Reconfiguring_taskName, branch.getBranchLocation().toString()), 100); //$NON-NLS-1$
			IBazaarClient client = getClient(branch, false);
			// Unfortunately bzr reconfigure --checkout fails in case of lightweight checkouts (see https://bugs.launchpad.net/bzr/+bug/562896)
			// so use some hacks to simulate bzr reconfigure instead.
			// client.reconfigure(branch.getBranchLocation(), new CommandProgressListener(monitor), IReconfigureOptions.CHECKOUT);
			File tempFolder = File.createTempFile("bzreclipse", null);
			tempFolder.delete();
			BranchLocation remoteLocation = getBranchLocation();
			if (remoteLocation != null) {
				// 1. run regular checkout
				client.checkout(remoteLocation, tempFolder, new CommandProgressListener(monitor));
				// 2. copy repository folder
				moveFolder(tempFolder, IBzrBranch.REPOSITORY_FOLDER);
				// 3. copy branch folder
				moveFolder(tempFolder, IBzrBranch.BRANCH_FOLDER);
				ResourceUtil.deleteRecursively(tempFolder);
			} else {
				//	throw an error?? 
			}
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} catch (IOException e) {
			throw BazaarException.wrapException(e);
		}
	}

	private void moveFolder(File tempFolder, String folder) {
		File srcRepository = new File(tempFolder.getAbsolutePath() + File.separator + IBzrBranch.DIR_META_FOLDER + File.separator + folder);
		File destRepository = branch.getPath().append(IBzrBranch.DIR_META_FOLDER).append(folder).toFile();
		ResourceUtil.deleteRecursively(destRepository);
		srcRepository.renameTo(destRepository);
	}

}
