/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class ResolveCommand extends ResourceChangeCommand {

	private final IBzrLocalResource[] resources;
	private final Option[] options;

	/**
	 * This constructor should to be used only on non-project resources (only files and folders)
	 * @param resources
	 */
	public ResolveCommand(Collection<IResource> resources) {
		this(BzrWorkspaceRoot.getBzrResourcesFor(resources.toArray(new IResource[0])));
	}

	/**
	 * @see #ResolveCommand(Collection)
	 * @param bzrResources
	 */
	public ResolveCommand(IBzrLocalResource[] bzrResources,  Option...options) {
		this.resources = bzrResources;
		this.options = options;
	}

	public ResolveCommand(Collection<IResource> resources, Option...options) {
		this(BzrWorkspaceRoot.getBzrResourcesFor(resources.toArray(new IResource[0])), options);
	}

	@Override
    public IResource[] execute(IProgressMonitor monitor) throws BazaarException {
		try {
			if(resources != null){
				return resolveResources(monitor);
			} else {
				return new IResource[0];
			}
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} finally {
			monitor.done();
		}
	}

	/**
	 * @param monitor
	 * @return {@link IResource}[] array of changed resources
	 * @throws BazaarException
	 * @throws BazaarClientException
	 */
	private IResource[] resolveResources(IProgressMonitor monitor) throws BazaarException, BazaarClientException {
		Map<IBzrBranch, List<File>> map = getFilesByBranchMap(resources);
		monitor.beginTask("Resolving", map.keySet().size());
		for (IBzrBranch branch : map.keySet()) {
			IBazaarClient client = getClient(branch);
			client.resolve(map.get(branch), options);
			monitor.worked(1);
		}
		List<IResource> changed = new ArrayList<IResource>(resources.length);
		for (IResource resource : getIResources(resources)) {
			changed.add(resource);
		}
		return changed.toArray(new IResource[changed.size()]);
	}

	@Override
	protected boolean isTouchRequired() {
		return true;
	}

}
