/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.vcs.bazaar.client.BazaarStatus;
import org.vcs.bazaar.client.BazaarStatusKind;
import org.vcs.bazaar.client.BazaarTreeStatus;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.IBazaarRevisionSpec;
import org.vcs.bazaar.client.IBazaarStatus;
import org.vcs.bazaar.client.commandline.commands.options.KeywordOption;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IStatusOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.BzrResourceStatus;

/**
 * @author Guillermo Gonzalez
 *
 * TODO: extend from abstract BazaarCommand, and implement multi project
 * selection
 */
public class StatusCommand extends BazaarCommand {

	private final IBzrLocalResource[] resources;
	private Map<IResource, IBzrResourceStatus> statusMap;
	private boolean noLog = false;
	/* if set folders with status 'unknown' will be scanned and all sub-folders/files will be added to status map with status 'unknown' */
	private boolean addNestedUnknown = true;
	private List<IBazaarLogMessage> pendingMerges;
	private Option[] options = new Option[0];

	/**
	 * Invoked by StatusCommand
	 * @param resources
	 */
	public StatusCommand(IBzrLocalResource[] resources) {
		this(resources, false);
		this.addNestedUnknown = false;
	}

	/**
	 * Used by commit action to build list of commitable resources
	 * @param project
	 * @param resources
	 * @param noLog
	 */
	public StatusCommand(IBzrLocalResource[] resources, boolean noLog) {
		super();
		this.resources = resources;
		this.noLog = noLog;
	}

	public StatusCommand(IBzrLocalResource[] resources, IBazaarRevisionSpec revision) {
		this(resources, true);
		options = new KeywordOption[]{new KeywordOption(IStatusOptions.REVISION.getOption(), revision.toString())};
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see org.vcs.bazaar.eclipse.core.commands.IBazaarCommand#run(org.eclipse.core.runtime.IProgressMonitor)
	 */
	@Override
    public void run(IProgressMonitor monitor) throws BazaarException {
		IProgressMonitor subMonitor = EclipseBazaarCore.subProgressMonitorFor(monitor);
		try {
			statusMap = new HashMap<IResource, IBzrResourceStatus>(0);
			pendingMerges = new ArrayList<IBazaarLogMessage>();
			Map<IBzrBranch, List<File>> map = getFilesByBranchMap(resources);
			for (IBzrBranch branch : map.keySet()) {
				if(!subMonitor.isCanceled()) {
					executeAndFilterLogOutput(branch, map.get(branch).toArray(new File[0]));
				}
			}
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e);
		} catch (BazaarException e) {
			throw e;
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		} finally {
			subMonitor.done();
		}
	}

	public IBzrResourceStatus[] getStatus() {
		List<IBzrResourceStatus> list = new ArrayList<IBzrResourceStatus>(statusMap.values().size());
		Collections.addAll(list, statusMap.values().toArray(new IBzrResourceStatus[0]));
		return list.toArray(new IBzrResourceStatus[0]);
	}

	public List<IBazaarLogMessage> getPendingMerges() {
		return pendingMerges;
	}

	/**
	 * This method is a workaround to prevent a lot of output when is called
	 * from DecoratorStatus
	 * @param branch {@link IBazaarClient}
	 * @param files {@link File}[]
	 * @throws BazaarClientException
	 * @throws CoreException
	 */
	private void executeAndFilterLogOutput(final IBzrBranch branch, File[] files) throws BazaarClientException, CoreException {
		IBazaarClient client = getClient(branch, noLog);
		BazaarTreeStatus treeStatus = client.status(files, options);
		pendingMerges = treeStatus.getPendingMerges();
		IBazaarStatus[] statuses = treeStatus.getStatusAsArray();
		Set<IContainer> unknown = new HashSet<IContainer>();
		for (IBazaarStatus aStatus : statuses) {
			IResource resource = ResourceUtil.getIResource(aStatus);
			if (resource != null) {
				IBzrResourceStatus resourceStatus = new BzrResourceStatus(BzrWorkspaceRoot.getBzrResourceFor(resource), aStatus);
				statusMap.put(resource, resourceStatus);
				if (addNestedUnknown && resourceStatus.isUnknown() && resource instanceof IContainer) {
					unknown.add((IContainer) resource);
				}
			}
		}
		/* if addNestedUnknown flag is set then add all files and sub-folders inside UNKNOWN folder to the map although 
		 * normally they are not reported by bzr status command */
		for (IContainer folder : unknown) {
			folder.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					if (!statusMap.containsKey(resource)) {
						IPath relPath = resource.getLocation().makeRelativeTo(branch.getPath());
						IBazaarStatus aStatus = BazaarStatus.create(relPath.toFile(), resource instanceof IContainer, branch.getFile(), BazaarStatusKind.UNKNOWN);
						IBzrResourceStatus resourceStatus = new BzrResourceStatus(BzrWorkspaceRoot.getBzrResourceFor(resource), aStatus);
						statusMap.put(resource, resourceStatus);
					}
					return true;
				}
			}, IResource.DEPTH_INFINITE, IFolder.EXCLUDE_DERIVED);
		}
		
	}
	
}
