/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.commands;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.vcs.bazaar.client.BazaarStatusKind;
import org.vcs.bazaar.client.BazaarTreeStatus;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarItemInfo;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.IBazaarStatus;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ILsOptions;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.core.model.BzrStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * This class is invoked form StatusCacheManager to build status map that is later used to decorate resources. 
 * @author Guillermo Gonzalez
 * @author Piotr Piastucki
 */
public class StatusUpdaterCommand extends BazaarCommand {

	private final IBzrLocalResource bzrLocalResource;
	private Map<IResource, BzrStatus> statusMap;
	private Map<IResource, IBazaarLogMessage> logMap;
	private final boolean shallow;
	private final boolean verbose;
	

	public StatusUpdaterCommand(IBzrLocalResource resource, boolean shallow, boolean verbose) {
		this.bzrLocalResource = resource;
		this.shallow = shallow;
		this.verbose = verbose;
	}

	@Override
    public void run(IProgressMonitor monitor) throws BazaarException {
		try {
			statusMap = new HashMap<IResource, BzrStatus>();
			logMap = new HashMap<IResource, IBazaarLogMessage>();
			IBazaarClient client = getClient(bzrLocalResource.getIResource().getProject(), true);
			executeAndFilterLogOutput(client);
		} catch (BazaarClientException e) {
			throw BazaarException.wrapException(e, bzrLocalResource.getIResource());
		} catch (BazaarException e) {
			throw e;
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		}
	}

	public Map<IResource, BzrStatus> getStatus() {
		return statusMap;
	}

	public Map<IResource, IBazaarLogMessage> getLogMap() {
		return logMap;
	}

	private void executeAndFilterLogOutput(IBazaarClient client) throws BazaarClientException, CoreException {
		Set<IResource> versionedSet = new HashSet<IResource>();
		Set<IResource> ignoredSet = new HashSet<IResource>();
		Set<IResource> unknownSet = new HashSet<IResource>();
		IResource resourceToRefresh = bzrLocalResource.getIResource();
		File file = resourceToRefresh.getLocation().toFile();
		/* get all versioned and ignored resources with ls command excluding derived resources */
		IBazaarItemInfo[] paths;
		List<Option> options = new ArrayList<Option>();
		options.add(ILsOptions.IGNORED);
		options.add(ILsOptions.VERSIONED);
		options.add(ILsOptions.UNKNOWN);
		if (shallow) {
			options.add(ILsOptions.NON_RECURSIVE);
		}
		if (verbose) {
			options.add(ILsOptions.VERBOSE);
		}
		paths = client.ls(file, null, options.toArray(new Option[0]));
		/* execute st command and turn its result into a status map */
		BazaarTreeStatus treeStatus = client.status(new File[]{file});
		IBazaarStatus[] statuses = treeStatus.getStatusAsArray();
		/* add everything returned by bzr status command excluding derived resources */
		for (IBazaarStatus aStatus : statuses) {
			IResource resource = ResourceUtil.getIResource(aStatus);
			if (resource != null) {
				if (ResourceUtil.shouldHandle(resource, false)) {
					BzrStatus resourceStatus = new BzrStatus(aStatus);
					statusMap.put(resource, resourceStatus);
					if (resourceStatus.isUnknown() && resource instanceof IContainer) {
						unknownSet.add(resource);
					}
				}
			}
		}
		if (paths != null) {
			for (IBazaarItemInfo itemInfo : paths) {
				Path path = new Path(itemInfo.getPath());
				int matchingSegments = resourceToRefresh.getProject().getLocation().matchingFirstSegments(path);
				IResource resource = resourceToRefresh.getProject().findMember(path.removeFirstSegments(matchingSegments));
				if (resource != null) {
					if (ResourceUtil.shouldHandle(resource, false)) {
						if (itemInfo.getLog() != null) {
							logMap.put(resource, itemInfo.getLog());
						}
						if (itemInfo.getStatusKind() == BazaarStatusKind.IGNORED) {
							ignoredSet.add(resource);
						} else if (itemInfo.getStatusKind() == BazaarStatusKind.UNKNOWN) {
							/*
							 * Try to work around bzr bug, 
							 * see https://bugs.launchpad.net/bzr-eclipse/+bug/1181926
							 * and https://answers.launchpad.net/bzr/+question/229372
							 */
							if (!statusMap.containsKey(resource)) {
								ignoredSet.add(resource);
							} else {
								unknownSet.add(resource);
							}
						} else {
							versionedSet.add(resource);
						}
					}
				}
			}
			if (resourceToRefresh.getType() == IResource.PROJECT && !versionedSet.contains(resourceToRefresh)) {
				versionedSet.add(resourceToRefresh);
			}
		}
		/* add all files and sub-folders inside IGNORED folders with IGNORED status, it needs to be done manually as 
		 * ignored resources are not reported recursively by bzr ls command */
		for (IResource ignored : ignoredSet) {
			ignored.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					if (!statusMap.containsKey(resource)) {
						statusMap.put(resource, BzrStatus.ignored());
					}
					return true;
				}
			}, IResource.DEPTH_INFINITE, IFolder.EXCLUDE_DERIVED);
		}
		/* add all files and sub-folders inside UNKNOWN folders with UNKNOWN status, it needs to be done manually as 
		 * untracked resources are not reported recursively by bzr status command */
		for (IResource unknown : unknownSet) {
			unknown.accept(new IResourceVisitor() {
				public boolean visit(IResource resource) throws CoreException {
					if (!statusMap.containsKey(resource)) {
						statusMap.put(resource, BzrStatus.unknown());
					}
					return true;
				}
			}, IResource.DEPTH_INFINITE, IFolder.EXCLUDE_DERIVED);
		}
		for (IResource versioned : versionedSet) {
			if (!statusMap.containsKey(versioned)) {
				statusMap.put(versioned, BzrStatus.unchanged());
			}
		}
	}
}
