/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.credentials;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.equinox.security.storage.EncodingUtils;
import org.eclipse.equinox.security.storage.ISecurePreferences;
import org.eclipse.equinox.security.storage.SecurePreferencesFactory;

/**
 * @author Piotr Piastucki
 */
public class SecureCredentialsStore {

	private static final String USER = "user"; //$NON-NLS-1$
	private static final String PASSWORD = "password"; //$NON-NLS-1$
	private static final String BZR_PATH_PREFIX = "/BZR/"; //$NON-NLS-1$
	private static final Pattern PATTERN = Pattern.compile("\\w+://([^@]*)@(.*)"); 
	private final ISecurePreferences preferences;

	public SecureCredentialsStore() {
		preferences =  SecurePreferencesFactory.getDefault();
	}
	
	private String getPath(String location) {
		return BZR_PATH_PREFIX + EncodingUtils.encodeSlashes(location);
	}

	public static String getUserFromLocation(String location) {
		Matcher m = PATTERN.matcher(location);
		if (m.matches()) {
			return m.group(1);
		}
		return null;
	}

	public BazaarCredentials getCredentials(String location) throws SecureCredentialsStoreException {
		try {
			String pathName = getPath(location);
			if (!preferences.nodeExists(pathName))
				return null;
			ISecurePreferences node = preferences.node(pathName);
			String user = node.get(USER, ""); //$NON-NLS-1$
			String password = node.get(PASSWORD, ""); //$NON-NLS-1$
			String locationUser = getUserFromLocation(location);
			if (locationUser != null && !locationUser.equals(user))
				return null;
			return new BazaarCredentials(user, password, true);
		} catch (Exception e) {
			throw new SecureCredentialsStoreException(e.getMessage());
		}
	}

	public void putCredentials(String location, BazaarCredentials credentials) throws SecureCredentialsStoreException {
		try {
			String pathName = getPath(location);
			ISecurePreferences node = preferences.node(pathName);
			node.put(USER, credentials.getUser(), false);
			node.put(PASSWORD, credentials.getPassword(), true);
			node.flush();
		} catch (Exception e) {
			throw new SecureCredentialsStoreException(e.getMessage());
		}
	}

	public void clearCredentials(String location) throws SecureCredentialsStoreException {
		try {
			String pathName = getPath(location);
			if (!preferences.nodeExists(pathName))
				return;
			ISecurePreferences node = preferences.node(pathName);
			node.removeNode();
			node.flush();
		} catch (Exception e) {
			throw new SecureCredentialsStoreException(e.getMessage());
		}
	}

}
