/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.history;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.core.history.provider.FileHistory;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;

/**
 *
 * @author Guillermo Gonzalez
 *
 */
public class BazaarFileHistory extends FileHistory {
	private IResource resource;

	protected SortedMap<BazaarRevision, BazaarFileRevision> revisionMap;
	protected List<IBazaarLogMessage> logMessages;

	protected BazaarFileHistory() {
	}

	public BazaarFileHistory(IResource resource) {
		this.resource = resource;
	}

	public BazaarFileHistory(List<IBazaarLogMessage> logMessages) {
		this.logMessages = logMessages;
		buildRevisionMap();
	}

	public IFileRevision[] getContributors(IFileRevision revision) {
		final BazaarFileRevision bazaarRevision = (BazaarFileRevision) revision;
		final List<IBazaarLogMessage> merged = bazaarRevision.getIBazaarLogMessage().getMerged();
		if (merged != null && merged.size() > 0) {
			final List<IFileRevision> revisions = new ArrayList<IFileRevision>(merged.size());
			for (IBazaarLogMessage mergeLog : merged) {
				revisions.add(buildRevision(mergeLog));
			}
			return revisions.toArray(new IFileRevision[revisions.size()]);
		} else {
			return new IFileRevision[]{revisionMap.get(bazaarRevision.getIBazaarLogMessage().getRevision())};
		}
	}

	public IFileRevision getFileRevision(String id) {
		System.out.println("BazaarFileHistory::getFileRevision(" + id + ")");
		return null;
	}

	public IFileRevision[] getFileRevisions() {
		return revisionMap.values().toArray(new IFileRevision[0]);
	}

	public IFileRevision[] getTargets(IFileRevision revision) {
		System.out.println("BazaarFileHistory::getTargets()");
		return null;
	}

	protected void getLogMessages(boolean limit) throws CoreException {
		logMessages =  EclipseBazaarCore.getDefault().getLogCacheManager().getLogs(resource, limit);
	}

	protected boolean isBazaarProvider() {
		RepositoryProvider provider = RepositoryProvider.getProvider(resource.getProject());
		return provider != null && provider instanceof BazaarTeamProvider;
	}

	public void refresh(IProgressMonitor monitor, boolean limitLogs) throws CoreException {
		if (isBazaarProvider()) {
			monitor.subTask("Fetching log messages...");
			getLogMessages(limitLogs); 
			monitor.worked(500);
			buildRevisionMap();
		}
		monitor.worked(300);
	}

	protected void buildRevisionMap() {
		if(revisionMap == null) {
			revisionMap = new TreeMap<BazaarRevision, BazaarFileRevision>(new Comparator<BazaarRevision>() {
				public int compare(BazaarRevision rev1, BazaarRevision rev2) {
					return rev1.compareTo(rev2);
				}
			});
		} else {
			revisionMap.clear();
		}
		if (logMessages != null) {
			for (IBazaarLogMessage log : logMessages) {
				revisionMap.put(log.getRevision(), buildRevision(log));
			}
		}
	}

	protected BazaarFileRevision buildRevision(IBazaarLogMessage log) {
		return new BazaarFileRevision(log, resource);
	}

}
