/*******************************************************************************
 * Copyright (c) 2003, 2006 Subclipse project and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Subclipse project committers - initial API and implementation
 ******************************************************************************/
package org.vcs.bazaar.eclipse.core.listeners;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.core.Team;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.core.utils.PreferenceHelper;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;

/**
 * This class performs several functions related to determining the modified
 * status of files under Bazaar control. First, it listens for change
 * delta's for files and folders, and brodcasts them to all listeners.
 *
 * @modifiedBy Guillermo Gonzalez
 */
public class FileModificationManager extends AbstractResourceChangeListener {

	private static final Object BZRIGNORE_NAME = ".bzrignore"; //$NON-NLS-1$

	protected int INTERESTING_CHANGES = IResourceDelta.CONTENT //
			| IResourceDelta.MOVED_FROM //
			| IResourceDelta.MOVED_TO //
			| IResourceDelta.OPEN //
			| IResourceDelta.REPLACED //
			| IResourceDelta.TYPE;

	public void resourceChanged(IResourceChangeEvent event) {
		try {
			if(event.getBuildKind() != 0) {
				return;
			}

			final List<IResource> modifiedResources = new ArrayList<IResource>();
			final AtomicReference<Boolean> bzrignoreChanged = new AtomicReference<Boolean>(false);
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "FileModificationManager#resourceChanged");
			event.getDelta().accept(new IResourceDeltaVisitor() {
				public boolean visit(IResourceDelta delta) throws CoreException {
					final IResource resource = delta.getResource();
					EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, " DeltaVisitor: ", resource);
					if (resource.getType() == IResource.ROOT) {
						return true;
					}
					if (resource.getName().equals(BZRIGNORE_NAME)) {
						bzrignoreChanged.set(true);
						modifiedResources.add(resource.getProject());
						return false;
					}
					if (RepositoryProvider.getProvider(resource.getProject(), EclipseBazaarCore.getProviderID()) == null) {
						handleAutoShare(modifiedResources, delta, resource);
						return false;
					}
					if (delta.getKind() == IResourceDelta.CHANGED && (delta.getFlags() & IResourceDelta.DERIVED_CHANGED) != 0) {
						modifiedResources.add(resource);
						return true;
					}
					if (Team.isIgnoredHint(resource) || !ResourceUtil.shouldHandle(resource, false)) {
						return false;
					}
					if (resource.getType() == IResource.FILE) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  File");
						return handleFile(modifiedResources, delta, resource);
					} else if (resource.getType() == IResource.FOLDER) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Folder");
						return handleFolder(modifiedResources, delta, resource);
					} else if (resource.getType() == IResource.PROJECT && resource.exists()) {
						EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Project");
						return handleProject(modifiedResources, delta, (IProject)resource);
					}
					return true;
				}

			}, IFolder.EXCLUDE_DERIVED);
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "FileModificationManager#collectedChanges: ", (Object[])modifiedResources.toArray(new IResource[0]));
			// we refresh all changed resources and broadcast the changes to all
			// listeners (ex : BazaarLightweightDecorator)
			if(modifiedResources.size() > 0) {
				refreshResources(modifiedResources, bzrignoreChanged.get() ? BzrChangeStateEvent.WT_STATE_MODIFY : BzrChangeStateEvent.RESOURCE_MODIFY);
			}

		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getStatus().getMessage(), e.getStatus().getSeverity());
		}
	}


	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 */
	private boolean handleFile(final List<IResource> modifiedResources, IResourceDelta delta, final IResource resource) {
		if(resource.exists() && delta.getKind() == IResourceDelta.CHANGED && (delta.getFlags() & INTERESTING_CHANGES) != 0) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   InterestingChanges: ", resource, delta.getFlags());
			modifiedResources.add(resource);
		} else if (delta.getKind() == IResourceDelta.ADDED && resource.exists()) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   added/changed: ", resource);
			modifiedResources.add(resource);
		} else if (delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Removed: ", resource);
			// provide notifications for deletions since they
			// may not have been managed
			// The move/delete hook would have updated the
			// parent counts properly
			modifiedResources.add(resource);
		}
		return true;
	}


	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 * @return
	 * @throws CoreException
	 */
	private boolean handleFolder(final List<IResource> modifiedResources, IResourceDelta delta, final IResource resource)
			throws CoreException {
		if (delta.getKind() == IResourceDelta.ADDED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Added: ", resource);
			modifiedResources.add(resource);
			return true;
		} else if (delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Removed: ", resource);
			modifiedResources.add(resource);
			return false;
		} else if (resource.exists() && delta.getKind() == IResourceDelta.CHANGED) {
			return !(resource.isLinked(IResource.CHECK_ANCESTORS) || resource.isDerived());
		}
		return true;
	}

	/**
	 * @param modifiedResources
	 * @param delta
	 * @param resource
	 * @throws CoreException
	 */
	private boolean handleProject(final List<IResource> modifiedResources, IResourceDelta delta, final IProject project)
			throws CoreException {
		if (!project.isAccessible()) {
			EclipseBazaarCore.getDefault().getLogCacheManager().purgeCache(project);
			EclipseBazaarCore.getDefault().getStatusCacheManager().purgeCache(project);
			return false;
		} else if ((delta.getFlags() & IResourceDelta.OPEN) != 0) {
			EclipseBazaarCore.getDefault().getLogCacheManager().purgeCache(project);
			modifiedResources.add(project);
			return false;
		}
		if (delta.getKind() == IResourceDelta.ADDED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "  Added: ", project);
			modifiedResources.add(project);
			return false;
		} else if (delta.getKind() == IResourceDelta.REMOVED) {
			EclipseBazaarCore.trace(ITracingOptions.FILE_MODIFICATION_MANAGER, "   Removed", project);
			modifiedResources.add(project);
			return false;
		}
		return true;
	}

	private void handleAutoShare(List<IResource> modifiedResources, IResourceDelta delta, IResource resource) {
		if (!PreferenceHelper.getBoolean(IBazaarCoreConstants.AUTO_SHARE_PROJECTS)) {
			return;
		}
		if (resource.getType() != IResource.PROJECT) {
			return;
		}
		if (!resource.exists()) {
			return;
		}
		final IProject project = (IProject) resource;
		RepositoryProvider provider = RepositoryProvider.getProvider(project);
		if (provider != null) {
			return;
		}
		if ((delta.getFlags() & IResourceDelta.OPEN) != 0) {
			IPath root = ResourceUtil.findRootLocation(project);
			if (root != null) {
				try {
					BazaarTeamProvider.registerProject(project);
					modifiedResources.add(resource);
				} catch (CoreException e) {
					EclipseBazaarCore.log(e.getStatus().getMessage(), e.getStatus().getSeverity());
				}
			}
		}
	}

}
