/*******************************************************************************
 * Copyright (c) 2008-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.log;

import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.QualifiedName;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.IBzrResource;
import org.vcs.bazaar.eclipse.core.utils.PreferenceHelper;

/**
 * A basic/dumb Log/history cache manager.
 * ATM it's just a façade, there is no cache working.
 * TODO: need to workout a solution about how (and when)refresh the cache (and how to force a refresh)
 *
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class LogCacheManager {

	private final static int INTERESTING_CHANGES = BzrChangeStateEvent.BRANCH_STATE_MODIFY;

	/**
	 * Name used for identifying Bazaar synchronization data in
	 * Resource>ResourceInfo#syncInfo storage
	 */
	public static final QualifiedName BZR_SYNC_KEY = new QualifiedName(EclipseBazaarCore.getPluginId(), "bzr-log-sync-key");

	private LogUpdater logUpdater;

	public LogCacheManager() {
		ResourcesPlugin.getWorkspace().getSynchronizer().add(LogCacheManager.BZR_SYNC_KEY);
		logUpdater = new LogUpdater();
	}

	/**
	 * Returns the last log of the given resource. If is not present in
	 * cache, it will be retrieved using the actual strategy.
	 *
	 * @param resource whose last log is required.
	 * @throws CoreException
	 */
	public IBazaarLogMessage getLastLog(IBzrResource resource) throws CoreException {
		return getLogs(resource, 1).get(0);
	}

	/**
	 * Returns the logs of the given resource. If is not present in
	 * cache, it will be retrieved using the actual strategy.
	 *
	 * @param resource whose last log is required.
	 * @return a List<IBazaarLogMessage> for the corresponding resource
	 * (the size is dependent on the limit setup in the preferences)
	 * @throws CoreException
	 */
	public List<IBazaarLogMessage> getLogs(IResource resource, boolean limit) throws CoreException {
		return getLogs(resource, limit ? getLimit() : null);
	}

	/**
	 * see {@link #getLogs(IResource)}
	 */
	public List<IBazaarLogMessage> getLogs(IBzrResource resource, boolean limit) throws CoreException {
		return getLogs(resource, limit ? getLimit() : null);
	}

	private List<IBazaarLogMessage> getLogs(IBzrResource resource, Integer limit) throws CoreException {
		return basicGetLog(resource, logUpdater, limit);
	}

	private List<IBazaarLogMessage> getLogs(IResource resource, Integer limit) throws CoreException {
		if (!resource.isAccessible() || resource.isPhantom() || resource.isLinked()) {
			return Collections.emptyList();
		}
		return getLogs(BzrWorkspaceRoot.getBzrResourceFor(resource), limit);
	}

	private Integer getLimit() {
		Integer limit = null;
		int logLimit = PreferenceHelper.getInt(IBazaarCoreConstants.LOG_REVISION_LIMIT);
		if(logLimit != 0) {
			limit = logLimit;
		}
		return limit;
	}

	/**
	 * refresh the cache and returns if the resource is in it.
	 */
	private List<IBazaarLogMessage> basicGetLog(IBzrResource resource, LogUpdater strategy, Integer limit) throws CoreException {
		EclipseBazaarCore.trace(ITracingOptions.LOG_CACHE, "LogCacheManager.basicGetlog: " + resource.getName());
		return strategy.fetchLog(resource, new NullProgressMonitor(), limit);
	}

	/**
	 * Purge the cache of the spacified IProject from the cache
	 */
	public void purgeCache(IProject root) throws CoreException {
	}

}
