/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.resources;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.io.File;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.team.IMoveDeleteHook;
import org.eclipse.core.resources.team.IResourceTree;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalFolder;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class BazaarMoveDeleteHook implements IMoveDeleteHook {

	private boolean protectBzrMetadata(IResourceTree tree, IResource res) {
		if (BzrWorkspaceRoot.getProviderFor(res.getProject()) != null) {
			/* show an error message when any file inside .bzr folder is about to be deleted */
			if (ResourceUtil.isInsideMetaFolder(res)) {
				tree.failed(new Status(IStatus.ERROR, EclipseBazaarCore.getPluginId(), CoreTexts.MoveDeleteHook_metaFolder));
				return true;
			}
		}
		return false;
	}
	
	public boolean deleteFile(IResourceTree tree, IFile file, int updateFlags, IProgressMonitor monitor) {
		return protectBzrMetadata(tree, file);
	}

	public boolean deleteFolder(IResourceTree tree, IFolder folder, int updateFlags, IProgressMonitor monitor) {
		return protectBzrMetadata(tree, folder);
	}

	public boolean deleteProject(IResourceTree tree, IProject project, int updateFlags, IProgressMonitor monitor) {
		IPath root = ResourceUtil.findRootLocation(project);
		/* Invoke standard project deletion */
		tree.standardDeleteProject(project, updateFlags, monitor);
		if (root != null) {
			File file = root.toFile();
			if (file.exists()) {
				/* Check if there are any .project files left inside the branch */
				if (ResourceUtil.findProjects(file).isEmpty()) {
					/* No remaining projects found - remove the branch */
					ResourceUtil.deleteRecursively(file);
				}
			}
		}
		return true;
	}

	public boolean moveFile(IResourceTree tree, IFile source, IFile destination, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(source))
			return false;

		IBzrLocalResource sourceResource = BzrWorkspaceRoot.getBzrFileFor(source);
		try {
			if (!isUnderBazaarControl(sourceResource))
				return false;
			if (!BzrWorkspaceRoot.getBzrResourceFor(destination.getParent()).getBranch().equals(sourceResource.getBranch())) {
				return false;
			}
			monitor.beginTask("Moving file(s)", 1000);
			IContainer parentDest = destination.getParent();
			IBzrLocalResource parentDestResource = BzrWorkspaceRoot.getBzrFolderFor(parentDest);
			// if the parent of the destination is not versioned we must add it
			// to Bazaar first
			if (parentDestResource == null || !parentDestResource.isManaged()) {
				parentDestResource.getBzrWorkspaceRoot().getClient().add(new File[] { parentDest.getLocation().toFile() });
			}
			sourceResource.move(destination.getLocation());
			tree.movedFile(source, destination);
			destination.refreshLocal(IResource.DEPTH_ZERO, monitor);
		} catch (BazaarClientException e) {
			tree.failed(BazaarException.wrapException(e).getStatus());
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} catch (CoreException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean moveFolder(IResourceTree tree, IFolder source, IFolder destination, int updateFlags, IProgressMonitor monitor) {
		if (!shouldHandle(source))
			return false;

		// FIXME: change this when Bug #107967 get fixed
		// if I try to move source into source/newlocation
		// bazaar tells: bzr: ERROR: Could not move source => newlocation:
		// Invalid argument
		// so for the moment we don't take care of this case
		if (source.getLocation().isPrefixOf(destination.getLocation())) {
			return false;
		}

		IBzrLocalResource sourceResource = BzrWorkspaceRoot.getBzrFolderFor(source);
		try {

			if (sourceResource == null || !isUnderBazaarControl(sourceResource))
				return false;
			IBzrLocalFolder bzrDest = BzrWorkspaceRoot.getBzrFolderFor(destination.getParent());
			if (bzrDest == null || !bzrDest.getBranch().equals(sourceResource.getBranch())) {
				return false;
			}
			monitor.beginTask("Removing folder(s)", 1000);
			sourceResource.move(destination.getLocation());
			tree.movedFolderSubtree(source, destination);
			destination.refreshLocal(IResource.DEPTH_INFINITE, monitor);
		} catch (BazaarException e) {
			tree.failed(e.getStatus());
		} catch (CoreException e) {
			tree.failed(e.getStatus());
		} finally {
			monitor.done();
		}
		return true;
	}

	public boolean moveProject(IResourceTree tree, IProject source, IProjectDescription description, int updateFlags, IProgressMonitor monitor) {
		// TODO Auto-generated method stub
		return false;
	}

	private static boolean isUnderBazaarControl(IBzrLocalResource resource) {
		try {
			if(resource != null) {
				return resource.isManaged();
			}
		} catch (CoreException e) {
			EclipseBazaarCore.log(e.getMessage(), e);
		}
		return false;
	}
}
