/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.resources;

import java.io.IOException;
import java.io.InputStream;

import org.eclipse.core.resources.IEncodedStorage;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.content.IContentDescription;
import org.eclipse.core.runtime.content.IContentTypeManager;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.CatCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author zingo
 * @modifiedby Guillermo Gonzalez
 * @modifiedby Piotr Piastucki
 *
 * This is a IStorage subclass that handle file revision
 *
 */
public class IStorageBazaarRevision extends PlatformObject implements IEncodedStorage {
	private final IBzrBranch branch;
	private final IResource resource;
	private final BazaarRevision revision;
	private InputStream contents;
	// set for remote branches
	private BranchLocation location;
	// set for local branches
	private IPath path;

	/**
	 * This constructor is used to create storage with remote content
	 * @param resource
	 * @param branch
	 * @param relativePath
	 * @param revision
	 */
	public IStorageBazaarRevision(IResource resource, IBzrBranch branch, String relativePath, BazaarRevision revision) {
		this.branch = branch;
		this.resource = resource;
		this.revision = revision;
		if (branch.getPath() != null) {
			// local branch
			path = branch.getPath().append(relativePath);
		} else {
			try {
				// remote branch
				location = new BranchLocation(branch.getBranchLocation() + "/" + relativePath);
			} catch (BazaarClientException e) {
				EclipseBazaarCore.log(BazaarException.wrapException(e));
			}
		}
	}

	public IStorageBazaarRevision(IResource resource, BazaarRevision revision) {
		this.branch = BranchFactory.findOrCreateFor(resource.getProject());
		this.resource = resource;
		this.revision = revision;
		this.path = resource.getLocation();
	}

	public IStorageBazaarRevision(IStorageBazaarRevision storage, BazaarRevision revision) {
		this.branch = storage.branch;
		this.resource = storage.resource;
		this.revision = revision;
		this.path = storage.path;
		this.location = storage.location;
	}

	public IStorageBazaarRevision(IResource resource) {
		this(resource, getLastRevision(resource));
	}

	private static BazaarRevision getLastRevision(IResource resource) {
		BazaarRevision rev = null;
		try {
			rev = EclipseBazaarCore.getDefault().getStatusCacheManager().getProjectRevision(resource);
		} catch (BazaarException e) {
			EclipseBazaarCore.log(e);
		}
		return rev;
	}

	@Override
    @SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	public InputStream getContents() throws CoreException {
		if (isContainer()) {
			return null;
		}
		// store the inputStream in a private member so we don't need to call 'bzr cat' again
		if (contents == null) {
			String charsetName = null;
			if (resource != null) {
				try {
					charsetName = ((IFile)resource).getCharset();
				} catch (CoreException e) {
				}
			}
			CatCommand cmd = path != null ? new CatCommand(branch, path.toFile(), charsetName, revision) 
											: new CatCommand(branch, location, charsetName, revision);
			cmd.run(new NullProgressMonitor());
			contents = cmd.getContent();
		} else {
			// if the inputStream has been already initialized, reset it to the
			// initial position so the client can get all the contents
			try {
				contents.reset();
			} catch (IOException e) {
				throw BazaarException.wrapException(e);
			}
		}
		return contents;
	}

	public IPath getFullPath() {
		if (path != null) {
			return path;
		} else if (resource != null) {
			return resource.getLocation();
		}
		return null;
	}

	public String getName() {
		if (resource != null) {
			return resource.getName();
		}
		if (path != null) {
			return path.lastSegment();
		}
		if (location != null) {
			String name = location.toString();
			int index = name.lastIndexOf('/');
			if (index >=0 ) {
				name = name.substring(index + 1);
			}
			return name;
		}
		return null;
	}

	public boolean isReadOnly() {
		// You can't write to old revisions e.g. ReadOnly
		return true;
	}

	public BazaarRevision getRevision() {
		return revision;
	}

	private String getResourceCharset() throws CoreException {
		// tries to obtain a description for this file contents
		String charset = null;
		try {
			IContentTypeManager contentTypeManager = Platform.getContentTypeManager();
			charset = contentTypeManager.getDescriptionFor(getContents(), getName(), IContentDescription.ALL).getCharset();
		} catch (Exception e) {
		}
		if(charset == null && resource != null && resource.getType() == IResource.FILE) {
			charset = ((IFile)resource).getCharset();
		} else if(charset == null && resource != null){
			// fallback to parent charset
			return resource.getParent().getDefaultCharset();
		}
		return charset;
	}

	public String getCharset() throws CoreException {
		return getResourceCharset();
	}

	public String getCachePath() {
		return (path != null ? branch.getBranchLocation() + "_" + path.toString() : location.toString()) + "_" + getRevision().toString(); //$NON-NLS-1$
	}
	
	public boolean isContainer() {
		if (resource != null && resource.getType() != IResource.FILE) {
			return true;
		}
		return false;
	}
}
