/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.status;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.core.runtime.QualifiedName;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.ITracingOptions;
import org.vcs.bazaar.eclipse.core.commands.RevnoCommand;
import org.vcs.bazaar.eclipse.core.commands.StatusUpdaterCommand;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.BzrStatus;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * Provides a method to get the status of a resource. <br>
 * It is much more efficient to get the status of a set a resources than only
 * one resource at a time. For that we use a
 *
 * @author Piotr Piastucki
 * @author Guillermo Gonzalez
 */
public class StatusCacheManager {

	/**
	 * Name used for identifying Bazaar synchronization data in
	 * Resource>ResourceInfo#syncInfo storage
	 */
	public static final QualifiedName BZR_SYNC_KEY = new QualifiedName(EclipseBazaarCore.getPluginId(), "bzr-status-sync-key");
	public static final Object FAMILY_CACHE_UPDATE = new Object();
	private final Map<IProject, StatusCacheEntry> cache = new ConcurrentHashMap<IProject, StatusCacheEntry>();
	private boolean logCacheEnabled = true;

	public StatusCacheManager() {
		ResourcesPlugin.getWorkspace().getSynchronizer().add(StatusCacheManager.BZR_SYNC_KEY);
	}

	protected StatusCacheEntry getEntry(IProject project) {
		synchronized (cache) {
			StatusCacheEntry entry = cache.get(project);
			if (entry == null) {
				entry = createCache(project, new NullProgressMonitor());
			}
			return entry;
		}
	}

	private StatusCacheEntry createCache(IProject project, IProgressMonitor monitor) {
		StatusCacheEntry entry = new StatusCacheEntry();
		cache.put(project, entry);
		scheduleRefresh(project, entry, monitor, false, BzrChangeStateEvent.BRANCH_STATE_MODIFY);
		return entry;
	}

	/**
	 * Get the status of the given resource. If the status is not present in
	 * cache, it will be retrieved using the actual updateStrategy. If recursive
	 * strategy is being used also all child resources would be updated,
	 * otherwise only direct children
	 *
	 * @param resource
	 *            whose status is required.
	 * @throws CoreException
	 */
	public BzrStatus getStatus(IResource resource) {
		if (!resource.exists() && !resource.isPhantom()) {
			return null;
		}
		if (!ResourceUtil.shouldHandle(resource, false)) {
			return null;
		}
		StatusCacheEntry entry = getEntry(resource.getProject());
		return entry.get(resource);
	}

	/**
	 * Get the last log of the given resource.
	 *
	 * @param resource
	 *            whose status is required.
	 * @throws CoreException
	 */
	public IBazaarLogMessage getLastLog(IResource resource) {
		StatusCacheEntry entry = getEntry(resource.getProject());
		return entry.getLogCache().get(resource);
	}

	/**
	 * The cached statuses do not provide revision numbers. This method
	 * allow to query for the revision of the resource explicitly.
	 *
	 * @param resource
	 * @return
	 * @throws BazaarException
	 */
	public BazaarRevision getProjectRevision(IResource resource) throws BazaarException {
		if (resource == null)
			return null;
		StatusCacheEntry entry = getEntry(resource.getProject());
		BazaarRevision rev = entry.getLogCache().getProjectRevision();
		if (rev == null) {
			entry.getLock().lock();
			try {
				rev = entry.getLogCache().getProjectRevision();
				if (rev == null) {
					RevnoCommand cmd = new RevnoCommand(resource.getProject());
					cmd.run(new NullProgressMonitor());
					rev = cmd.getRevision();
					entry.getLogCache().setProjectRevision(rev);
				}
			} finally {
				entry.getLock().unlock();
			}
		}
		return rev;
	}
	

	private Map<IProject, List<IResource>> getResourcesByProject(Collection<IResource> resources) {
		Map<IProject, List<IResource>> projects = new HashMap<IProject, List<IResource>>();
		for (IResource resource : resources) {
			List<IResource> resourcesByProject = projects.get(resource.getProject());
			if (resourcesByProject == null) {
				resourcesByProject = new ArrayList<IResource>();
				projects.put(resource.getProject(), resourcesByProject);
			}
			resourcesByProject.add(resource);
		}
		return projects;
	}

	public void scheduleRefresh(Collection<IResource> resources, IProgressMonitor monitor, int decoratorChangeType) throws BazaarException {
		if (!resources.isEmpty()) {
			Map<IProject, List<IResource>> projects = getResourcesByProject(resources);
			for (List<IResource> res : projects.values()) {
				scheduleRefreshPerProject(res, monitor, decoratorChangeType);
			}
		}
	}
	
	/**
	 * Refresh status of the given resources by finding the most top level common parent and refreshing it.
	 * @param resources
	 * @param monitor
	 * @return
	 * @throws BazaarException
	 */
	private void scheduleRefreshPerProject(List<IResource> resources, IProgressMonitor monitor, int decoratorChangeType) throws BazaarException {
		boolean shallow = decoratorChangeType == BzrChangeStateEvent.RESOURCE_MODIFY;
		IResource resource = resources.get(0);
		IContainer commonParent = (IContainer) (resource.getType() == IResource.PROJECT ? resource : resource.getParent());
		if (resources.size() > 1) {
			List<IContainer> parents = new ArrayList<IContainer>();
			for (IContainer parent = commonParent; parent != null && parent.getType() != IResource.ROOT; parent = parent.getParent()) {
				parents.add(parent);
			}
			int index = 0; 
			for (IResource res : resources) {
				IContainer container = (IContainer) (res.getType() == IResource.PROJECT ? res : res.getParent());
				for (IResource parent = container; parent != null && parent.getType() != IResource.ROOT; parent = parent.getParent()) {
					int i = parents.indexOf(parent);
					if (i == -1) {
						shallow = false;
						continue;
					} else if (i > index) {
						index = i;
					}
					break;
				}
			}
			if (index != 0) {
				shallow = false;
			}
			commonParent = parents.get(index);
		}
		StatusCacheEntry entry;
		synchronized (cache) {
			entry = cache.get(commonParent.getProject());
			if (entry == null) {
				createCache(commonParent.getProject(), monitor);
				return;
			}
		}
		scheduleRefresh(commonParent, entry, monitor, shallow, decoratorChangeType);
	}

	private void waitForWorkspaceLock(IProgressMonitor monitor) {
		// Wait for the workspace lock to avoid starting the calculation
		// of an IndexDiff while the workspace changes (e.g. due to a
		// branch switch).
		// The index diff calculation jobs do not lock the workspace
		// during execution to avoid blocking the workspace.
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		try {
			Job.getJobManager().beginRule(root, monitor);
		} catch (OperationCanceledException e) {
			return;
		} finally {
			Job.getJobManager().endRule(root);
		}
	}
	
	/**
	 * Refresh the status of the given resource to the give depth. The depth can
	 * be deeper in case of phantom resources. These have to be traversed to
	 * infinite always ...
	 *
	 * @param resource
	 * @param monitor the progress monitor to use for reporting progress to the user. It is the caller's responsibility
	 *        to call done() on the given monitor. Accepts <code>null</code>, indicating that no progress should be
	 *        reported and that the operation cannot be cancelled
	 * @param recursive
	 * @return array of resources which were refreshed (including all phantoms
	 *         and their children)
	 * @throws BazaarException
	 */
	private void scheduleRefresh(final IContainer resource, final StatusCacheEntry cache, IProgressMonitor monitor, 
			final boolean shallow, final int decoratorChangeType) {
		if (decoratorChangeType == BzrChangeStateEvent.BRANCH_STATE_MODIFY) {
			/* clear cached revision */
			cache.getLogCache().setProjectRevision(null);
		}
		final long updateStart = System.currentTimeMillis();
		Job job = new Job("Refreshing Bazaar status cache") {
			@Override
			public IStatus run(IProgressMonitor monitor) {
				waitForWorkspaceLock(monitor);
				if (monitor.isCanceled()) {
					return Status.CANCEL_STATUS;
				}
				cache.getLock().lock();
				try {
					boolean verbose = decoratorChangeType == BzrChangeStateEvent.BRANCH_STATE_MODIFY && logCacheEnabled;
					if (updateStart < cache.getFullReloadTimestamp()) {
						if (!verbose || updateStart < cache.getLogCache().getFullReloadTimestamp()) {
							return Status.OK_STATUS;
						}
					}
					long timestamp = System.currentTimeMillis();
					// get new status map
					StatusUpdaterCommand cmd = runCommand(resource, shallow, verbose, monitor);
					if (verbose) {
						updateLastLogCache(resource, cache, timestamp, cmd);
					}
					Set<IResource> refreshed = updateStatusCache(resource, cache, shallow, timestamp, cmd);
					// broadcast changes so that decorations will get updated
					BzrChangeStateEvent event = new BzrChangeStateEvent(this, refreshed, decoratorChangeType);
					EclipseBazaarCore.broadcastModificationStateChanges(event);
					EclipseBazaarCore.trace(ITracingOptions.STATUS_CACHE, "Bzr cache updated: ", resource, shallow, (System.currentTimeMillis() - timestamp));
					return Status.OK_STATUS;
				} catch (Exception e) {
					return Status.OK_STATUS;
				} finally {
					cache.getLock().unlock();
				}
			}

			
			@Override
			public boolean belongsTo(Object family) {
				if (family.equals(FAMILY_CACHE_UPDATE))
					return true;
				return super.belongsTo(family);
			}
			
		};
		job.schedule();
	}

	protected void updateLastLogCache(IContainer resource, StatusCacheEntry cache, long timestamp,
			StatusUpdaterCommand cmd) {
		cache.getLogCache().update(cmd.getLogMap());
		if (resource.getType() == IResource.PROJECT) {
			cache.getLogCache().setFullReloadTimestamp(timestamp);
		}
	}

	protected Set<IResource> updateStatusCache(final IContainer resource, final StatusCacheEntry cache, final boolean shallow,
			long timestamp, StatusUpdaterCommand cmd) {
		Set<IResource> refreshed = new HashSet<IResource>();
		Map<IResource, BzrStatus> newMap = cmd.getStatus();
		final HashMap<IResource, BzrStatus> fullMap = new HashMap<IResource, BzrStatus>();
		// in case of partial update keep the existing cache
		if (shallow || resource.getType() != IResource.PROJECT) {
			fullMap.putAll(cache.getStatusMap());
			if (!shallow) {
				// scan folders and remove files that should be refreshed
				Set<IResource> obsolete = new HashSet<IResource>();
				for (IResource res : fullMap.keySet()) {
					if (!res.equals(resource) && resource.getFullPath().isPrefixOf(res.getFullPath())) {
						obsolete.add(res);
					}
				}
				fullMap.keySet().removeAll(obsolete);
				refreshed.addAll(obsolete);
			}
		} else {
			refreshed.addAll(cache.getStatusMap().keySet());
		}
		// combine existing and new status maps together
		fullMap.putAll(newMap);
		refreshed.addAll(newMap.keySet());
		refreshed.add(resource);
		// apply special handling for folders with modified members
		addPseudoModifiedFolders(resource, newMap, fullMap, cache);
		// finally, swap cache content
		cache.update(fullMap);
		if (!shallow && resource.getType() == IResource.PROJECT) {
			cache.setFullReloadTimestamp(timestamp);
		}
		return refreshed;
	}
	/**
	 * Scan new statuses and for each modified resource mark its parents as modified too.
	 * @param resource
	 * @param newMap
	 * @param fullMap
	 * @param cache
	 */
	private void addPseudoModifiedFolders(IContainer resource, 
										  Map<IResource, BzrStatus> newMap, 
										  Map<IResource, BzrStatus> fullMap, 
										  StatusCacheEntry cache) {
		/* scan all modified resources and set parent folder status to modified unless the parent is unknown */
		cache.removePseudoModified(newMap.keySet());
		Map<IResource, BzrStatus>modified = new HashMap<IResource, BzrStatus>();
		for (Entry<IResource, BzrStatus> entry : newMap.entrySet()) {
			if (entry.getValue().isAnyModifiedStatus()) {
				for (IContainer parent = entry.getKey().getParent(); parent.getType() != IResource.ROOT; parent=parent.getParent()) {
					if (!modified.containsKey(parent)) {
						BzrStatus status = fullMap.get(parent);
						if (status == null || status.isUnchanged()) {
							modified.put(parent, BzrStatus.modified());
							cache.setPseudoModified(parent);
						}
					}
				}
			}
		}
		fullMap.putAll(modified);
		for (IContainer parent = resource; parent.getType() != IResource.ROOT; parent=parent.getParent()) {
			if (modified.containsKey(parent)) {
				break;
			}
			if (cache.isPseudoModified(parent)) {
				boolean stillPseudoModified = false;
				try {
					for (IResource child : parent.members(IFolder.EXCLUDE_DERIVED)) {
						if (modified.containsKey(child)) {
							stillPseudoModified = true;
							break;
						}
						BzrStatus status = fullMap.get(child);
						if (status != null && status.isAnyModifiedStatus()) {
							stillPseudoModified = true;
							break;
						}
					}
				} catch (CoreException e) {
				}
				if (stillPseudoModified) {
					break;
				}
				fullMap.put(parent, BzrStatus.unchanged());
				cache.removePseudoModified(parent);
			}
		}
	}

	protected StatusUpdaterCommand runCommand(IResource resource, boolean shallow, boolean verbose, IProgressMonitor monitor) throws BazaarException {
		EclipseBazaarCore.trace(ITracingOptions.STATUS_COMMAND, "StatusUpdater.statusesToUpdate: " + resource.getName()+ "/" + shallow + "/" + verbose);
		StatusUpdaterCommand cmd = new StatusUpdaterCommand(BzrWorkspaceRoot.getBzrResourceFor(resource), shallow, verbose);
		cmd.run(monitor);
		return cmd;
	}
	
	public void purgeCache(IProject project) throws CoreException {
		cache.remove(project);
	}

	public void setLogCacheEnabled(boolean logCacheEnabled) {
		if (logCacheEnabled != this.logCacheEnabled && logCacheEnabled) {
			cache.clear();
		}
		this.logCacheEnabled = logCacheEnabled;
	}
}

