/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.subscriber;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.subscribers.Subscriber;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.IResourceVariantComparator;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.resources.IStorageBazaarRevision;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * @author Guillermo Gonzalez
 *
 */
public class BazaarStatusSuscriber extends Subscriber {

	private static final String NAME = "BazaarStatusSuscriber";

	private final BazaarLocalVariantComparator comparatorObj = new BazaarLocalVariantComparator();

	private static final BazaarStatusSuscriber instance = new BazaarStatusSuscriber();

	public static BazaarStatusSuscriber getInstance(){
		return instance;
	}

	private BazaarStatusSuscriber() {}

	@Override
    public String getName() {
		return NAME;
	}

	@Override
    public boolean isSupervised(IResource resource) throws TeamException {
		// TODO: check if this resource is .bzr
		return shouldHandle(resource);
	}

	@Override
    public IResource[] members(IResource resource) throws TeamException {
		try {
			if (resource.getType() == IResource.FILE)
				return new IResource[0];
			final IContainer container = (IContainer) resource;
			final List<IResource> existingChildren = new ArrayList<IResource>(Arrays.asList(container.members()));
			existingChildren.addAll(Arrays.asList(container.findDeletedMembersWithHistory(IResource.DEPTH_INFINITE, null)));
			return existingChildren.toArray(new IResource[existingChildren.size()]);
		} catch (CoreException e) {
			throw TeamException.asTeamException(e);
		}
	}

	@Override
    public IResource[] roots() {
		return ResourceUtil.findProjects();
	}

	@Override
    public SyncInfo getSyncInfo(IResource resource) throws TeamException {
		// TODO: is ok to pass both as null?
		if (!resource.exists()) {
			return null;
		}
		return getSyncInfo(resource, null, null);
	}

	private SyncInfo getSyncInfo(IResource resource, IStorageBazaarRevision base, IStorageBazaarRevision remote) throws TeamException {
		try {
			if (base == null) {
				base = new IStorageBazaarRevision(resource, 
						EclipseBazaarCore.getDefault().getStatusCacheManager().getProjectRevision(resource));
			}
			final BazaarResourceVariant baseFileHist = new BazaarResourceVariant(base);
			BazaarResourceVariant remoteFileHist = null;
			if (remote != null) {
				remoteFileHist = new BazaarResourceVariant(remote);
			}
			final SyncInfo info = new SyncInfo(resource, baseFileHist, remoteFileHist, comparatorObj);
			info.init();
			return info;
		} catch (CoreException e) {
			throw BazaarException.wrapException(e);
		}
	}

	@Override
    public IResourceVariantComparator getResourceComparator() {
		return comparatorObj;
	}

	@Override
    public void refresh(IResource[] resources, int depth, IProgressMonitor monitor) throws TeamException {
	// TODO: implement me

	}

}
