/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.synchronize;

import static org.vcs.bazaar.eclipse.core.utils.ResourceUtil.shouldHandle;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.core.subscribers.Subscriber;
import org.eclipse.team.core.subscribers.SubscriberChangeEvent;
import org.eclipse.team.core.synchronize.SyncInfo;
import org.eclipse.team.core.variants.IResourceVariantComparator;
import org.eclipse.team.core.variants.ResourceVariantByteStore;
import org.eclipse.team.core.variants.SessionResourceVariantByteStore;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.core.resources.IStorageBazaarRevision;
import org.vcs.bazaar.eclipse.core.subscriber.BazaarResourceVariant;
import org.vcs.bazaar.eclipse.core.synchronize.BazaarSyncCacheEntry.ChangeType;
import org.vcs.bazaar.eclipse.core.synchronize.BazaarSyncCacheEntry.Direction;

/**
 * @author Piotr Piastucki
 */
public class BazaarResourceVariantTreeSubscriber extends Subscriber {

	private static final String NAME = "BazaarResourceVariantTreeSubscriber";
	
	private final BazaarSynchronizeData syncData;
	private final BazaarResourceVariantComparator comparatorObj = new BazaarResourceVariantComparator();
	protected ResourceVariantByteStore remoteSyncStateStore = new SessionResourceVariantByteStore();
	private BazaarSyncCache cache;
	
	private IResource[] roots;
	
	public BazaarResourceVariantTreeSubscriber(BazaarSynchronizeData syncData) {
		this.syncData = syncData;
	}
	

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public boolean isSupervised(IResource resource) throws TeamException {
		if (resource.getType() != IResource.FILE) {
			return false;
		}
		return shouldHandle(resource);
	}
	
	@Override
	public IResource[] members(IResource res) throws TeamException {
		if(res.getType() == IResource.FILE || !syncData.isIncluded(res)) {
			return new IResource[0];
		}
		Set<IResource> members = new HashSet<IResource>();
		try {
			for (IResource member : ((IContainer) res).members()) {
				members.add(member);
			}
			for (IResource resource : syncData.getResources()) {
				if (!res.equals(resource) && res.getFullPath().isPrefixOf(resource.getFullPath())) {
					members.add(resource);
				}
			}
		} catch (CoreException e) {
			EclipseBazaarCore.log(e);
		}
		return members.toArray(new IResource[members.size()]);
	}
	

	@Override
	public IResource[] roots() {
		if (roots == null) {
			roots = syncData.getProjects();
		}
		return roots;
	}

	@Override
	public IResourceVariantComparator getResourceComparator() {
		return comparatorObj;
	}

	@Override
    public void refresh(IResource[] resources, int depth, IProgressMonitor monitor) throws TeamException {
		monitor.beginTask(CoreTexts.BazaarResourceVariantTreeSubscriber_fetchTaskName, 100);
		try {
			cache = BazaarSyncCacheManager.getCache(syncData);
			fireTeamResourceChange(SubscriberChangeEvent.asSyncChangedDeltas(this, cache.getChangedResources()));
		} finally {
			monitor.done();
		}
	}

	public void init(IProgressMonitor monitor) {
		monitor.beginTask(CoreTexts.BazaarResourceVariantTreeSubscriber_fetchTaskName, 100);
		try {
			cache = BazaarSyncCacheManager.getCache(syncData);
		} finally {
			monitor.done();
		}
	}
	
	@Override
	public SyncInfo getSyncInfo(IResource resource) throws TeamException {
		if (cache == null) {
			return null;
		}
		BazaarSyncCacheEntry entry = cache.getEntry(resource);
		if (entry == null) {
			return null;
		}
		IBzrBranch branch = BranchFactory.getBranch(syncData.getRemoteLocation());
		BazaarResourceVariant base = null;
		BazaarResourceVariant remote = null;
		if (entry.getChangeType() != ChangeType.ADD) {
			base = new BazaarResourceVariant(new IStorageBazaarRevision(resource, entry.getBaseRevision()));
		}
		if (!((entry.getChangeType() == ChangeType.DELETE && entry.getDirection() == Direction.INCOMING) ||
			  (entry.getChangeType() == ChangeType.ADD && entry.getDirection() == Direction.OUTGOING))) {
			remote = new BazaarResourceVariant(new IStorageBazaarRevision(resource, branch, entry.getPath(), entry.getRemoteRevision()));
		}
		BazaarSyncInfo info = new BazaarSyncInfo(entry, resource, base, remote, comparatorObj);
		info.init();
		return info;
	}
	
}
