/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.core.utils;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.RepositoryProvider;
import org.vcs.bazaar.client.IBazaarStatus;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.internal.core.model.ProjectBranch;

/**
 * <p>
 * provides static helping functionality for working with resources.
 * </p>
 *
 * @author Guillermo Gonzalez
 * @basedOn ResourceUtil from eclipsedarcs by Leif Frenzel
 *
 */
public class ResourceUtil {

	public static boolean isUnderBazaarScope(final IProject project) {
		IPath projectLocation = project.getLocation();
		return findRootLocation(projectLocation.toFile()) != null;
	}

	/**
	 * Return all open projects that use bzr provider
     * @return List of projects
	 */
	public static IProject[] findProjects() {
		final List<IProject> result = new ArrayList<IProject>();
		final IProject[] projects = ResourcesPlugin.getWorkspace().getRoot().getProjects();
		for (IProject project : projects) {
			if (project.isAccessible()) {
				final RepositoryProvider provider = RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID());
				if (provider != null) {
					result.add(project);
				}
			}
		}
		return result.toArray(new IProject[result.size()]);
	}
	
	/**
	 * find the root location of the repository that contains the given project;
	 * if the project itself is a repository (contains the <code>.bzr</code>
	 * metafolder), the path to the project is returned. May be null, if no
	 * repository is found.
	 */
	public static IPath findRootLocation(final IProject project) {
		File projectFile = project.getLocation().toFile();
		File location = findRootLocation(projectFile);
		return location != null?new Path(location.toString()):null;
	}

	/**
	 * FIXME: support for multiple project per branch
	 */
	public static void setMetaFolderTeamPrivate(final IProject project, final boolean teamPrivate) {
		try {
			IFolder folder = project.getFolder(IBzrBranch.DIR_META_FOLDER);
			if (folder.exists()) {
				folder.setTeamPrivateMember(teamPrivate);
			}
		} catch (CoreException e) {
			EclipseBazaarCore.getDefault().getLog().log(e.getStatus());
		}
	}


	/**
	 * returns the path to the repository in which the specified resource is
	 * shared, or null, if the resource is not in a shared project.
	 */
	public static IPath getRepositoryPath(final IResource resource) {
		IPath result = null;
		if (resource == null)
			return null;
		IProject project = resource.getProject();
		if (RepositoryProvider.isShared(project)) {
			RepositoryProvider prov = RepositoryProvider.getProvider(project);
			if (prov instanceof BazaarTeamProvider) {
				BzrWorkspaceRoot bzrWorkspaceRoot = ((BazaarTeamProvider) prov).getBzrWorkspaceRoot();
				if(bzrWorkspaceRoot != null)
					result = bzrWorkspaceRoot.getBranch().getPath();
				else {
					result = findRootLocation(project);
				}
			}
		} else {
			result = findRootLocation(project);
		}
		return result;
	}

	/**
	 * <p>
	 * returns true, if the project directory is a Bazaar branch, i.e.
	 * contains the <code>.bzr/</code> directory.
	 * </p>
	 *
	 * <p>
	 * Note: this returns false if the project is not already shared.
	 * </p>
	 */
	public static boolean isRepoDirInProject(final IProject project) {
		IPath repoPath = getRepositoryPath(project);
		return project.getLocation().equals(repoPath);
	}

	/**
	 * Set dirty properties in resource (or it's project, branch, etc)
	 *
	 * @param lastResource
	 * @param resource
	 * @return the resource
	 */
	public static void markAsDirty(IResource resource) {
		IProject project = resource.getProject();
		IBzrBranch branch = BzrWorkspaceRoot.getBzrWorkspaceRootFor(project).getBranch();
		if (branch instanceof ProjectBranch) {
			((ProjectBranch)branch).setDirty(true);
		}
	}

	/**
	 * @see IWorkspaceRoot#getFileForLocation(IPath)
	 *
	 * @param absolutePath
	 * @return {@link IFile}
	 */
	public static IFile getIFileForLocation(String absolutePath) {
		return ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(new Path(absolutePath));
	}

	/**
	 * @see IWorkspaceRoot#getContainerForLocation(IPath)
	 *
	 * @param absolutePath
	 * @return {@link IContainer}
	 */
	public static IContainer getIContainerForLocation(String absolutePath) {
		return ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(absolutePath));
	}


	public static IResource getIFile(IBazaarStatus aStatus) {
		IResource resource = getIFileForLocation(aStatus.getAbsolutePath());
		if(resource == null) {
			resource = ResourcesPlugin.getWorkspace().getRoot().getFile(getRelativePathToWorkspaceRoot(aStatus));
		}
		return resource;
	}

	public static IResource getIContainer(IBazaarStatus aStatus) {
		IResource resource = getIContainerForLocation(aStatus.getAbsolutePath());
		if(resource == null) {
			resource = ResourcesPlugin.getWorkspace().getRoot().getFolder(getRelativePathToWorkspaceRoot(aStatus));
		}
		return resource;
	}

	public static IResource getIResource(IBazaarStatus aStatus) {
		return aStatus.isDirectory() ? getIContainer(aStatus) : getIFile(aStatus);
	}

	public static IResource getIResource(IPath path, boolean directory) {
		return directory ? ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(path) :
			ResourcesPlugin.getWorkspace().getRoot().getFileForLocation(path);
	}
	
	private static IPath getRelativePathToWorkspaceRoot(IBazaarStatus status) {
		IWorkspaceRoot root = ResourcesPlugin.getWorkspace().getRoot();
		Path statusPath = new Path(status.getAbsolutePath());
		IPath relativePath = statusPath.makeRelativeTo(root.getLocation());
		return relativePath;
	}

	public static boolean shouldHandle(IResource resource) {
		boolean shouldHandle = !(resource.isDerived(IResource.CHECK_ANCESTORS) || resource.isLinked(IResource.CHECK_ANCESTORS));
		IProject project = null;
		if(resource.getType() == IResource.PROJECT) {
			project = (IProject)resource;
		} else {
			project = resource.getProject();
		}
		return shouldHandle &&  project != null && RepositoryProvider.getProvider(project, EclipseBazaarCore.getProviderID()) != null;
	}

	public static boolean shouldHandle(IResource resource, boolean checkProvider) {
		if (checkProvider) {
			return shouldHandle(resource);
		}
		return !resource.isLinked(IResource.CHECK_ANCESTORS) && !resource.isDerived(IResource.CHECK_ANCESTORS);
	}
	
	// helper methods
	// ////////////////

	private static File findRootLocation(final File file) {
		File metaFolder = new File(file, IBzrBranch.DIR_META_FOLDER);
		File result = null;
		if (metaFolder.exists() && metaFolder.isDirectory() && new File(metaFolder, "branch").exists()) {
			result = metaFolder.getParentFile();
		} else {
			File parent = file.getParentFile();
			// reached the root, no '.bzr' in sight, else recurse until reach read-only parent
			if(parent != null && parent.canWrite())
				result = (parent == null) ? null : findRootLocation(parent);
			else
				result = null;
		}
		return result;
	}

    /**
     * Search the given location for Eclipse projects.
     * 
     * This is done after the location has been fetched from Bazaar. The
     * function performs the search recursively, stopping when it finds a
     * project file.
     * 
     * @return List of File objects, the directories containing Bazaar projects.
     */
    public static List<File> findProjects(File location) {
        List<File> ps = new LinkedList<File>();
        File dotProj = new File(location, ".project");
        if (dotProj.exists()) {
            ps.add(location);
            return ps;
        }
        // Search directories
        File[] dirs = location.listFiles(new FileFilter() {
            public boolean accept(File pathname) {
                return pathname.isDirectory() && !pathname.getName().equals(".bzr");
            }
        });

        for (File dir : dirs) {
            ps.addAll(findProjects(dir));
        }

        return ps;
    }

	/**
	 * Remove files and folders recursively. 
	 * Used internally for cleaning up on errors and project deletion.
	 */
	public static void deleteRecursively(File file) {
		if (file.isDirectory()) {
			for (File child : file.listFiles()) {
				deleteRecursively(child);
			}
		}
		file.delete();
	}

	/**
	 * Return true if the given resource is either .bzr directory or located inside .bzr directory
	 * @param resource
	 * @return
	 */
	public static boolean isInsideMetaFolder(IResource resource) {
		IProject project = resource.getProject();
		return project.getFolder(new Path(".bzr")).getFullPath().isPrefixOf(resource.getFullPath());
	}

}
