/**
 * LICENCSE + COPYRIGHT
 * Copyright (c) 2004-2006 by Leif Frenzel - see http://leiffrenzel.de
 */
package org.vcs.bazaar.eclipse.internal.core.model;

import java.io.File;
import java.net.URI;
import java.util.HashMap;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.vcs.bazaar.client.IBazaarClient;
import org.vcs.bazaar.client.IBazaarInfo;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrMissingRevisions;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * <p>
 * Implementation of a local branch, which corresponds to a folder on the local disk (.bzr)
 * </p>
 *
 * @author Guillermo Gonzalez
 * @basedOn LocalDarcsRL by Leif Frenzel
 *
 */
public class LocalBranch extends BzrBranch implements IBzrLocalBranch {

	private static final long serialVersionUID = 1L;

	public LocalBranch(BranchLocation location) {
		super(location);
		// make sure the URI scheme is "file"
		assert ("file".equals(location.getScheme()));
	}


	public boolean isLocal() {
		return true;
	}

	public IProject getProject() {
		return null;
	}

	@Override
    public URI getLocation() {
		return location.getUri();
	}

	public File getFile() {
		return getPath().toFile();
	}

	@Override
	public IPath getPath() {
		return new Path(location.getRaw());
	}

	public String getNick() {
		try {
			IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			client.setWorkDir(getFile());
			return client.nick(null);
		} catch (BazaarException e) {
			throw BazaarException.makeUnChecked(e);
		} catch (BazaarClientException e) {
			throw BazaarClientException.makeUnChecked(e);
		}
	}

	@Override
    public IBzrBranch getParent() {
		IBazaarInfo info = null;
		if(info == null) {
			info = getInfo();
		}
		if(info == null || info.getRelatedBranches() == null || info.getRelatedBranches().getParentBranch() == null)
			return null;
		return BranchFactory.findOrCreateFor(info.getRelatedBranches().getParentBranch());
	}

	public IBzrBranch getPushBranch() {
		IBazaarInfo info = null;
		if(info == null) {
			info = getInfo();
		}
		if(info == null || info.getRelatedBranches() == null)
			return null;
		if(info.getRelatedBranches().getPushBranch() != null && !"".equals(info.getRelatedBranches().getPushBranch().trim())) { 
			return BranchFactory.findOrCreateFor(info.getRelatedBranches().getPushBranch());
		}
		return null;
	}

	public IBazaarInfo getInfo() {
		try {
			IBazaarClient client = EclipseBazaarCore.getDefault().createClient();
			client.setWorkDir(getFile());
			return client.info(getFile());
		} catch (BazaarException e) {
			throw BazaarException.makeUnChecked(e);
		} catch (BazaarClientException e) {
			throw BazaarClientException.makeUnChecked(e);
		}
	}

	/**
	 * FIXME: support for multiple project per branch
	 */
	@Override
	public boolean isBound() {
		IBazaarInfo info = getInfo();
		if (info == null) {
			return false;
		}
		IBazaarInfo.Locations locations = info.getLocations();
		return !(locations.getCheckoutOfBranch() == null && locations.getCheckoutRoot() == null
				&& locations.getLightCheckoutRoot() == null && locations.getBoundToBranch() == null
				&& locations.getRepositoryCheckoutRoot() == null);
	}

	public IBzrMissingRevisions getMissing(IBzrBranch otherBranch) throws BazaarException {
		return new BzrMissingRevisions(new HashMap<String, List<IBazaarLogMessage>>());
	}


	public BranchLocation getPushLocation() {
		IBzrBranch pushBranch = getPushBranch();
		if (pushBranch != null) {
			return pushBranch.getBranchLocation();
		}
		return null;
	}

}
