/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.atomic.AtomicBoolean;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ProgressMonitorDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.custom.BusyIndicator;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.internal.TeamAction;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * @author Guillermo Gonzalez
 *
 * <p>
 * Base class for all Bazaar actions.
 * </p>
 * <p>
 * Most of this class is based on ideas taken from SVNAction from subclipse
 * </p>
 */
public abstract class BzrAction extends TeamAction {

	protected IAction action = null;

	/**
	 * Common run method for all Bazaar actions.
	 */
	@Override
    final public void run(IAction action) {
		try {
			if(!needEnablementCheck() && !isEnabled()) {
				handle(new BazaarException(getStatusNotEnabled(action)), UITexts.BzrAction_errorTitle, "The selected action cannot be executed");
			} else {
				if (!beginExecution(action)) return;
				execute(action);
			}
		} catch (Exception e) {
			handle(e);
		}
	}

	private IStatus getStatusNotEnabled(final IAction action) {
		return new Status(IStatus.ERROR, EclipseBazaarUI.getPluginId(), action.getText() + " is not enabled in this context", null);
	}

	/**
	 * Actions must override to do their work.
	 */
	abstract protected void execute(IAction action) throws Exception;


	@Override
	protected void setActionEnablement(IAction action) {
		if(needEnablementCheck()) {
			super.setActionEnablement(action);
		} else {
			action.setEnabled(true);
		}
	}

	/**
	 * @return
	 */
	private Boolean needEnablementCheck() {
		IPreferenceStore prefStore = EclipseBazaarUI.getDefault().getPreferenceStore();
		Boolean checkEnablement = prefStore.getBoolean(BazaarPreferenceConstants.CHECK_ACTION_AVAILABILITY);
		return checkEnablement==null?true:checkEnablement;
	}

	/**
	 * Convenience method for running an operation with the appropriate
	 * progress. Any exceptions are propogated so they can be handled by the
	 * <code>BzrAction#run(IAction)</code> error handling code.
	 *
	 * @param runnable
	 *            the runnable which executes the operation
	 * @param cancelable
	 *            indicate if a progress monitor should be cancelable
	 * @param progressKind
	 *            one of PROGRESS_BUSYCURSOR or PROGRESS_DIALOG
	 */
	final protected void run(final IRunnableWithProgress runnable, boolean cancelable, int progressKind) throws InvocationTargetException,
			InterruptedException {
		final Exception[] exceptions = new Exception[] { null };

		// Ensure that no repository view refresh happens until after the action
		final IRunnableWithProgress innerRunnable = new IRunnableWithProgress() {
			public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
				runnable.run(monitor);
			}
		};

		switch (progressKind) {
		case PROGRESS_BUSYCURSOR:
			final Display display = EclipseBazaarUI.getStandardDisplay();
			BusyIndicator.showWhile(display, new Runnable() {
				public void run() {
					try {
						innerRunnable.run(new NullProgressMonitor());
					} catch (InvocationTargetException e) {
						exceptions[0] = e;
					} catch (InterruptedException e) {
						exceptions[0] = e;
					}
				}
			});
			break;
		case PROGRESS_DIALOG:
		default:
			new ProgressMonitorDialog(getShell()).run(true, cancelable,/*
																		 * cancelable,
																		 * true,
																		 */innerRunnable);
			break;
		}
		if (exceptions[0] != null) {
			if (exceptions[0] instanceof InvocationTargetException) {
                throw (InvocationTargetException) exceptions[0];
            } else {
                throw (InterruptedException) exceptions[0];
            }
		}
	}

	/**
	 * <p>
	 * Return only the specified type of resources from the selection.
	 * </p>
	 *
	 * @param type
	 *            the type of resource to filter by (IResource.FILE,
	 *            IResource.FOLDER, etc)
	 * @return IResource[]
	 */
	protected IResource[] getSelectedResources(int type) {
		Set<IResource> resources = new HashSet<IResource>();
		for (IResource resource : getSelectedResources()) {
			if (resource.getType() == type) {
				resources.add(resource);
			}
		}
		return resources.toArray(new IResource[0]);
	}

	protected IProject[] getEnclosingProjectsForSelectedResources() {
		Set<IProject> list = new HashSet<IProject>();
		for (IProject project : getSelectedProjects()) {
			list.add(project);
		}
		for (IResource resource : getSelectedResources()) {
			if (resource.getType() != IResource.PROJECT) {
				list.add(resource.getProject());
			}
		}
		return list.toArray(new IProject[0]);
	}

	protected static final Map<IBzrBranch, List<IResource>> getFilesByBranchMap(IResource[] resources) {
		Map<IBzrBranch, List<IResource>> map = new HashMap<IBzrBranch, List<IResource>>();
		for (IResource resource : resources) {
			IBzrBranch branch = BranchFactory.findOrCreateFor(resource.getProject());
			List<IResource> list = map.get(branch);
			if (list == null) {
				list = new ArrayList<IResource>();
				map.put(branch, list);
			}
			list.add(resource);
		}
		return map;
	}
	
	/**
	 * Helper to simplify the call to {@link #handle(Exception, String, String)}
	 * form {@link #run(IAction)}
	 *
	 * @param exception
	 */
	protected void handle(Exception exception) {
		if (exception != null) {
			if(!(exception instanceof InvocationTargetException)) {
                EclipseBazaarUI.log(exception.getMessage(), exception);
            }
			handle(exception, UITexts.BzrAction_errorTitle, exception.getLocalizedMessage());
		}
	}

	/*
	 * @see org.eclipse.ui.actions.ActionDelegate#init(org.eclipse.jface.action.IAction)
	 */
	@Override
    public void init(IAction action) {
		super.init(action);
		this.action = action;
	}

	/**
	 * We can use this method to dispose of any system resources we previously
	 * allocated.
	 *
	 * @see IWorkbenchWindowActionDelegate#dispose
	 */
	@Override
    public void dispose() {
		super.dispose();
		this.action = null;
	}

	protected IAction getAction() {
		return action;
	}

	/**
	 * @return true if all selected (enclosing) projects belong to the same branch
	 */
	protected boolean isSameBranchInEnclosingProjects() {
		IProject[] projects = getEnclosingProjectsForSelectedResources();
		IBzrBranch branch = null;
		boolean sameBranch = false;
		for (IProject project : projects) {
			BzrWorkspaceRoot root = BzrWorkspaceRoot.getBzrWorkspaceRootFor(project);
			if(root == null) {
				continue;
			}
			if(branch == null) {
				branch = root.getBranch();
				continue;
			}
			sameBranch = branch.equals(root.getBranch());
			if(!sameBranch) {
                return sameBranch;
            }
		}
		return sameBranch;
	}

	protected boolean beginExecution(IAction action) {
		if(isAnyEditorDirty()) {
			if(!saveAllEditors()) {
				return false;
			}
		}
		return true;
	}

	protected boolean needsToSaveDirtyEditors() {
		return false;
	}

	private boolean isAnyEditorDirty() {
		if (needsToSaveDirtyEditors()) {
			IResource[] selectedResources = getSelectedResources();
			if (selectedResources != null && selectedResources.length > 0) {
				IEditorReference[] editorReferences = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage().getEditorReferences();
				for (IEditorReference editorReference : editorReferences) {
					if (editorReference.isDirty()) {
						try {
							IEditorInput editorInput = editorReference.getEditorInput();
							if (editorInput instanceof IFileEditorInput) {
								IFile file = ((IFileEditorInput)editorInput).getFile();
								if (isSelectedResource(file, selectedResources)) {
									return true;
								}
							}
						} catch (PartInitException e) {}
					}
				}
			}
		}
		return false;
	}

	private boolean isSelectedResource(IFile file, IResource[] selectedResources) {
		if (file != null) {
			IResource parent = file;
			while (parent != null) {
				for (IResource selectedResource : selectedResources) {
					if (selectedResource.equals(parent)) {
						return true;
					}
				}
				parent = parent.getParent();
			}
		}
		return false;
	}

	private boolean saveAllEditors() {
		final AtomicBoolean okToContinue = new AtomicBoolean(true);
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				okToContinue.set(PlatformUI.getWorkbench().saveAllEditors(true));
			}
		});
		return okToContinue.get();
	}
}
