/**
 * org.vcs.bazaar.eclipse (c) Vectrace Jan 31, 2006
 * Created by zingo
 */
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.IJobStatus;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.CommitCommand;
import org.vcs.bazaar.eclipse.core.commands.StatusCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.BzrJobStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.dialogs.CommitDialog;
import org.vcs.bazaar.eclipse.ui.operations.AddOperation;
import org.vcs.bazaar.eclipse.ui.operations.PushOperation;

/**
 * @author zingo
 * @modifiedBy Guillermo Gonzalez
 *
 * TODO: needs revision & test (maybe split into Operation and Action)
 */
public class CommitAction extends WorkbenchAction {

	/**
	 * The action has been activated. The argument of the method represents the
	 * 'real' action sitting in the workbench UI.
	 *
	 * @throws BazaarException
	 * @throws InterruptedException
	 * @throws InvocationTargetException
	 *
	 * @see IWorkbenchWindowActionDelegate#run
	 */

	@Override
    public void execute(final IAction action) throws BazaarException, InvocationTargetException, InterruptedException {
		final CommitDialog commitDialog = new CommitDialog(getShell());
		UIJob job = new UIJob(this.getShell().getDisplay(), "Getting commitable resources...") {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				monitor.beginTask("Getting commitable resources..", 100);
				try {
					CommitableResources resourceStauses = createCommitableResources(monitor, getEnclosingProjectsForSelectedResources());
					CommitableResources selectedResource = createCommitableResources(monitor, getSelectedResources());
					monitor.worked(100);
					if (resourceStauses.getAllStatuses().isEmpty()) {
						return BzrJobStatus.INFO(this, UITexts.CommitAction_nothingToCommit);
					}
					for (IBzrResourceStatus status : resourceStauses.getAllStatuses()) {
						if (status.isHasConflicts()) {
							return BzrJobStatus.INFO(this, UITexts.CommitAction_resolveConflicts);
						}
					}
					List<CommitableResources> results = new ArrayList<CommitableResources>(2);
					results.add(resourceStauses);
					results.add(selectedResource);
					return new BzrJobStatus<List<CommitableResources>>(IJobStatus.OK, this, results, "Commitable resources calculated");
				} catch (Exception e) {
					return UIJob.errorStatus(e);
				} finally {
					monitor.done();
				}
			}
		};
		job.setUser(true);
		job.setPriority(UIJob.LONG);
		job.addJobChangeListener(new JobChangeAdapter() {
            @SuppressWarnings("unchecked")
            @Override
            public void done(IJobChangeEvent event) {
				IStatus status = event.getResult();
				if(status instanceof BzrJobStatus) {
					if(status.getSeverity() == IStatus.INFO) {
                        MessageDialog.openInformation(getParentShell(), UITexts.Dialog_informationTitle, status.getMessage());
                    } else if(status.getSeverity() == IStatus.OK) {
						List<CommitableResources> result = ((BzrJobStatus<List<CommitableResources>>)status).getResult();
						int exit = commitDialog.open(result.get(0), result.get(1), true);
						if (exit == Window.OK && commitDialog.getCommitMessage() != null) {
							try {
								handleSelected(result.get(0), commitDialog);
							} catch (Exception e) {
								CommitAction.this.handle(e);
							}
						}
					}
				}
				event.getJob().removeJobChangeListener(this);
			}
		});
		job.schedule();
	}

	/**
	 * Given an IResource[] returns the list of commitable resources as a list of IBzrResourceStatus.
	 *
	 * @param monitor a IProgressMonitor
	 * @param resources an array of resources to check for inclusion in the commitable list
	 * @return List<IBzrResourceStatus>
	 * @throws BazaarException
	 */
	protected static List<IBzrResourceStatus> getCommitableResources(IProgressMonitor monitor, IResource[] resources) throws BazaarException {
		return createCommitableResources(monitor, resources).getAllStatuses();
	}

	static CommitableResources createCommitableResources(IProgressMonitor monitor, IResource[] resources) throws BazaarException {
		CommitableResources commitableResource = new CommitableResources();
		IBzrLocalResource[] bzrResources = null;
		bzrResources = BzrWorkspaceRoot.getBzrResourcesFor(resources);
		StatusCommand cmd = new StatusCommand(bzrResources, true);
		cmd.run(monitor);
		IBzrResourceStatus[] result = cmd.getStatus();
		if (result.length > 0) {
			commitableResource.add(Arrays.asList(result));
		}
		commitableResource.addPendingMerge(cmd.getPendingMerges());
		return commitableResource;
	}

	private void handleSelected(final CommitableResources commitableResources, CommitDialog commitDialog) throws BazaarException, InvocationTargetException, InterruptedException {

		final List<IResource> filesToAdd = getUnknownResources(commitDialog.getSelectedFiles()); 
		final List<IBzrLocalResource> filesToCommit = getResources(commitDialog.getSelectedFiles());
		final String commitText = commitDialog.getCommitMessage();
		final boolean pushAfterCommit = commitDialog.isPushAfterCommit();
		final List<Option> options = commitDialog.getOptions();
		final boolean fullRefresh = !getUnknownOrAddedResources(commitDialog.getSelectedFiles()).isEmpty();
		Job job = new Job("Running commit") {
			@Override
			public IStatus run(IProgressMonitor monitor) {
				try {
					int work = (filesToAdd != null && filesToAdd.size() > 0 ? 1000 : 0) +
							   (filesToCommit != null && filesToCommit.size() > 0 ? 1000 : 0) +
							   (pushAfterCommit && filesToCommit != null && filesToCommit.size() > 0 ? 1000 : 0);
					monitor.beginTask(CoreTexts.CommitCommand_commit, work); //$NON-NLS-1$
					// first we execute a 'bzr add'
					if (!filesToAdd.isEmpty()) {
						new AddOperation(filesToAdd.toArray(new IResource[0]), true).run(EclipseBazaarCore.subProgressMonitorFor(monitor));
					}
					// now the commit
					if (filesToCommit != null && filesToCommit.size() > 0) {
						CommitCommand commitCmd = null;
						if(commitableResources.getAllPendingMerges().size() > 0) {
							// commit all modified files (including unknowns)
							// currently we do this, by committing without passing any file to the commit cmd
							commitCmd = new CommitCommand(filesToCommit, commitText, options, true, true);
						} else {
							commitCmd = new CommitCommand(filesToCommit, commitText, options, false, fullRefresh);
						}
						commitCmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
					}
					// handle push after commit
					if(pushAfterCommit && filesToCommit != null && filesToCommit.size() > 0) {
						final IBzrLocalBranch branch = BzrWorkspaceRoot.getBranchFor(filesToCommit.get(0).getIResource().getProject());
						BranchLocation remoteLocation = branch.getPushLocation();
						if (remoteLocation != null) {
							new PushOperation(branch, remoteLocation, false, false, false).run(EclipseBazaarCore.subProgressMonitorFor(monitor));
						} else {
							EclipseBazaarUI.getDefault().getWorkbench().getDisplay().syncExec(new Runnable() {
								public void run() {
									PushAction action = new PushAction();
									try {
										action.execute(branch);
									} catch (BazaarException e) {
									} catch (InvocationTargetException e) {
									} catch (InterruptedException e) {
									}
								}
							});
						}
					}
				} catch (CoreException e) {
					EclipseBazaarUI.log(e.getMessage(), e);
					return e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(e.getMessage(), e);
					return BzrJobStatus.ERROR(this, e, "Error while executing commit");
				} finally {
					monitor.done();
				}
				return BzrJobStatus.OK(this, filesToCommit,"New revision committed");
			}
		};
		job.setUser(true);
		job.setPriority(UIJob.LONG);
		job.schedule();
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}

	@Override
	protected boolean needsToSaveDirtyEditors() {
		return true;
	}

	public static class CommitableResources {

		List<IBazaarLogMessage> pendingMerges;
		List<IBzrResourceStatus> statuses;

		public CommitableResources() {
			pendingMerges = new ArrayList<IBazaarLogMessage>();
			statuses = new ArrayList<IBzrResourceStatus>();
		}

		public void addPendingMerge(List<IBazaarLogMessage> pendingMerges) {
			this.pendingMerges.addAll(pendingMerges);
		}

		public void add(List<IBzrResourceStatus> resourceStatus) {
			statuses.addAll(resourceStatus);
		}

		public List<IBzrResourceStatus> getAllStatuses() {
			return statuses;
		}

		public List<IBazaarLogMessage> getAllPendingMerges() {
			return pendingMerges;
		}
	}
}
