/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrMissingRevisions;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.core.utils.BzrJobStatus;
import org.vcs.bazaar.eclipse.core.utils.BzrTeamStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.dialogs.LocationInputDialog;
import org.vcs.bazaar.eclipse.ui.dialogs.LocationInputDialog.LocationDialogInfo;
import org.vcs.bazaar.eclipse.ui.views.MissingView;

/**
 * @author Guillermo Gonzalez
 * TODO: Test (split into Operation and Action?)
 */
public class MissingAction extends WorkbenchAction {


	/**
	 * The action has been activated. The argument of the method represents the
	 * 'real' action sitting in the workbench UI.
	 *
	 * @see IWorkbenchWindowActionDelegate#run
	 */
	@Override
    public void execute(IAction action) {
		// XXX: get only the first element in the selection
		IResource resource = getSelectedResources()[0];
		IBzrBranch otherLocation = null;
		try {
			otherLocation = getRemoteLocation(resource.getProject());
		} catch (BazaarException e) {
			MessageDialog.openError(getParentShell(), "An unexpected error occurred", "An Error ocurred when retrieving branch locations");
		}
		if(otherLocation == null) {
            return;
        }
		final IBzrLocalBranch localBranch = (IBzrLocalBranch)BzrWorkspaceRoot.getBzrWorkspaceRootFor(resource.getProject()).getBranch();
		try {
			IWorkbenchPage activePage = EclipseBazaarUI.getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage();
			// Show View
			IViewPart viewPart = null;
			viewPart = activePage.showView(MissingView.VIEW_ID);
			final MissingView missingView = (MissingView) viewPart;
			// Send out put to the view
			final IBzrBranch finalOtherLocation = otherLocation;
			Job fetchMissingJob = new Job("Calculating missing/extra revisions...") {
				@Override
                public IStatus run(IProgressMonitor monitor) {
					IBzrMissingRevisions missingRevs = null;
					try {
						missingRevs = localBranch.getMissing(finalOtherLocation);
					} catch (BazaarException e) {
						EclipseBazaarUI.log(e);
						return new BzrTeamStatus(BzrTeamStatus.ERROR, "Error while calculating revisions", e);
					}
					return BzrJobStatus.OK(this, missingRevs, "ok");
				}
			};
			fetchMissingJob.addJobChangeListener(new JobChangeAdapter() {
				@Override
				public void done(IJobChangeEvent event) {
					final IStatus status = event.getResult();
                    if (status.isOK() && status instanceof BzrJobStatus<?>) {
						MissingAction.this.getShell().getDisplay().asyncExec(new Runnable() {
							public void run() {
                                missingView.setInput(((BzrJobStatus<?>) status).getResult());
							}
						});
					}
					super.done(event);
				}
			});
			fetchMissingJob.setPriority(UIJob.LONG);
			fetchMissingJob.setUser(true);
			fetchMissingJob.schedule();
		} catch (PartInitException e) {
			EclipseBazaarUI.getDefault().getLog().log(e.getStatus());
			handle(e);
		}
	}

	private IBzrBranch getRemoteLocation(IProject project) throws BazaarException {
		List<IBzrBranch> locations = new ArrayList<IBzrBranch>();
		IBzrBranch branch = BzrWorkspaceRoot.getBranchFor(project);
		IBzrBranch parent = null;
		if(branch.isLocal()) {
			parent = ((IBzrLocalBranch)branch).getParent();
			if(parent != null) {
                locations.add(parent);
            }
		}
		LocationDialogInfo dialogInfo = new LocationDialogInfo();
		LocationInputDialog dialog = LocationInputDialog.missingLocationDialog(getShell(), locations, dialogInfo);
		dialog.setBlockOnOpen(true);
		if (dialog.open() == LocationInputDialog.OK) {
			if("".equals(dialogInfo.getLocation()) && parent != null) {
				return parent;
			} else {
                return BranchFactory.findOrCreateFor(dialogInfo.getLocation());
			}
		}
		return null;
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}

	@Override
	protected boolean isEnabledForUnmanagedResources() {
		return false;
	}

	@Override
	protected boolean isEnabledForAddedResources() {
		return false;
	}

	@Override
	protected boolean isEnabledByType(IResource resource) {
		return resource.getType() == IResource.PROJECT;
	}
}
