/*******************************************************************************
* Copyright (c) 2007-2011 Guillermo Gonzalez
* Copyright (c) 2013 Piotr Piastucki
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IStorage;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Platform;
import org.eclipse.core.runtime.PlatformObject;
import org.eclipse.core.runtime.content.IContentType;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorRegistry;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.IStorageEditorInput;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.actions.BaseSelectionListenerAction;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.model.IWorkbenchAdapter;
import org.eclipse.ui.progress.IProgressService;

public class OpenBazaarRevisionAction extends BaseSelectionListenerAction {

	public class BazaarRevisionEditorInput extends PlatformObject implements IWorkbenchAdapter, IStorageEditorInput {

		private IFileRevision fileRevision;
		private IStorage storage;

		public BazaarRevisionEditorInput(IFileRevision revision) {
			this.fileRevision = revision;
			try {
				this.storage = revision.getStorage(new NullProgressMonitor());
			} catch (CoreException e) {
			}
		}

		public Object[] getChildren(Object o) {
			return new Object[0];
		}

		public ImageDescriptor getImageDescriptor(Object object) {
			return null;
		}

		public String getLabel(Object o) {
			return getName();
		}

		public Object getParent(Object o) {
			return null;
		}

		public IStorage getStorage() throws CoreException {
			return storage;
		}

		public boolean exists() {
			return true;
		}

		public ImageDescriptor getImageDescriptor() {
			return null;
		}

		public String getName() {
			return new Path(fileRevision.getName()).lastSegment() + " " + fileRevision.getContentIdentifier(); //$NON-NLS-1$
		}

		public IPersistableElement getPersistable() {
			return null; // Can to save editor changes
		}

		public String getToolTipText() {
			return fileRevision.getName() + " " + fileRevision.getContentIdentifier();
		}

        @SuppressWarnings("unchecked")
        @Override
        public Object getAdapter(Class adapter) {
			if (adapter == IWorkbenchAdapter.class) {
				return this;
			}
			if (adapter == IFileRevision.class) {
				return fileRevision;
			} else if (adapter == IStorage.class) {
				return storage;
			}
			return super.getAdapter(adapter);
		}

		@Override
		public int hashCode() {
			return fileRevision.hashCode();
		}

		@Override
		public boolean equals(Object obj) {
			if (obj == this) {
				return true;
			}
			if (obj instanceof BazaarRevisionEditorInput) {
				return ((BazaarRevisionEditorInput)obj).fileRevision.equals(fileRevision);
			}
			return false;
		}
	}

	protected IStructuredSelection selection;
	private final IWorkbenchSite site;

	public OpenBazaarRevisionAction(String text, IWorkbenchSite site) {
		super(text);
		this.site = site;
	}

	@Override
    public void run() {
		IStructuredSelection structSel = selection;

		Object[] objArray = structSel.toArray();

		for (Object tempRevision : objArray) {
			final IFileRevision revision = (IFileRevision) tempRevision;
			if (revision == null || !revision.exists()) {
				MessageDialog.openError(site.getShell(), "Deleted Revision", "Can't open a deleted revision");
			} else {
				IRunnableWithProgress runnable = new IRunnableWithProgress() {
					public void run(IProgressMonitor monitor) throws InvocationTargetException, InterruptedException {
						IStorage file;
						try {
							file = revision.getStorage(monitor);
							String id = getEditorID(file.getName(), file.getContents());

							if (file instanceof IFile) {
								// if this is the current workspace file, open
								// it
								IDE.openEditor(site.getPage(), (IFile) file);
							} else if (showEditor(revision)) {
								site.getPage().openEditor(new BazaarRevisionEditorInput(revision), id);
							}
						} catch (CoreException e) {
							throw new InvocationTargetException(e);
						}

					}

					private boolean showEditor(IFileRevision revision) {
						return true;
					}
				};
				IProgressService progressService = PlatformUI.getWorkbench().getProgressService();
				try {
					progressService.run(false, false, runnable);
				} catch (InvocationTargetException e) {
				} catch (InterruptedException e) {
				}
			}

		}
	}

	String getEditorID(String fileName, InputStream contents) {
		IWorkbench workbench = PlatformUI.getWorkbench();
		IEditorRegistry registry = workbench.getEditorRegistry();
		IContentType type = null;
		if (contents != null) {
			try {
				type = Platform.getContentTypeManager().findContentTypeFor(contents, fileName);
			} catch (IOException e) {
			}
		}
		if (type == null) {
			type = Platform.getContentTypeManager().findContentTypeFor(fileName);
		}
		IEditorDescriptor descriptor = registry.getDefaultEditor(fileName, type);
		String id;
		if (descriptor == null || descriptor.isOpenExternal()) {
			id = "org.eclipse.ui.DefaultTextEditor"; //$NON-NLS-1$
		} else {
			id = descriptor.getId();
		}

		return id;
	}

	@Override
    protected boolean updateSelection(IStructuredSelection selection) {
		this.selection = selection;
		return shouldShow();
	}

	protected boolean shouldShow() {
		Object[] objArray = selection.toArray();

		if (objArray.length == 0) {
            return false;
        }

		for (Object element : objArray) {
			IFileRevision revision = (IFileRevision) element;
			// check to see if any of the selected revisions are deleted
			// revisions
			if (revision != null && !revision.exists()) {
                return false;
            }
		}

		return true;
	}

}
