/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.Collection;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.BazaarCommand;
import org.vcs.bazaar.eclipse.core.commands.ConflictsCommand;
import org.vcs.bazaar.eclipse.core.commands.RebaseAbortCommand;
import org.vcs.bazaar.eclipse.core.commands.RebaseCommand;
import org.vcs.bazaar.eclipse.core.commands.RebaseContinueCommand;
import org.vcs.bazaar.eclipse.core.commands.RebaseToDoCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.dialogs.RebaseDialog;
import org.vcs.bazaar.eclipse.ui.dialogs.RebaseDialog.RebaseDialogInfo;

/**
 * @author Piotr Piastucki
 */
public class RebaseAction extends WorkbenchAction {

	@Override
	protected void execute(IAction action) throws BazaarException, InvocationTargetException, InterruptedException {
		final IProject[] projects = getEnclosingProjectsForSelectedResources();
		final IBzrBranch branch = BzrWorkspaceRoot.getBranchFor(projects[0]);
		RebaseToDoCommand cmd = new RebaseToDoCommand(branch);
		cmd.run(new NullProgressMonitor());
		if (cmd.isRebaseInterrupted()) {
			resumeRebase(branch);
		} else {
			startRebase(branch);
		}
	}

	private void resumeRebase(IBzrBranch branch) throws BazaarException, InvocationTargetException, InterruptedException {
		ConflictsCommand cmd = new ConflictsCommand(branch);
		cmd.run(new NullProgressMonitor());
		if (!cmd.getConflicts().isEmpty()) {
			MessageDialog dialog = new MessageDialog(getParentShell(),
													 UITexts.BazaarRebaseDialogTitle,
													 null,
													 UITexts.RebaseConflictsDialog_label,
													 MessageDialog.QUESTION,
													 new String[] {UITexts.RebaseResumeDialog_abort, IDialogConstants.CLOSE_LABEL},
													 1);
			dialog.setBlockOnOpen(true);
			int result = dialog.open();
			if (result == 0) {
				abortRebase(branch);
			}
		} else {
			MessageDialog dialog = new MessageDialog(getParentShell(),
					 								 UITexts.BazaarRebaseDialogTitle,
					 								 null,
					 								 UITexts.RebaseResumeDialog_label,
					 								 MessageDialog.QUESTION_WITH_CANCEL,
					 								 new String[] {
																   UITexts.RebaseResumeDialog_continue,
																   IDialogConstants.CLOSE_LABEL,
																   UITexts.RebaseResumeDialog_abort
																   },
					 								 0);
			int result = dialog.open();
			if (result == 2) {
				abortRebase(branch);
			} else if (result == 0) {
				continueRebase(branch);
			}
		}
	}

	private void abortRebase(final IBzrBranch branch) throws InvocationTargetException, InterruptedException {
		runRebaseCommand(new RebaseAbortCommand(branch));
	}

	private void continueRebase(final IBzrBranch branch) throws InvocationTargetException, InterruptedException {
		runRebaseCommand(new RebaseContinueCommand(branch));
	}

	public void startRebase(final IBzrBranch branch) throws BazaarException, InvocationTargetException,
			InterruptedException {
		final RebaseDialogInfo dialogInfo = new RebaseDialogInfo();
		final BranchLocation remoteLocation = getRemoteLocation(branch, dialogInfo);
		RebaseCommand cmd = new RebaseCommand(branch, remoteLocation, dialogInfo.getOptions().toArray(new Option[0]));
		if (remoteLocation != null) {
			runRebaseCommand(cmd);
		}
	}

	private BranchLocation getRemoteLocation(IBzrBranch branch, RebaseDialogInfo dialogInfo) throws BazaarException {
		Collection<IBzrBranch> locations = getAllBranchLocations(branch);
		IBzrBranch initialBranch = null;
		if (branch instanceof IBzrLocalBranch) {
			initialBranch = ((IBzrLocalBranch) branch).getParent();
		}
		RebaseDialog dialog = new RebaseDialog(getShell(), locations, initialBranch, dialogInfo);
		dialog.setBlockOnOpen(true);
		if (dialog.open() == RebaseDialog.OK) {
			try {
				return new BranchLocation(dialogInfo.getLocation());
			} catch (BazaarClientException e) {
			}
		}
		return null;
	}

	private void runRebaseCommand(final BazaarCommand cmd) throws InvocationTargetException, InterruptedException {
		run(new WorkspaceModifyOperation() {
			@Override
			public void execute(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					monitor.beginTask("", 1000);
					monitor.setTaskName(CoreTexts.RebaseCommand_taskName);
					cmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
				} catch (BazaarException e) {
					EclipseBazaarUI.log(e.getMessage(), e);
					throw new InvocationTargetException(e, e.getMessage());
				} finally {
					monitor.done();
				}
			}
		}, false /* no cancelable */, PROGRESS_DIALOG);
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}
	
	@Override
	protected boolean isEnabledByType(IResource resource) {
		return resource.getType() == IResource.PROJECT && EclipseBazaarCore.getDefault().getPlugin("rewrite") != null;
	}

}
