/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.IJobStatus;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.window.Window;
import org.eclipse.ui.actions.WorkspaceModifyOperation;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.IBazaarShelf;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IUnShelveOptions;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.CoreTexts;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.commands.ShelveListCommand;
import org.vcs.bazaar.eclipse.core.commands.UnShelveCommand;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.utils.BzrJobStatus;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.dialogs.UnShelveDialog;

/**
 * @author Piotr Piastucki
 */
public class UnShelveAction extends WorkbenchAction {

	@Override
	protected void execute(IAction action) throws Exception {
		final IProject[] projects = getEnclosingProjectsForSelectedResources();
		final UnShelveDialog unshelveDialog = new UnShelveDialog(getShell());
		final IBzrBranch branch = BzrWorkspaceRoot.getBranchFor(projects[0]);
		UIJob job = new UIJob(getShell().getDisplay(), UITexts.UnShelveAction_gettingShelvesTaskName) {
			@Override
			public IStatus runInUIThread(IProgressMonitor monitor) {
				monitor.beginTask(UITexts.UnShelveAction_gettingShelvesTaskName, 1000);
				try {
					ShelveListCommand cmd = new ShelveListCommand(branch);
					cmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
					if (cmd.getShelves().isEmpty()) {
						return BzrJobStatus.INFO(this, UITexts.UnShelveAction_nothingToUnshelve);
					}
					return new BzrJobStatus<List<IBazaarShelf>>(IJobStatus.OK, this, cmd.getShelves(), "Resources calculated");
				} catch (Exception e) {
					return UIJob.errorStatus(e);
				} finally {
					monitor.done();
				}
			}
		};
		job.setUser(true);
		job.setPriority(UIJob.LONG);
		job.addJobChangeListener(new JobChangeAdapter() {
			@SuppressWarnings("unchecked")
			@Override
			public void done(IJobChangeEvent event) {
				IStatus status = event.getResult();
				if (status instanceof BzrJobStatus) {
					if (status.getSeverity() == IStatus.INFO) {
						MessageDialog.openInformation(getParentShell(), UITexts.Dialog_informationTitle,
								status.getMessage());
					} else if (status.getSeverity() == IStatus.OK) {
						int exit = unshelveDialog.open(((BzrJobStatus<List<IBazaarShelf>>) status).getResult());
						if (exit == Window.OK) {
							try {
								unshelve(branch, unshelveDialog);
							} catch (Exception e) {
								handle(e);
							}
						}
					}
				}
				event.getJob().removeJobChangeListener(this);
			}
		});
		job.schedule();
	}

	private void unshelve(final IBzrBranch branch, UnShelveDialog unshelveDialog) throws InvocationTargetException, InterruptedException {
		final String shelfId = unshelveDialog.getShelfId();
		final List<Option> options = new ArrayList<Option>();
		if (unshelveDialog.isDryRun()) {
			options.add(IUnShelveOptions.DRY_RUN);
		}
		if (unshelveDialog.isKeep()) {
			options.add(IUnShelveOptions.KEEP);
		}
		if (unshelveDialog.isDeleteOnly()) {
			options.add(IUnShelveOptions.DELETE_ONLY);
		}
		run(new WorkspaceModifyOperation() {
			@Override
            public void execute(IProgressMonitor monitor) throws InvocationTargetException {
				try {
					monitor.beginTask(CoreTexts.UnShelveCommand_taskName, 1000);
					final UnShelveCommand cmd = new UnShelveCommand(branch, shelfId, options.toArray(new Option[0]));
					cmd.run(EclipseBazaarCore.subProgressMonitorFor(monitor));
				} catch (Exception e) {
					EclipseBazaarUI.log(e.getMessage(), e);
				} finally {
					monitor.done();
				}
			}
		}, false /* no cancelable */, PROGRESS_DIALOG);
	}

	@Override
	protected boolean isEnabledForMultipleEclosingProjects() {
		return isSameBranchInEnclosingProjects();
	}

}

