/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.actions;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;

import org.eclipse.core.resources.IResource;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.IWorkbenchWindowActionDelegate;
import org.eclipse.ui.PlatformUI;
import org.vcs.bazaar.client.IBazaarInfo;
import org.vcs.bazaar.client.IBazaarInfo.RelatedBranches;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.internal.core.model.LocalBranch;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.branch.BranchLocationManager;

/**
 * <p>
 * Base class for actions that may be tied to the workbench.
 * </p>
 * <p>
 * i.e: An action that is displayed in a menu, toolbar, etc
 * </p>
 *
 * @author Guillermo Gonzalez
 *
 */
public abstract class WorkbenchAction extends WorkspaceAction implements IWorkbenchWindowActionDelegate {

	protected IWorkbenchWindow window;

	/**
	 * We will cache window object in order to be able to provide parent shell
	 * for the message dialog.
	 *
	 * @see IWorkbenchWindowActionDelegate#init
	 */
	@Override
    public void init(IWorkbenchWindow window) {
//		System.out.println(this.getClass().getName() + ":init(window)");
		this.window = window;
	}

	/**
	 * Find and return the shell of the parent "window"
	 *
	 * @return the parent window Shell or a workbench Shell
	 */
	protected Shell getParentShell() {
		Shell shell;
		if ((window != null) && (window.getShell() != null)) {
			shell = window.getShell();
		} else {
			IWorkbench workbench = PlatformUI.getWorkbench();
			shell = workbench.getActiveWorkbenchWindow().getShell();
		}
		return shell;
	}

	protected IWorkbenchPage getActivePage() {
		if(window != null)
			return window.getActivePage();
		IWorkbench wb = PlatformUI.getWorkbench();
		IWorkbenchWindow win = wb.getActiveWorkbenchWindow();
		return win.getActivePage();

	}


	protected Collection<IBzrBranch> getAllSavedBranches() throws BazaarException {
		Set<IBzrBranch> locations = new TreeSet<IBzrBranch>();
		for (IBzrBranch branch : BranchLocationManager.getAllBranches()) {
			if (branch != null && branch.getBranchLocation() != null)
				locations.add(branch);
		}
		return locations;
	}
	
	private void addBranchIfValid(Collection<IBzrBranch> branches, String location) {
		IBzrBranch branch = BranchFactory.findOrCreateFor(location);
		if (branch != null) {
			branches.add(branch);
		}
	}
	
	/**
	 * @param project
	 * @return
	 * @throws BazaarException
	 */
	protected Collection<IBzrBranch> getAllBranchLocations(IBzrBranch branch) throws BazaarException {
		Collection<IBzrBranch> locations = getAllSavedBranches();
		if (branch instanceof LocalBranch) {
			IBazaarInfo info = ((LocalBranch)branch).getInfo();
			RelatedBranches rb = info.getRelatedBranches();
			if (rb != null) {
				addBranchIfValid(locations, rb.getParentBranch());
				addBranchIfValid(locations, rb.getPushBranch());
				addBranchIfValid(locations, rb.getPublicBranch());
				addBranchIfValid(locations, rb.getSubmitBranch());
			}
		}
		return locations;
	}

	public static List<IResource> getUnknownOrAddedResources(List<IBzrResourceStatus>selection) {
		ArrayList<IResource> result = new ArrayList<IResource>();
		for (IBzrResourceStatus status : selection) {
			if (status.isAdded() || status.isUnknown()) {
				result.add(status.getResource().getIResource());
			}
		}
		return result;
	}

	public static List<IResource> getUnknownResources(List<IBzrResourceStatus>selection) {
		ArrayList<IResource> result = new ArrayList<IResource>();
		for (IBzrResourceStatus status : selection) {
			if (status.isUnknown()) {
				result.add(status.getResource().getIResource());
			}
		}
		return result;
	}

	public static List<IBzrLocalResource> getResources(List<IBzrResourceStatus>selection) {
		ArrayList<IBzrLocalResource> result = new ArrayList<IBzrLocalResource>();
		for (IBzrResourceStatus status : selection) {
			result.add(status.getResource());
		}
		return result;
	}

	protected String handleConflicts(InvocationTargetException ite) throws InvocationTargetException {
		String message = null;
		Throwable ie = ite.getCause();
		if (ie instanceof BazaarException) {
		    if (ie.getCause() != null) {
		        ie = ie.getCause();
		    }
		    if (ie.getMessage().contains("conflicts encountered")) {
		    	message = UITexts.Conflicts_error + ie.getMessage();
		        ite = null;
		    }
		}
		if (ite != null) {
		    throw ite;
		}
		return message;
	}

}
