/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.branch;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.repository.BranchFactory;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * @author Piotr Piastucki
 */
public class BranchLocationManager {

	private static final String KEY_LOCATION = "location"; //$NON-NLS-1$

	private static final String KEY_LOCATIONS = "locations"; //$NON-NLS-1$

	private static Set<IBzrBranch> branches;
	/*
	 * Return an ordered list of all repository locations that are presently
	 * known.
	 */
	public static Set<IBzrBranch> getAllBranches() {
		if (branches == null) {
			branches = new TreeSet<IBzrBranch>();
			for (BranchLocation location : getLocations()) {
				branches.add(BranchFactory.getBranch(location));
			}
		}
		return branches;
	}

	private static Set<BranchLocation> getLocations() {
		String all = getPreferenceStore().getString(BazaarPreferenceConstants.BRANCH_LOCATION_HISTORY);
		if (all.length() == 0) {
			return Collections.emptySet();
		}
		int max = getLocationHistorySize();
		if (max < 1) {
			return Collections.emptySet();
		}
		XMLMemento memento;
		try {
			memento = XMLMemento.createReadRoot(new StringReader(all));
		} catch (WorkbenchException e) {
			return Collections.emptySet();
		}
		Set<BranchLocation> locations = new LinkedHashSet<BranchLocation>();
		for (IMemento child : memento.getChildren(KEY_LOCATION)) {
			try {
				locations.add(new BranchLocation(child.getTextData()));
				if (locations.size() == max) {
					break;
				}
			} catch (BazaarClientException e) {
			}
		}
		return locations;
	}

	/**
	 * Save a new commit message.
	 *
	 * @param message
	 */
	public static void saveBranchLocation(String location) {
		if (location == null || location.trim().isEmpty()) {
			return;
		}
		try {
			int size = getLocationHistorySize();
			ArrayList<BranchLocation> locations = new ArrayList<BranchLocation>();
			if (size > 0) {
				BranchLocation bl = new BranchLocation(location);
				locations.add(bl);
				if (locations.size() < size) {
					Set<BranchLocation> history = getLocations();
					history.remove(bl);
					for (BranchLocation previous : history) {
						locations.add(previous);
						if (locations.size() == size)
							break;
					}
				}
			}
			saveBranchLocations(locations, BazaarPreferenceConstants.BRANCH_LOCATION_HISTORY);
			branches = null;
		} catch (BazaarClientException e) {
		}
	}
	
	private static void saveBranchLocations(Collection<BranchLocation> locations, String preferenceKey) {
		XMLMemento memento = XMLMemento.createWriteRoot(KEY_LOCATIONS);
		for (BranchLocation l : locations) {
			memento.createChild(KEY_LOCATION).putTextData(l.toString());
		}
		StringWriter writer = new StringWriter();
		try {
			memento.save(writer);
			getPreferenceStore().setValue(preferenceKey, writer.toString());
		} catch (IOException e) {
		}
	}

	private static IPreferenceStore getPreferenceStore() {
		return EclipseBazaarUI.getDefault().getPreferenceStore();
	}

	private static int getLocationHistorySize() {
		return 10;
	}

}
