/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.compare;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import org.eclipse.compare.IContentChangeListener;
import org.eclipse.compare.IEditableContent;
import org.eclipse.compare.IStreamContentAccessor;
import org.eclipse.compare.ITypedElement;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.ui.compare.internal.Utilities;

/**
 * @author Piotr Piastucki
 */
public class BazaarLocalResourceNode extends BazaarResourceNode {

	private boolean fDirty= false;
	private IFile fDeleteFile;
	private IContentChangeListener listener;

	public BazaarLocalResourceNode(IBzrLocalResource resource) {
		super(resource);
	}

	public BazaarLocalResourceNode(IResource resource) {
		super(resource);
	}

	@Override
	public void addContentChangeListener(IContentChangeListener listener) {
		super.addContentChangeListener(listener);
		this.listener = listener;
	}
	
	@Override
	public void setContent(byte[] contents) {
		fDirty = true;
		super.setContent(contents);
	}
	
	@Override
	public ITypedElement replace(ITypedElement child, ITypedElement other) {
		
		if (child == null) {	// add resource
			// create a node without a resource behind it!
			IResource resource = getResource();
			if (resource instanceof IContainer) {
				IContainer container = (IContainer) resource;
				IFile file= container.getFile(new Path(other.getName()));
				BazaarLocalResourceNode localNode = new BazaarLocalResourceNode(file);
				localNode.addContentChangeListener(listener);
				child = localNode;
			}
		}
		
		if (other == null) {	// delete resource
			IResource resource= getResource();
			if (resource instanceof IContainer) {
				IContainer container = (IContainer) resource;
				IFile file= container.getFile(new Path(child.getName()));
				if (file != null && file.exists()) {
					fDeleteFile= file;
					fDirty= true;
					fireContentChanged();
				}
			}
			return null;
		}
		
		if (other instanceof IStreamContentAccessor && child instanceof IEditableContent) {
			IEditableContent dst= (IEditableContent) child;
			
			try {
				InputStream is= ((IStreamContentAccessor)other).getContents();
				
				byte[] bytes= Utilities.readBytes(is);
				if (bytes != null)
					dst.setContent(bytes);
			} catch (CoreException ex) {
				// NeedWork
			}
		}
		return child;
	}
	
	public void commit(IProgressMonitor monitor) throws CoreException {
		if (fDirty) {
			InputStream is = null;
			try {
				if (fDeleteFile != null) {
					fDeleteFile.delete(true, true, monitor);
					return;
				}
				byte[] content = getContent();
				if (content == null) {
					content = new byte[0];
				}
				is = new ByteArrayInputStream(content);
				IResource resource= getResource();
				if (resource instanceof IFile) {
					IFile file = (IFile)getResource();
					if (file.exists()) {
						file.setContents(is, false, true, monitor);
					} else {
						file.create(is, false, monitor);
					}
				}
				fDirty= false;
			} finally {
				if (is != null) {
					try {
						is.close();
					} catch (IOException e) {
						// ignore
					}
				}
			}
		}
	}
	
}
