/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.compare;

import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.compare.IContentChangeListener;
import org.eclipse.compare.ResourceNode;
import org.eclipse.compare.structuremergeviewer.DiffNode;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.core.resources.IResource;

/**
 * @author Piotr Piastucki
 */
public class BazaarResourceDiffNodeTree {

	private boolean readonly;
	private IResource[] roots;
	private List<DiffNode> nodes;
	private IContentChangeListener listener;

	public BazaarResourceDiffNodeTree(boolean readonly, IResource[] roots, IContentChangeListener listener) {
		this.readonly = readonly;
		this.roots = roots;
		this.nodes = new ArrayList<DiffNode>();
		this.listener = listener;
	}

	public void add(ResourceNode left, ResourceNode right) {
		DiffNode diffNode;
		if (left != null && right != null) {
			diffNode = new DiffNode(null, Differencer.CHANGE, null, left, right);
		} else if (left == null) {
			diffNode = new DiffNode(Differencer.ADDITION, null, null, right);
		} else {
			diffNode = new DiffNode(Differencer.DELETION, null, left, null);
		}
		nodes.add(diffNode);
	}

	private boolean isRoot(IResource resource) {
		for (IResource root : roots) {
			if (root.equals(resource)) {
				return true;
			}
		}
		return false;
	}

	public IResource findBestRoot(ResourceNode node) {
		IResource resource = node.getResource();
		IResource root = null;
		while (resource != null) {
			if (isRoot(resource)) {
				root = resource;
			}
			resource = resource.getParent();
		}
		return root;
	}

	public DiffNode getNodes() {
		Map<IResource, DiffNode> nodeMap = new LinkedHashMap<IResource, DiffNode>();
		for (DiffNode node : nodes) {
			ResourceNode brn = (ResourceNode) (node.getLeft() != null ? node.getLeft() : node.getRight());
			nodeMap.put(brn.getResource(), node);
		}
		for (DiffNode node : nodes) {
			ResourceNode brn = (ResourceNode) (node.getLeft() != null ? node.getLeft() : node.getRight());
			IResource root = findBestRoot(brn);
			for (IResource resource = brn.getResource(); resource.getParent() != null && !resource.equals(root);) {
				resource = resource.getParent();
				DiffNode parentNode = nodeMap.get(resource);
				if (parentNode != null) {
					parentNode.add(node);
					break;
				}
				ResourceNode folderNodeLeft = readonly ? new ResourceNode(resource) : new BazaarLocalResourceNode(resource);
				folderNodeLeft.addContentChangeListener(listener);
				ResourceNode folderNodeRight = new ResourceNode(resource);
				parentNode = new DiffNode(null, Differencer.CHANGE, null, folderNodeLeft, folderNodeRight);
				nodeMap.put(resource, parentNode);
				parentNode.add(node);
				node = parentNode;
			}
		}
		if (nodeMap.size() == 1) {
			// special handling for single file comparison
			return nodeMap.values().iterator().next();
		}
		// just 1 folder, skip root folder name node to simplify the tree
		if (roots.length == 1) {
			DiffNode node = nodeMap.get(roots[0]);
			if (node != null) {
				return node;
			}
		}
		// structured comparison, dummy root node
		DiffNode rootNode = new DiffNode(null, Differencer.CHANGE, null, new ResourceNode(roots[0].getParent()), new ResourceNode(roots[0].getParent()));
		for (IResource root : roots) {
			DiffNode node = nodeMap.get(root);
			if (node != null) {
				rootNode.add(node);
			}
		}
		return rootNode;
	}

}
