/*******************************************************************************
 * Copyright (c) 2005-2008 Polarion Software.
 * Copyright (c) 2012 Idereal Limited
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Gurov - Initial API and implementation
 *    Alexander Taler - Copied from Subversive project to bzr-eclipse
 *******************************************************************************/

package org.vcs.bazaar.eclipse.ui.compare;

import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.lang.reflect.InvocationTargetException;

import org.eclipse.compare.CompareConfiguration;
import org.eclipse.compare.CompareEditorInput;
import org.eclipse.compare.CompareUI;
import org.eclipse.compare.ITypedElement;
import org.eclipse.compare.structuremergeviewer.Differencer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.swt.graphics.Image;

/**
 * Conflicting files merger editor input implementation
 * 
 * @author Alexander Gurov
 */
public class ConflictingFileEditorInput extends CompareEditorInput {
    protected IFile target;
    protected IFile left;
    protected IFile right;
    protected IFile ancestor;

    protected MergeElement targetElement;

    public ConflictingFileEditorInput(CompareConfiguration configuration,
        IFile target,
        IFile left,
        IFile right,
        IFile ancestor)
    {
        super(configuration);
        this.target = target;
        this.left = left;
        this.right = right;
        this.ancestor = ancestor;
    }

    @Override
	protected Object prepareInput(IProgressMonitor monitor)
        throws InvocationTargetException, InterruptedException
    {
        CompareConfiguration cc = this.getCompareConfiguration();
        Image img = CompareUI.getImage(this.target);
        cc.setLeftImage(img);
        cc.setRightImage(img);
        cc.setAncestorImage(img);
        cc.setLeftLabel(this.target.getName() + " Local Revision"); //$NON-NLS-1$
        cc.setLeftEditable(true);
        cc.setRightLabel(this.target.getName() + " Incoming Revision"); //$NON-NLS-1$
        cc.setRightEditable(false);
        cc.setAncestorLabel(this.target.getName() + " Common Base"); //$NON-NLS-1$

        this.setTitle(this.target.getName() + " Conflicts"); //$NON-NLS-1$

        InputStream stream = null;
        try {
            stream = this.left.getContents();
            byte[] buf = new byte[2048];
            int len = 0;
            ByteArrayOutputStream output = new ByteArrayOutputStream();
            while ((len = stream.read(buf)) > 0) {
                output.write(buf, 0, len);
            }
            this.targetElement = new MergeElement(this.target, output.toByteArray(), true);
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new InvocationTargetException(e);
        } finally {
            if (stream != null) {
                try {
                    stream.close();
                } catch (Exception ex) {
                }
            }
        }

        try {
            MergeElement rightRef = new MergeElement(this.right);
            rightRef.setCharsetReference(this.targetElement);
            MergeElement ancestorRef = null;
            if (this.ancestor != null) {
            	ancestorRef = new MergeElement(this.ancestor);
                ancestorRef.setCharsetReference(this.targetElement);
            }
            return new Differencer().findDifferences(true, monitor, null, ancestorRef,
                this.targetElement, rightRef);
        } finally {
            monitor.done();
        }
    }

    @Override
	public Object getAdapter(Class adapter) {
        if (IFile.class.equals(adapter)) {
            // disallow auto-flush of editor content
            return this.target;
        }
        return super.getAdapter(adapter);
    }

    @Override
	public void saveChanges(IProgressMonitor pm) throws CoreException {
        // flush editor content...
        super.saveChanges(pm);
        // ...and save it
        this.targetElement.commit(pm);

        this.setDirty(false);
    }

    protected class MergeElement extends BazaarLocalResourceNode {
        protected boolean editable;
        protected BazaarLocalResourceNode charsetReference;

        public BazaarLocalResourceNode getCharsetReference() {
            return this.charsetReference;
        }

        public void setCharsetReference(BazaarLocalResourceNode charsetReference) {
            this.charsetReference = charsetReference;
        }

        public MergeElement(IResource resource) {
            this(resource, null, false);
        }

        public MergeElement(IResource resource, byte[] initialContent, boolean editable) {
            super(resource);
            this.editable = editable;
            if (initialContent != null) {
                this.setContent(initialContent);
            }
        }

        @Override
		public String getCharset() {
            return this.charsetReference != null ? this.charsetReference.getCharset() : super
                .getCharset();
        }

        @Override
		public String getType() {
            String extension = ConflictingFileEditorInput.this.target.getFileExtension();
            return extension == null ? ITypedElement.UNKNOWN_TYPE : extension;
        }

        @Override
		public boolean isEditable() {
            return this.editable;
        }
    }

}
