/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.decorator;

import java.text.DateFormat;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.IDecoration;
import org.eclipse.jface.viewers.ILightweightLabelDecorator;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.LabelProviderChangedEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.widgets.Display;
import org.eclipse.team.core.RepositoryProvider;
import org.eclipse.team.ui.TeamUI;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.themes.ITheme;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.listeners.IResourceStateListener;
import org.vcs.bazaar.eclipse.core.model.BzrStatus;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProvider;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.ITracingOptions;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class BazaarLightweightDecorator extends LabelProvider implements ILightweightLabelDecorator, IResourceStateListener, IPropertyChangeListener {

	public static final DateFormat dateFormat = DateFormat.getInstance();
	private final IPreferenceStore store = EclipseBazaarUI.getDefault().getPreferenceStore();

	public BazaarLightweightDecorator() {
		super();
		EclipseBazaarCore.addResourceStateListener(this);
		TeamUI.addPropertyChangeListener(this);
		PlatformUI.getWorkbench().getThemeManager().getCurrentTheme().addPropertyChangeListener(this);
		store.addPropertyChangeListener(this);
		ensureFontAndColorsCreated();
	}

	public static void refresh() {
		EclipseBazaarCore.getDefault().getStatusCacheManager().setLogCacheEnabled(
				EclipseBazaarUI.getDefault().getPreferenceStore().getBoolean(BazaarPreferenceConstants.FILE_TEXT_DECORATOR));
		EclipseBazaarUI.getDefault().getWorkbench().getDecoratorManager().update(BazaarLightweightDecorator.class.getName());
	}

	public void decorate(Object element, IDecoration decoration) {
		IResource resource = (IResource) element;
		if(resource.isAccessible() && !resource.isLinked(IResource.CHECK_ANCESTORS) && !resource.isDerived() && !resource.isTeamPrivateMember()) {
			IProject project = resource.getProject();
			if (project != null && RepositoryProvider.isShared(project) && getRepositoryProvider(resource) != null) {
				decorate(resource, project, decoration);
			}
		}
	}

	private boolean isInsideMetaFolder(IResource objectResource) {
		for (String segment : objectResource.getLocation().segments()) {
			if (IBzrBranch.DIR_META_FOLDER.equals(segment)) {
				return true;
			}
		}
		return false;
	}

	private void decorate(IResource objectResource, IProject objectProject, IDecoration decoration) {
		// Do not decorate the stuff under .bzr and derived resources
		if (objectResource.isAccessible() 
				&& !isInsideMetaFolder(objectResource)
				&& !objectResource.isLinked(IResource.CHECK_ANCESTORS) 
				&& !objectResource.isDerived(IResource.CHECK_ANCESTORS)) {
			try {
				decorateByType(objectResource, decoration);
			} catch (CoreException e) {
				EclipseBazaarUI.log(e);
			} catch (RuntimeException e) {
				// I can get a runtimeException if the project is closed when
				// I'm still computing decorations
				EclipseBazaarUI.traceException(ITracingOptions.LABEL_DECORATOR, e);
				if (objectProject.isOpen())
					EclipseBazaarUI.log("Unexpected error while decorating: "+objectResource.getName(), e);
			}
		}
	}

	private void decorateByType(IResource resource, IDecoration decoration) throws CoreException {
		
		IBzrLocalResource bzrResource = BzrWorkspaceRoot.getBzrResourceFor(resource);
		BzrStatus status = bzrResource.getStatus();
		if (resource.getType() == IResource.PROJECT) {
			if (store.getBoolean(BazaarPreferenceConstants.PROJECT_TEXT_DECORATOR)) {
				// set the label (only if the resource is a project)
				String nick = bzrResource.getBranch().getNick();
				if (nick != null) {
					decoration.addSuffix("  [" + nick + "]");
				}
			}
		} else if (status != null && !status.isAdded() && !status.isUnknown() && !status.isIgnored()) {
			if (store.getBoolean(BazaarPreferenceConstants.FILE_TEXT_DECORATOR)) {
				IBazaarLogMessage logMessage = getLastLogMessage(resource);
				if (logMessage != null) {
					decoration.addSuffix("  " + logMessage.getRevision().getValue() + " " + logMessage.getCommiter() + " " +
							dateFormat.format(logMessage.getDate()));
				}
			}
		}
		if (status != null) {
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_TEXT_DECORATOR)) {
				decoration.addPrefix(DecoratorTextLabels.getStatusTextLabel(status));
			}
			if (store.getBoolean(BazaarPreferenceConstants.STATUS_OVERLAY_DECORATOR)) {
				decoration.addOverlay(DecoratorImages.getImageDescriptor(status));
			}
			decorateFontAndColour(status, decoration);
		}
	}

	private void decorateFontAndColour(BzrStatus status, IDecoration decoration) {
		if (status.isAnyModifiedStatus()) {
			ITheme current = PlatformUI.getWorkbench().getThemeManager().getCurrentTheme();
			Color bc = current.getColorRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_BACKGROUND);
			Color fc = current.getColorRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FOREGROUND);
			Font f = current.getFontRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FONT);
			decoration.setBackgroundColor(bc);
			decoration.setForegroundColor(fc);
			decoration.setFont(f);
		}
	}
	
	/**
	 * This method will ensure that the fonts and colors used by the decorator
	 * are cached in the registries. This avoids having to syncExec when
	 * decorating since we ensure that the fonts and colors are pre-created.
	 *
	 * @param actFonts fonts ids to cache
	 * @param actColors color ids to cache
	 */
	private void ensureFontAndColorsCreated() {
		Display.getDefault().syncExec(new Runnable() {
			public void run() {
				ITheme theme  = PlatformUI.getWorkbench().getThemeManager().getCurrentTheme();
				theme.getColorRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FOREGROUND);
				theme.getColorRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_BACKGROUND);
				theme.getFontRegistry().get(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FONT);
			}
		});
	}
	
	@Override
    public void dispose() {
		super.dispose();
		EclipseBazaarCore.removeResourceStateListener(this);
		TeamUI.removePropertyChangeListener(this);
		PlatformUI.getWorkbench().getThemeManager().getCurrentTheme().removePropertyChangeListener(this);
		store.removePropertyChangeListener(this);
	}

	@Override
    public boolean isLabelProperty(Object element, String property) {
		return false;
	}

	public void resourceStateChanged(final BzrChangeStateEvent event) {
		postLabelChangeEvent();
	}

	private static BazaarTeamProvider getRepositoryProvider(IResource resource) {
		RepositoryProvider repoProvider = RepositoryProvider.getProvider(resource.getProject());
		if (!(repoProvider instanceof BazaarTeamProvider)) {
			// Resource not unsing this plugin
			return null;
		}
		if (((BazaarTeamProvider)repoProvider).getBzrWorkspaceRoot() == null) {
			// root is null most likely due to missing .bzr folder 
			return null;
		}
		return (BazaarTeamProvider) repoProvider;
	}

	private static IBazaarLogMessage getLastLogMessage(IResource resource) throws CoreException {
		return EclipseBazaarCore.getDefault().getStatusCacheManager().getLastLog(resource);
	}

	public void propertyChange(PropertyChangeEvent event) {
		final String prop = event.getProperty();
		if (prop.equals(TeamUI.GLOBAL_IGNORES_CHANGED) || prop.equals(TeamUI.GLOBAL_FILE_TYPES_CHANGED)) {
			postLabelChangeEvent();
		} else if (prop.equals(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FOREGROUND)
				|| prop.equals(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_BACKGROUND)
				|| prop.equals(BazaarPreferenceConstants.THEME_UNCOMMITTED_CHANGE_FONT)) {
			ensureFontAndColorsCreated();
			postLabelChangeEvent();
		}
	}

	private void postLabelChangeEvent() {
		LabelEventJob.getInstance().postLabelEvent(this);
	}

	private void fireLabelChangeEvent() {
		final LabelProviderChangedEvent event = new LabelProviderChangedEvent(this);
		Display.getDefault().asyncExec(new Runnable() {
			public void run() {
				fireLabelProviderChanged(event);
			}
		});
	}

	
	/**
	 * Job reducing label events to prevent unnecessary (i.e. redundant) event
	 * processing
	 */
	private static class LabelEventJob extends Job {

		/**
		 * Constant defining the waiting time (in milliseconds) until an event is fired
		 */
		private static final long DELAY = 100L;

		private static LabelEventJob instance = new LabelEventJob("LabelEventJob"); //$NON-NLS-1$

		private BazaarLightweightDecorator decorator = null;

		static LabelEventJob getInstance() {
			return instance;
		}

		private LabelEventJob(final String name) {
			super(name);
		}

		void postLabelEvent(final BazaarLightweightDecorator decorator) {
			this.decorator = decorator;
			if (getState() == SLEEPING || getState() == WAITING) {
				cancel();
			}
			schedule(DELAY);
		}

		@Override
		protected IStatus run(IProgressMonitor monitor) {
			if (decorator != null) {
				decorator.fireLabelChangeEvent();
			}
			return Status.OK_STATUS;
		}
	}
}
