/*******************************************************************************
 * Copyright (c) 2006 Software Balm Consulting Inc.
 *
 * This software is licensed under the zlib/libpng license.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from the
 * use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it freely,
 * subject to the following restrictions:
 *
 *  1. The origin of this software must not be misrepresented; you must not
 *            claim that you wrote the original software. If you use this
 *            software in a product, an acknowledgment in the product
 *            documentation would be appreciated but is not required.
 *
 *   2. Altered source versions must be plainly marked as such, and must not be
 *            misrepresented as being the original software.
 *
 *   3. This notice may not be removed or altered from any source distribution.
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IResource;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.text.DocumentEvent;
import org.eclipse.jface.text.IDocumentListener;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerFilter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ControlAdapter;
import org.eclipse.swt.events.ControlEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.TabFolder;
import org.eclipse.swt.widgets.TabItem;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ICommitOptions;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.UIUtils;
import org.vcs.bazaar.eclipse.ui.actions.CommitAction.CommitableResources;
import org.vcs.bazaar.eclipse.ui.history.HistoryTreeViewer;
import org.vcs.bazaar.eclipse.ui.history.HistoryTreeViewer.Column;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;

/**
 * <br>
 * A commit dialog box allowing choosing of what files to commit and a commit
 * message for those files. Untracked files may also be chosen. <br>
 *
 * @author Peter Hunnisett <peter_hge at softwarebalm dot com> <br>
 * @modifiedBy Guillermo Gonzalez
 * @modifiedBy Piotr Piastucki
 *
 */
public class CommitDialog extends TrayDialog {

    private class CommittableFilesFilter extends ViewerFilter {

        public CommittableFilesFilter() {
            super();
        }

        /**
         * Filter out uncommittable files (i.e. ! -> deleted but still tracked)
         */
        @Override
        public boolean select(Viewer viewer, Object parentElement, Object element) {
            if (element instanceof IBzrResourceStatus) {
                IBzrResourceStatus status = ((IBzrResourceStatus) element);
                IBzrLocalResource bzrResource = status.getResource();
                if (status.isUnchanged()
                    || status.isIgnored()
                    || (!status.isDeleted() && bzrResource.getIResource().getType() == IResource.PROJECT))
                {
                    return false;
                }
            }
            return true;
        }
    }

	private static final int COMMIT_AND_PUSH_ID = 3;

    private static Column[] columns =
        new Column[] { Column.REVID, Column.COMMITTER, Column.TIMESTAMP, Column.MESSAGE };

    private SpellcheckableMessageArea commitText;

    private Group optionGroup;

    private Label commitTextLabel;

    private Button localCommitButton;
    
    private CommitableResources commitableResources;

    private List<IBzrResourceStatus> selectedFiles;

    private String commitMessage;

    private boolean pushAfterCommit;

    private CommitableResources selectedResources;

    private Text authorText;

    private BzrResourceSelectionComposite resourceSelectionComposite;

    private Text fixesText;

    private ArrayList<Option> options = new ArrayList<Option>(0);

    private boolean pendingMerge;

    public CommitDialog(Shell shell, CommitableResources commitableResources,
        CommitableResources selectedResources)
    {
        this(shell);
        this.commitableResources = commitableResources;
        this.selectedResources = selectedResources;
    }

    public CommitDialog(Shell shell) {
        super(shell);
        setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.MAX | SWT.BORDER | SWT.TITLE);
        setBlockOnOpen(false);
    }

    public String getCommitMessage() {
        return commitMessage;
    }

	public List<IBzrResourceStatus> getSelectedFiles() {
		return selectedFiles;
	}

	@Override
	public void create() {
		super.create();
		getButton(IDialogConstants.OK_ID).setEnabled(false);
		getButton(COMMIT_AND_PUSH_ID).setEnabled(false);
	}
	
    @Override
    protected Control createDialogArea(Composite parent) {
        Composite container = (Composite) super.createDialogArea(parent);
        getShell().setText(UITexts.CommitDialog_dialogTitle);
        container.setLayout(new GridLayout(1, false));
        container.setLayoutData(new GridData(GridData.FILL_BOTH));
        
        Composite messageArea = new Composite(container, SWT.NONE);
		GridLayoutFactory.swtDefaults().applyTo(messageArea);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(messageArea);
        commitTextLabel = new Label(messageArea, SWT.NONE);
        commitTextLabel.setText(UITexts.CommitDialog_commentsTitle);

        
        commitText = new CommitMessageArea(messageArea, "", SWT.BORDER) {
			@Override
			protected CommitMessageContentProposalProcessor getCommitProposalProcessor() {
				return new CommitMessageContentProposalProcessor(commitableResources.getAllStatuses());
			}
        	
        };
        GridData textBoxData = new GridData(GridData.FILL_HORIZONTAL);
        textBoxData.heightHint = 150;
        commitText.setLayoutData(textBoxData);
		UIUtils.addBulbDecorator(commitText.getTextWidget(), UITexts.CommitDialog_contentAssist);

		commitText.getDocument().addDocumentListener(new IDocumentListener() {

			public void documentChanged(DocumentEvent event) {
				enableOkButton();
			}

			public void documentAboutToBeChanged(DocumentEvent event) {
			}
		});


		Composite textArea = new Composite(container, SWT.NONE);
		GridLayoutFactory.swtDefaults().numColumns(2).applyTo(textArea);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(textArea);

		createAuthorTextBox(textArea);
		createFixesTextBox(textArea);

		resourceSelectionComposite = new BzrResourceSelectionComposite(!pendingMerge, !pendingMerge) {

            @Override
            protected void createOption(Composite container) {
                optionGroup = new Group(container, SWT.NONE);
                optionGroup.setText(UITexts.Dialog_optionsGroup);
                optionGroup.setLayout(new GridLayout(3, false));

                GridData data;
                data = new GridData();
                data.horizontalAlignment = GridData.FILL;
                data.verticalAlignment = GridData.END;
                data.grabExcessHorizontalSpace = true;
                data.grabExcessVerticalSpace = true;
                optionGroup.setLayoutData(data);

                localCommitButton = new Button(optionGroup, SWT.CHECK);
                localCommitButton.setText(UITexts.CommitDialog_localCommit);
            }

            @Override
            protected String getUpperLabel() {
                if (!pendingMerge) {
                	return "Select Files:";
                }
                return "";
            }

            @Override
            protected void createMiddleControls(Composite container) {
                if (pendingMerge) {
                    TabFolder folder = new TabFolder(container, SWT.NONE);
                    folder.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
                    TabItem fileListItem = new TabItem(folder, SWT.NONE);
                    fileListItem.setText("File list");
                    super.createMiddleControls(folder);
                    fileListItem.setControl(super.getTableViewer().getControl().getParent());
                    TabItem logItem = new TabItem(folder, SWT.NONE);
                    TreeViewer tv = new HistoryTreeViewer(folder, columns);
                    logItem.setControl(tv.getControl().getParent());
                    logItem.setText("Pending merges");
            		tv.setInput(new BazaarFileHistory(commitableResources.getAllPendingMerges()));
                } else {
                    super.createMiddleControls(container);
                }
            }
        };
        Composite listComposite = resourceSelectionComposite.createControls(container);
        resourceSelectionComposite.getTableViewer().addFilter(new CommittableFilesFilter());

        resourceSelectionComposite.setResources(this.commitableResources.getAllStatuses());
        if (pendingMerge) {
            List<IBzrResourceStatus> filteredResources =
                new ArrayList<IBzrResourceStatus>(this.commitableResources.getAllStatuses().size());
            for (IBzrResourceStatus status : this.commitableResources.getAllStatuses()) {
                if (!status.isUnknown()) {
                    filteredResources.add(status);
                }
            }
            resourceSelectionComposite.setSelection(filteredResources);
            resourceSelectionComposite.addUntrackedFilesFilter();
        } else {
            resourceSelectionComposite.setSelection(selectedResources.getAllStatuses());
        	resourceSelectionComposite.addUntrackedFilesFilterIfNoUntrackedSelected();
        }
        resourceSelectionComposite.refreshTableViewer();
        resourceSelectionComposite.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				enableOkButton();
			}
		});
        listComposite.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, true));
        makeActions();
        return container;
    }

    private void createAuthorTextBox(Composite parent) {
        final Composite container = parent;//new Composite(parent, SWT.NONE);
        final Label author = new Label(container, SWT.HORIZONTAL);
        author.setText("Author:");
        authorText = new Text(container, SWT.SINGLE | SWT.WRAP | SWT.BORDER);
        authorText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        container.addControlListener(new ControlAdapter() {
            @Override
            public void controlResized(ControlEvent e) {
                Point currSize = authorText.getSize();
                currSize.x = container.getSize().x;
                authorText.setSize(currSize);
            }
        });
    }

    private void createFixesTextBox(Composite parent) {
        final Composite container = parent;//new Composite(parent, SWT.NONE);
        final Label author = new Label(container, SWT.HORIZONTAL);
        author.setText("Fixes:");
        fixesText = new Text(container, SWT.SINGLE | SWT.WRAP | SWT.BORDER);
        fixesText.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
        container.addControlListener(new ControlAdapter() {
            @Override
            public void controlResized(ControlEvent e) {
                Point currSize = fixesText.getSize();
                currSize.x = container.getSize().x;
                fixesText.setSize(currSize);
            }
        });
    }

    private void makeActions() {

        commitText.setCapture(true);
        localCommitButton.setSelection(false); // Start unselected
        boolean localCommitAllowed = false;
        // TODO: for the moment only one branch is supported in commit so we can do this
        for (IBzrResourceStatus commitable : commitableResources.getAllStatuses()) {
            if (!commitable.isDeleted() && commitable.getResource().getBranch().isBound()) {
            	localCommitAllowed = true;
                break;
            }
        }
        GridData data = (GridData) optionGroup.getLayoutData();
        data.exclude = !localCommitAllowed;
        optionGroup.setVisible(localCommitAllowed);
/*
        // set Ctrl + Return listener (fix Bug #128071)
        commitTextBox.addTraverseListener(new TraverseListener() {
            public void keyTraversed(TraverseEvent e) {
                if (e.detail == SWT.TRAVERSE_RETURN && (e.stateMask & SWT.CTRL) != 0) {
                    okPressed();
                }
            }
        });
*/
    }

	private void enableOkButton() {
		String text = commitText.getText(); 
		boolean enabled = text != null && !text.trim().isEmpty();
		enabled &= !resourceSelectionComposite.getSelection().isEmpty() || pendingMerge;
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
		getButton(COMMIT_AND_PUSH_ID).setEnabled(enabled);
	}

	/**
     * Override the OK button pressed to capture the info we want first and then
     * call super.
     */
    @Override
    protected void okPressed() {
        selectedFiles = resourceSelectionComposite.getSelection();
        commitMessage = commitText.getText();
        String author = authorText.getText();
        String fixes = fixesText.getText();
        this.options = new ArrayList<Option>(0);
        if (localCommitButton.getSelection()) {
            options.add(ICommitOptions.LOCAL);
        }
        if (author != null && !author.trim().isEmpty()) {
            options.add(ICommitOptions.AUTHOR.setArgument(author));
        }
        if (fixes != null && !fixes.trim().isEmpty()) {
            options.add(ICommitOptions.FIXES.setArgument(fixes));
        }
        CommitMessageHistory.saveCommitHistory(commitMessage);
        super.okPressed();
    }

/*
    @Override
    public boolean close() {
    	if (super.close()) {
    		CommitMessageHistory.saveCommitHistory(commitText.getText());
    		return true;
    	}
        return false;
    }
*/
    @Override
    protected Point getInitialSize() {
        return new Point(700, 700);
    }

    public boolean isPushAfterCommit() {
        return pushAfterCommit;
    }

    public ArrayList<Option> getOptions() {
        return options;
    }

    public int open(CommitableResources commitableResources,
        CommitableResources selectedResources,
        boolean shouldBlock)
    {
        setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.MAX | SWT.BORDER | SWT.TITLE);
        setBlockOnOpen(shouldBlock);
        this.commitableResources = commitableResources;
        this.selectedResources = selectedResources;
        this.pendingMerge = commitableResources.getAllPendingMerges().size() > 0;
        return open();
    }

	protected void createButtonsForButtonBar(Composite parent) {
		createButton(parent, COMMIT_AND_PUSH_ID, UITexts.CommitDialog_commitAndPush, false);
		createButton(parent, IDialogConstants.OK_ID, UITexts.CommitDialog_commit, true);
		createButton(parent, IDialogConstants.CANCEL_ID, IDialogConstants.CANCEL_LABEL, false);
	}

	protected void buttonPressed(int buttonId) {
		if (buttonId == COMMIT_AND_PUSH_ID) {
			pushAfterCommit = true;
			okPressed();
		} else {
			pushAfterCommit = false;
			super.buttonPressed(buttonId);
		}
	}
	
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		CommitDialog dialog = new CommitDialog(shell);
		int exit = dialog.open(new CommitableResources(), new CommitableResources(), true);
		System.out.println("exit: " + exit);
	}
}
