/*******************************************************************************
 * Copyright (C) 2012, Kevin Sawicki <kevin@github.com>
 * and other copyright owners as documented in the project's IP log.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *    Piotr Piastucki
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.io.IOException;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedHashSet;
import java.util.Set;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.WorkbenchException;
import org.eclipse.ui.XMLMemento;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

public class CommitMessageHistory {
	
	private static final String KEY_MESSAGE = "message"; //$NON-NLS-1$

	private static final String KEY_MESSAGES = "messages"; //$NON-NLS-1$

	public static Set<String> getCommitHistory() {
		return getMessages(BazaarPreferenceConstants.COMMIT_DIALOG_HISTORY_MESSAGES);
	}

	public static Set<String> getTemplates() {
		return getMessages(BazaarPreferenceConstants.COMMIT_DIALOG_TEMPLATES);
	}
	
	/**
	 * @return saved commit messages
	 */
	public static Set<String> getMessages(String preferenceKey) {
		String all = getPreferenceStore().getString(preferenceKey);
		if (all.length() == 0) {
			return Collections.emptySet();
		}
		int max = getCommitHistorySize();
		if (max < 1) {
			return Collections.emptySet();
		}
		XMLMemento memento;
		try {
			memento = XMLMemento.createReadRoot(new StringReader(all));
		} catch (WorkbenchException e) {
			return Collections.emptySet();
		}
		Set<String> messages = new LinkedHashSet<String>();
		for (IMemento child : memento.getChildren(KEY_MESSAGE)) {
			messages.add(child.getTextData());
			if (messages.size() == max) {
				break;
			}
		}
		return messages;
	}

	/**
	 * Save a new commit message.
	 *
	 * @param message
	 */
	public static void saveCommitHistory(String message) {
		if (message == null || message.trim().isEmpty()) {
			return;
		}
		int size = getCommitHistorySize();
		ArrayList<String> messages = new ArrayList<String>();
		if (size > 0) {
			messages.add(message);
			if (messages.size() < size) {
				Set<String> history = getCommitHistory();
				history.remove(message);
				for (String previous : history) {
					messages.add(previous);
					if (messages.size() == size)
						break;
				}
			}
		}
		saveMessages(messages, BazaarPreferenceConstants.COMMIT_DIALOG_HISTORY_MESSAGES);
	}

	public static void saveTemplates(Collection<String> messages) {
		saveMessages(messages, BazaarPreferenceConstants.COMMIT_DIALOG_TEMPLATES);
	}

	private static void saveMessages(Collection<String> messages, String preferenceKey) {
		XMLMemento memento = XMLMemento.createWriteRoot(KEY_MESSAGES);
		for (String m : messages) {
			memento.createChild(KEY_MESSAGE).putTextData(m);
		}
		StringWriter writer = new StringWriter();
		try {
			memento.save(writer);
			getPreferenceStore().setValue(preferenceKey, writer.toString());
		} catch (IOException e) {
		}
	}

	private static IPreferenceStore getPreferenceStore() {
		return EclipseBazaarUI.getDefault().getPreferenceStore();
	}

	private static int getCommitHistorySize() {
		return getPreferenceStore().getInt(BazaarPreferenceConstants.COMMIT_DIALOG_HISTORY_SIZE);
	}

}
