/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Shell;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.IRevertOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;
import org.vcs.bazaar.eclipse.ui.widgets.RevisionNumber;

/**
 *
 * @author Guillermo Gonzalez <guillo.gonzo AT gmail.com>
 * @BasedOn RevertDialog from MercurialEclipse
 */
public class RevertDialog extends Dialog {

	private Button forgetMerges, noBackup, wholeBranch;
	private RevisionNumber revision;
	private ArrayList<Option> options;
	private BzrResourceSelectionComposite resourcesComposite;
	private List<IBzrResourceStatus> selection;
	private boolean isWholeBranch;
	private boolean wholeBranchEnabled;

	/**
	 * Create the dialog
	 *
	 * @param parentShell
	 * @param selectedResources
	 * @param commiteableResources
	 */
	public RevertDialog(Shell parentShell, List<IBzrResourceStatus> resources, List<IBzrResourceStatus> selectedResources) {
		super(parentShell);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		resourcesComposite = getBzrResourceSelectionComposite();
		resourcesComposite.setSelection(selectedResources);
		resourcesComposite.setResources(resources);
	}

	private BzrResourceSelectionComposite getBzrResourceSelectionComposite() {
		return new BzrResourceSelectionComposite() {

			@Override
            protected String getUpperLabel() {
				return "Checked resources will be reverted:";
			}

			@Override
			protected void createOption(Composite optionComposite) {
				if(wholeBranchEnabled) {
					wholeBranch = new Button(optionComposite, SWT.CHECK);
					wholeBranch.setText("Revert the whole branch");
				}
				Group optionGroup = new Group(optionComposite, SWT.NONE);
				optionGroup.setText(UITexts.Dialog_optionsGroup);
				optionGroup.setLayout(new GridLayout(2, false));

				revision = new RevisionNumber(optionGroup);
				revision.showLogButton(false); // diable show log button

				Composite checkboxContainer = new Composite(optionGroup, SWT.NONE);
				checkboxContainer.setLayout(new GridLayout());
				forgetMerges = new Button(checkboxContainer, SWT.CHECK);
				forgetMerges.setText("Forget merges");
				noBackup = new Button(checkboxContainer, SWT.CHECK);
				noBackup.setText("No backup");
				GridData data;
		        data = new GridData();
		        data.horizontalAlignment = GridData.FILL;
		        data.verticalAlignment = GridData.CENTER;
		        data.grabExcessHorizontalSpace = true;
		        data.grabExcessVerticalSpace = false;
		        optionGroup.setLayoutData(data);
				checkboxContainer.setLayoutData(data);

				revision.addShowLogButtonSelectionListener(new SelectionAdapter() {
					@Override
					public void widgetSelected(SelectionEvent e) {
						showLog();
					}
				});
			}
		};
	}


	protected void showLog() {
		IBzrBranch branch = this.resourcesComposite.getResources().get(0).getResource().getBranch();
		if(branch != null) {
			ShowLogDialog dialog = new ShowLogDialog(getShell(), branch);
			if(dialog.open() == Dialog.OK && dialog.getSelectedRevision() != null) {
				revision.setRevision(dialog.getSelectedRevision().getValue());
			}
		}
	}

	@Override
	public void create() {
		super.create();
		enableOkButton();
	}

	/**
	 * Create contents of the dialog
	 *
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout());
		Composite control = resourcesComposite.createControls(container);
		control.setLayoutData(new GridData(GridData.FILL_BOTH));
		resourcesComposite.refreshTableViewer();
		resourcesComposite.addUntrackedFilesFilter();
		resourcesComposite.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				enableOkButton();
			}
		});
		makeActions();
		return container;
	}

	private void makeActions() {
		if(wholeBranchEnabled) {
			wholeBranch.setSelection(false);
			wholeBranch.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					resourcesComposite.getTableViewer().getTable().setEnabled(!wholeBranch.getSelection() && !forgetMerges.getSelection());
					enableOkButton();
				}
			});
		}
		forgetMerges.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				boolean enabled = !forgetMerges.getSelection();
				if (wholeBranchEnabled) {
					enabled &= !wholeBranch.getSelection();
				}
				resourcesComposite.getTableViewer().getTable().setEnabled(enabled);
				enableOkButton();
			}
		});
	}

	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(700, 500);
	}

	public Option[] getOptions() {
		return options.toArray(new Option[options.size()]);
	}

	private void enableOkButton() {
		boolean enabled = !resourcesComposite.getSelection().isEmpty();
		enabled |= wholeBranchEnabled ? wholeBranch.getSelection() : false;
		enabled |= forgetMerges.getSelection();
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
	}

	@Override
	protected void okPressed() {
		selection = resourcesComposite.getSelection();
		this.options = new ArrayList<Option>(0);
		if(forgetMerges.getSelection()) {
			options.add(IRevertOptions.FORGET_MERGES);
		}
		if(noBackup.getSelection()) {
			options.add(IRevertOptions.NO_BACKUP);
		}
		if(revision.isRevisionSelected()) {
			options.add(IRevertOptions.REVISION.setArgument(revision.getRevision()));
		}
		isWholeBranch = wholeBranchEnabled?wholeBranch.getSelection():false;
		isWholeBranch |= forgetMerges.getSelection();
		super.okPressed();

	}

	/**
	 *
	 * @return List<IBzrResourceStatus> of the selected resources
	 */
	public List<IBzrResourceStatus> getSelection() {
		return selection;
	}

	public boolean isWholeBranch() {
		return isWholeBranch;
	}

	public void setWholeBranchEnabled(boolean enable) {
		this.wholeBranchEnabled = enable;
	}

	// this main is for test the dialog during development
	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		RevertDialog dialog = new RevertDialog(shell, new ArrayList<IBzrResourceStatus>(0), new ArrayList<IBzrResourceStatus>(0));
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
