/*******************************************************************************
* Copyright (c) 2007-2013 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.client.commandline.commands.options.Option;
import org.vcs.bazaar.client.commandline.syntax.ISendOptions;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 *
 * @author Javier Der Derian <javierder AT gmail.com>
 * @BasedOn RevertDialog from BzrEclipse
 */
public class SendDialog extends LocationInputDialog {

	private static final String TITLE = UITexts.BazaarSendDialogTitle;
	private static final String MESSAGE = UITexts.BazaarSendDialogMessage;
	private ArrayList<Option> options;
	private Button nobundle;
	private Button nopatch;
	private Button send;
	private Button save;

	private SendDialogInfo dialogInfo;
	private Text submitLocationText;
	private Text messageText;
	private Text formatText;

	/**
	 * Create the dialog
	 *
	 * @param parentShell
	 * @param selectedResources
	 * @param commiteableResources
	 */
	public SendDialog(Shell parentShell, Collection<IBzrBranch> locations, IBzrBranch initialBranch, SendDialogInfo dialogInfo) {
		super(parentShell, TITLE, MESSAGE, initialBranch, new BaseURIValidator(), locations, dialogInfo, true, false, false, false, false);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
		this.dialogInfo = dialogInfo;
	}

	@Override
    protected void createOptionsArea(Composite composite) {
		setTitle(TITLE);

		Group typeBoxGroup = new Group(composite, SWT.NONE);
		typeBoxGroup.setText(UITexts.SendDialog_EnterLocation);
		typeBoxGroup.setLayout(new GridLayout());
		typeBoxGroup.setLayoutData(new GridData(GridData.FILL, GridData.BEGINNING, true, false));
		
		Composite typeBoxRadioArea = new Composite(typeBoxGroup, SWT.NONE);
		GridData typeBoxGrid = new GridData();

		typeBoxGrid.horizontalAlignment = SWT.FILL;
		typeBoxRadioArea.setLayout(new GridLayout(2, true));
		typeBoxRadioArea.setLayoutData(typeBoxGrid);

		send = new Button(typeBoxRadioArea, SWT.RADIO);
		send.setText(UITexts.SendDialog_SendEmail);
		send.setSelection(true); // Start selected.
		send.setToolTipText(UITexts.SendDialog_SendEmailTooltip);
		save = new Button(typeBoxRadioArea, SWT.RADIO);
		save.setText(UITexts.SendDialog_SaveLocation);
		save.setSelection(false); // Start unselected.
		save.setToolTipText(UITexts.SendDialog_SaveLocationTooltip);

		final Composite submitComosite = new Composite(typeBoxGroup,SWT.None);
		submitComosite.setLayout(new GridLayout(2, false));

		GridData submitLocationGrid = new GridData();

		submitLocationGrid.horizontalAlignment = SWT.FILL;
		submitLocationGrid.widthHint = 440;
		submitLocationText = new Text(submitComosite, SWT.BORDER | SWT.SINGLE);
		submitLocationText.setLayoutData(submitLocationGrid);

		submitLocationText.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if(send.getSelection()) {
					validateInput();
				}
			}
		});

		GridData locationBoxGrid = new GridData();
		locationBoxGrid.horizontalAlignment = SWT.FILL;
		submitComosite.setLayoutData(locationBoxGrid);

		final Button browseButton = new Button(submitComosite, SWT.PUSH);
		browseButton.setText(UITexts.BazaarSendDialogBrowse);
		browseButton.addSelectionListener(new SelectionAdapter() {
			@Override
            public void widgetSelected(SelectionEvent e) {
				final FileDialog dialog = new FileDialog(getShell(), SWT.SAVE);
				dialog.setText("Save As");
				final String dir = dialog.open();
				if (dir != null){
					submitLocationText.setText(dir);
				}
			}
		});
		browseButton.setEnabled(false);
		save.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				browseButton.setEnabled(true);
				validateInput();
				super.widgetSelected(e);
			}
		});
		send.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				browseButton.setEnabled(false);
				validateInput();
				super.widgetSelected(e);
			}
		});

		final Label label2 = new Label(composite, SWT.NONE);
		label2.setText("Message");

		GridData messageGrid = new GridData();

		messageGrid.horizontalAlignment = SWT.FILL;
		messageText = new Text(composite, SWT.BORDER | SWT.SINGLE);
		messageText.setLayoutData(messageGrid);

		Composite container = new Composite(composite, SWT.NONE);
		container.setLayout(new GridLayout(2, false));
		container.setLayoutData(new GridData(GridData.BEGINNING, GridData.BEGINNING, true, false));

		Group checkBoxGroup = new Group(container, SWT.NONE);
		checkBoxGroup.setLayout(new GridLayout(2, false));
		super.createOptions(checkBoxGroup);
		nobundle = new Button(checkBoxGroup, SWT.CHECK);
		nobundle.setText(UITexts.SendDialog_NoBundle);
		nobundle.setSelection(false); // Start unselected.
		nobundle.setToolTipText(UITexts.SendDialog_NoBundleTooltip);
		nopatch = new Button(checkBoxGroup, SWT.CHECK);
		nopatch.setText(UITexts.SendDialog_NoPatch);
		nopatch.setSelection(false); // Start unselected.
		nopatch.setToolTipText(UITexts.SendDialog_NoPatchTooltip);

		Composite formatBoxGroup = new Composite(checkBoxGroup, SWT.NONE);

		GridData formatBoxGrid = new GridData();

		formatBoxGrid.horizontalAlignment = SWT.FILL;
		formatBoxGroup.setLayout(new GridLayout(2, false));
		formatBoxGroup.setLayoutData(formatBoxGrid);

		final Label label3 = new Label(formatBoxGroup, SWT.NONE);
		label3.setText(UITexts.SendDialog_Format);
		formatText = new Text(formatBoxGroup, SWT.BORDER | SWT.SINGLE);

		Group revisionGroup = new Group(container, SWT.NONE);
		revisionGroup.setLayout(new GridLayout());
		createRevisionSelector(revisionGroup, false);

		revisionGroup.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
		checkBoxGroup.setLayoutData(new GridData(GridData.FILL, GridData.FILL, true, false));
		container.setLayoutData(new GridData(GridData.FILL_BOTH));
	}


    @Override
    public void setErrorMessage(String errorMessage) {
    	if (errorMessage == null) {
	    	if (send != null && !send.isDisposed() && send.getSelection()) {
				Pattern p = Pattern.compile(".+@.+\\.[a-z]+");
				Matcher m = p.matcher(getPath());
				boolean matchFound = m.matches();
				if (!matchFound) {
					errorMessage = UITexts.SendDialog_WrongEmail;
				}
	    	}
    	}
		super.setErrorMessage(errorMessage);
    }
	
	public String getPath()	{
		return submitLocationText.getText();
	}

	@Override
	public String getMessage() {
		return messageText.getText();
	}

	public String getFormat() {
		return formatText.getText();
	}


	public Integer getType(){
		// 0 means send by email, 1 means save in file
		if(send.getSelection()) {
            return 0;
        } else {
            return 1;
        }
	}

	protected Boolean isNoBundle() {
		return nobundle.getSelection();
	}

	protected Boolean isNoPatch() {
		return nopatch.getSelection();
	}

	public Option[] getOptions() {
		return options.toArray(new Option[options.size()]);

	}

	@Override
	protected void okPressed() {
		this.options = new ArrayList<Option>(0);

		if(getPath().length() > 0 && getValue().length() > 0 ) {
			if(getType() == 0){
				options.add(ISendOptions.MAIL_TO.setArgument(getPath()));
			}
			else {
				options.add(ISendOptions.OUTPUT.setArgument(getPath()));
			}
		}
		else {
			//FIXME: give some kind of message
			return;
		}

		options.add(ISendOptions.MESSAGE.setArgument(getMessage()));

		if(isRemember()) {
			options.add(ISendOptions.REMEMBER);
		}

		if(isNoBundle()) {
			options.add(ISendOptions.NO_BUNDLE);
		}

		if(isNoPatch()) {
			options.add(ISendOptions.NO_PATCH);
		}
		if(isRevisionSelected()) {
			options.add(ISendOptions.REVISION.setArgument(getSelectedRevision()));
		}

		if(getFormat().length() > 0) {
			options.add(ISendOptions.FORMAT.setArgument(getFormat()));
		}

		this.dialogInfo.options = options;
		super.okPressed();
	}

	public static class SendDialogInfo extends LocationDialogInfo {
		List<Option> options;

		public List<Option> getOptions() {
			return options;
		}
	}

	// this main is for test the dialog during development
    @SuppressWarnings("unchecked")
    public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		SendDialog dialog = new SendDialog(shell, Collections.EMPTY_LIST, null, new SendDialogInfo());
		dialog.setBlockOnOpen(true);
		int exit = dialog.open();
		System.out.println("exit: " + exit);
	}
}
