/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.GridLayoutFactory;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;

/**
 * @author Piotr Piastucki
 */
public class ShelveDialog extends Dialog {

	private BzrResourceSelectionComposite resourcesComposite;
	private Button wholeBranch;
	private Text messageTextBox;
	private Label messageTextLabel;
	private boolean wholeBranchEnabled = true;
	private List<IBzrResourceStatus> shelveableResources;
	private List<IBzrResourceStatus> selectedResources;
	private List<IBzrResourceStatus> selectedFiles;
	private String message;
	private boolean isWholeBranch;

	public ShelveDialog(Shell parentShell) {
		super(parentShell);
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(true);
	}

	@Override
	public void create() {
		super.create();
		enableOkButton();
	}
	
	@Override
	protected Control createDialogArea(Composite parent) {
		getShell().setText(UITexts.ShelveDialog_title);
		Composite container = (Composite) super.createDialogArea(parent);
		container.setLayout(new GridLayout(1, false));
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite messageArea = new Composite(container, SWT.NONE);
		GridLayoutFactory.swtDefaults().applyTo(messageArea);
		GridDataFactory.fillDefaults().grab(true, false).applyTo(messageArea);

		messageTextLabel = new Label(messageArea, SWT.NONE);
		messageTextLabel.setText(UITexts.ShelveDialog_messageLabel);

		messageTextBox = new Text(messageArea, SWT.V_SCROLL | SWT.MULTI | SWT.BORDER | SWT.WRAP);
		GridData textBoxData = new GridData(GridData.FILL_HORIZONTAL);
		textBoxData.heightHint = 100;
		messageTextBox.setLayoutData(textBoxData);

		resourcesComposite = getBzrResourceSelectionComposite();
		Composite control = resourcesComposite.createControls(container);
		control.setLayoutData(new GridData(GridData.FILL_BOTH));

		resourcesComposite.setResources(shelveableResources);
		resourcesComposite.setSelection(selectedResources);
		resourcesComposite.addUntrackedFilesFilterIfNoUntrackedSelected();
		resourcesComposite.refreshTableViewer();
		resourcesComposite.addCheckStateListener(new ICheckStateListener() {
			public void checkStateChanged(CheckStateChangedEvent event) {
				enableOkButton();
			}
		});
		makeActions();
		return container;
	}

	private void makeActions() {
		if (wholeBranchEnabled) {
			wholeBranch.setSelection(false);
			wholeBranch.addSelectionListener(new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					resourcesComposite.getTableViewer().getTable().setEnabled(!wholeBranch.getSelection());
					enableOkButton();
				}
			});
		}
	}

	public boolean isWholeBranch() {
		return isWholeBranch;
	}

	public String getMessage() {
		return message;
	}

	public void setWholeBranchEnabled(boolean enable) {
		this.wholeBranchEnabled = enable;
	}

	private void enableOkButton() {
		boolean enabled = !resourcesComposite.getSelection().isEmpty();
		enabled |= wholeBranchEnabled ? wholeBranch.getSelection() : false;
		getButton(IDialogConstants.OK_ID).setEnabled(enabled);
	}

	@Override
	protected void okPressed() {
		selectedFiles = resourcesComposite.getSelection();
		isWholeBranch = wholeBranchEnabled ? wholeBranch.getSelection() : false;
		message = messageTextBox.getText();
		super.okPressed();
	}

	@Override
	protected Point getInitialSize() {
		return new Point(700, 500);
	}

	private BzrResourceSelectionComposite getBzrResourceSelectionComposite() {
		return new BzrResourceSelectionComposite(true, true) {
			@Override
			protected String getUpperLabel() {
				return UITexts.ShelveDialog_resourceLabel;
			}

			@Override
			protected void createOption(Composite optionComposite) {
				if (wholeBranchEnabled) {
					wholeBranch = new Button(optionComposite, SWT.CHECK);
					wholeBranch.setText(UITexts.ShelveDialog_wholeBranch);
				}
			}
		};
	}

	public List<IBzrResourceStatus> getSelectedFiles() {
		return selectedFiles;
	}

	public int open(List<IBzrResourceStatus> shelveableResources, List<IBzrResourceStatus> selectedResources,
			boolean shouldBlock) {
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.MAX | SWT.BORDER | SWT.TITLE);
		setBlockOnOpen(shouldBlock);
		this.shelveableResources = shelveableResources;
		this.selectedResources = selectedResources;
		return open();
	}

	public static void main(String[] args) {
		Display display = new Display();
		Shell shell = new Shell(display);
		shell.setMinimumSize(280, 400);
		ShelveDialog dialog = new ShelveDialog(shell);
		int exit = dialog.open(new ArrayList<IBzrResourceStatus>(0), new ArrayList<IBzrResourceStatus>(0), true);
		System.out.println("exit: " + exit);
	}

}
