/*******************************************************************************
* Copyright (c) 2007-2013 Guillermo Gonzalez
* Copyright (c) 2013 Piotr Piastucki
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.ui.dialogs;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.jface.dialogs.TrayDialog;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.client.BazaarRevisionRange;
import org.vcs.bazaar.eclipse.core.history.BazaarBranchHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileRevision;
import org.vcs.bazaar.eclipse.core.model.IBzrBranch;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.history.HistoryTreeViewer;

public class ShowLogDialog extends TrayDialog {
	private TreeViewer viewer;
	private IBzrBranch branch;
	private IResource resource;
	private RefreshLog refreshLogJob;
	private BazaarRevisionRange selectedRevisions;
	private boolean range;

	private ShowLogDialog(Shell shell, boolean range) {
		super(shell);
		this.range = range;
		setShellStyle(SWT.RESIZE | SWT.MODELESS | SWT.CLOSE | SWT.BORDER | SWT.TITLE);
	}
	
	public ShowLogDialog(Shell shell, IBzrBranch branch, boolean range) {
		this(shell, range);
		this.branch = branch;
	}

	public ShowLogDialog(Shell shell, IBzrBranch branch) {
		this(shell, false);
		this.branch = branch;
	}

	public ShowLogDialog(Shell shell, IResource resource) {
		this(shell, false);
		this.resource = resource;
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite  = (Composite)super.createDialogArea(parent);
		viewer = new HistoryTreeViewer(composite, (range ? SWT.MULTI : 0) | SWT.H_SCROLL | SWT.V_SCROLL);
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				okPressed();
			}
		});
		refresh();
		return composite;
	}


	/**
	 * Return the initial size of the dialog
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(700, 400);
	}

	public BazaarRevision getSelectedRevision() {
		return selectedRevisions != null ? selectedRevisions.getStart() : null;
	}

	public String getSelectedRevisionAsString() {
		if (selectedRevisions != null) {
			if (selectedRevisions.getStart().equals(selectedRevisions.getEnd())) {
				return selectedRevisions.getStart().getValue();
			} else {
				return String.format("%1$s..%2$s", selectedRevisions.getStart().getValue(), selectedRevisions.getEnd().getValue());
			}
		}
		return null;
	}

	@Override
	protected void okPressed() {
		IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
		if (selection.size() > 0) {
			Object[] selected = selection.toArray();
			BazaarFileRevision rev1 = (BazaarFileRevision) selected[0];
			BazaarFileRevision rev2 = (BazaarFileRevision) selected[selected.length -1];
			selectedRevisions = BazaarRevisionRange.getRange(rev2.getRevision(), rev1.getRevision());
		}
		super.okPressed();
	}

	public void refresh() {
		if (this.branch != null || this.resource != null) {
			if (refreshLogJob == null) {
				refreshLogJob = new RefreshLog();
			}
			if (refreshLogJob.getState() != Job.NONE) {
				refreshLogJob.cancel();
			}
			refreshLogJob.setResource(this.branch);
			refreshLogJob.schedule();
		}
	}

	private class RefreshLog extends Job {
		private IBzrBranch branch;

		public RefreshLog() {
			super("Fetching Bazaar revisions..."); //$NON-NLS-1$
			this.setUser(true);
			this.setPriority(UIJob.LONG);
		}

		public void setResource(IBzrBranch branch) {
			this.branch = branch;
		}

		@Override
        public IStatus run(IProgressMonitor monitor) {
			IStatus status = Status.OK_STATUS;
			if (this.branch != null || resource != null) {
				monitor.beginTask("", 1000);
				try {
					final BazaarFileHistory branchHistory = branch != null ? new BazaarBranchHistory(branch) : new BazaarFileHistory(resource);
					branchHistory.refresh(monitor, true);
					if(!monitor.isCanceled()) {
						ShowLogDialog.this.getShell().getDisplay().asyncExec(new Runnable() {
							public void run() {
								viewer.setInput(branchHistory);
							}
						});
					}
					monitor.worked(400);
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
					status = e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(BazaarException.wrapException(e));
					return UIJob.errorStatus(e);
				} finally {
					monitor.done();
				}
			}
			return status;
		}
	}

	@Override
	public boolean close() {
		if (refreshLogJob != null && refreshLogJob.getState() != Job.NONE) {
			refreshLogJob.cancel();
		}
		return super.close();
	}

}
