/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.history;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.jface.viewers.IOpenListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.ITreeSelection;
import org.eclipse.jface.viewers.OpenEvent;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.team.core.history.IFileHistory;
import org.eclipse.team.core.history.IFileRevision;
import org.eclipse.team.ui.history.HistoryPage;
import org.eclipse.team.ui.history.IHistoryPageSite;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.part.IPageSite;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.BazaarRevision;
import org.vcs.bazaar.eclipse.core.history.BazaarBranchHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.history.BazaarFileRevision;
import org.vcs.bazaar.eclipse.core.model.local.IBzrLocalResource;
import org.vcs.bazaar.eclipse.core.utils.BzrTeamStatus;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UIIcons;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.actions.AbstractCompareAction;
import org.vcs.bazaar.eclipse.ui.actions.OpenBazaarRevisionAction;
import org.vcs.bazaar.eclipse.ui.compare.CompareUtils;
import org.vcs.bazaar.eclipse.ui.preferences.BazaarPreferenceConstants;

/**
 * 
 * @author Guillermo Gonzalez
 * @author Piotr Piastucki (affected paths viewer)
 * @basedOn MercurialHistoryPage by zingo
 * 
 */
public class BazaarHistoryPage extends HistoryPage {

	private BazaarHistoryPageInput input;
	private TreeViewer viewer;
	private SashForm control;
	private LogTreeContentProvider logTreeContentProvider;
	private BazaarFileHistory bazaarFileHistory;
	private AffectedPathsViewer pathsViewer;
	IFileRevision[] entries;
	private final IPreferenceStore store;
	private boolean showPaths;
	private RefreshBazaarHistory refreshFileHistoryJob;
	private BazaarRevision selectedRevision;

	private class RefreshBazaarHistory extends Job {
		BazaarFileHistory bazaarFileHistory;

		public RefreshBazaarHistory() {
			super("Fetching Bazaar revisions..."); //$NON-NLS-1$
			this.setUser(true);
			this.setPriority(UIJob.LONG);
		}

		public void setFileHistory(BazaarFileHistory bazaarFileHistory) {
			this.bazaarFileHistory = bazaarFileHistory;
		}

		@Override
		public IStatus run(IProgressMonitor monitor) {
			IStatus status = Status.OK_STATUS;
			if (bazaarFileHistory != null) {
				monitor.beginTask(getName(), 1000);
				try {
					bazaarFileHistory.refresh(monitor, true);
					if (viewer.getContentProvider() == null) {
						setContentProvider();
					}
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
					status = e.getStatus();
				} catch (Exception e) {
					EclipseBazaarUI.log(BazaarException.wrapException(e));
					status = new BzrTeamStatus(BzrTeamStatus.ERROR, e.getMessage(), e);
				} finally {
					monitor.done();
				}
			}
			return status;
		}

	}

	class LogTreeContentProvider implements ITreeContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
			entries = null;
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			if (entries != null) {
				return entries;
			}

			final IFileHistory fileHistory = (IFileHistory) parent;
			entries = fileHistory.getFileRevisions();

			return entries;
		}

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof BazaarFileRevision) {
				return ((BazaarFileRevision) parentElement).getChildren();
			}
			return null;
		}

		public Object getParent(Object element) {
			if (element instanceof BazaarFileRevision && ((BazaarFileRevision) element).hasParent()) {
				return ((BazaarFileRevision) element).getParent();
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof BazaarFileRevision) {
				BazaarFileRevision revision = (BazaarFileRevision) element;
				return revision.hasChildren();
			}
			return false;
		}

	}

	public BazaarHistoryPage(BazaarHistoryPageInput input) {
		super();
		this.input = input;
		store = EclipseBazaarUI.getDefault().getPreferenceStore();
		showPaths = store.getBoolean(BazaarPreferenceConstants.HISTORY_SHOW_AFFETCED_PATHS);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.HistoryPage#inputSet()
	 */
	@Override
	public boolean inputSet() {
		if (isValidInput(getInput())) {
			IResource resource = getIResource();
			if (resource != null) {
				bazaarFileHistory = new BazaarFileHistory(resource);
			} else {
				bazaarFileHistory = new BazaarBranchHistory(input.getResource().getBranch());
			}
			refresh();
			return true;
		}
		return false;
	}

	private IResource getIResource() {
		if (input.getResource() instanceof IBzrLocalResource) {
			return ((IBzrLocalResource) input.getResource()).getIResource();
		}
		return null;
	}

	@Override
	public void createControl(Composite parent) {
		control = new SashForm(parent, SWT.VERTICAL);
		control.setLayoutData(new GridData(GridData.FILL_BOTH));
		viewer = new HistoryTreeViewer(control);
		pathsViewer = new AffectedPathsViewer(control, getSite(), input);
		contributeActions();
		layout();
	}

	private void setContentProvider() {
		if (logTreeContentProvider == null) {
			logTreeContentProvider = new LogTreeContentProvider();
		}
		viewer.setContentProvider(logTreeContentProvider);
	}

	private void contributeActions() {
		final OpenBazaarRevisionAction openAction = new OpenBazaarRevisionAction(UITexts.ActionOpenInEditor_label,
				getSite()) {
			@Override
			protected boolean shouldShow() {
				return getIResource() instanceof IFile;
			}
		};
		final AbstractCompareAction compareWithCurrentAction = new AbstractCompareAction(
				UITexts.ActionCompareWithCurrent_label) {
			@Override
			public void internalRun() {
				try {
					IResource resource = getIResource();
					if (resource instanceof IFile) {
						CompareUtils.openSaveableEditor(resource, getStorage(0), null);
					} else if (resource instanceof IResource) {
						openEditor(resource, getRevision(0).getRevision());
					} else {
						openEditor(input.getResource().getBranch(), getRevision(0).getRevision());
					}
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
				}
			}

			@Override
			protected boolean shouldShow(IStructuredSelection selection) {
				return selection.size() == 1;
			}
		};
		final AbstractCompareAction compareWithEachOther = new AbstractCompareAction(
				UITexts.ActionCompareWithEachOther_label) {
			@Override
			public void internalRun() {
				try {
					IResource resource = getIResource();
					if (resource instanceof IFile) {
						CompareUtils.openEditor(resource, getStorage(0), getStorage(1));
					} else if (resource instanceof IResource) {
						openEditor(resource, getRevision(1).getRevision(), getRevision(0).getRevision());
					} else {
						openEditor(input.getResource().getBranch(), getRevision(0).getRevision(), getRevision(1).getRevision());
					}
				} catch (CoreException e) {
					EclipseBazaarUI.log(e);
				}
			}

			@Override
			public boolean shouldShow(IStructuredSelection selection) {
				return selection.size() == 2;
			}

		};
		viewer.getTree().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				openAction.selectionChanged(selection);
				compareWithCurrentAction.selectionChanged(selection);
				compareWithEachOther.selectionChanged(selection);
				if (selection.size() == 1) {
					BazaarFileRevision rev = (BazaarFileRevision) selection.getFirstElement();
					selectedRevision = rev.getRevision();
				} else {
					selectedRevision = null;
				}
				if (showPaths) {
					pathsViewer.setRevision(selectedRevision);
				}
			}
		});
		viewer.addOpenListener(new IOpenListener() {
			public void open(OpenEvent event) {
				if (getIResource() instanceof IFile) {
					IStructuredSelection selection = (IStructuredSelection) event.getSelection();
					openAction.selectionChanged(selection);
					openAction.run();
				}
			}
		});
		if (input.getResource().getBranch().isProjectBranch()) {
			// Contribute actions to popup menu
			MenuManager menuMgr = new MenuManager();
			Menu menu = menuMgr.createContextMenu(viewer.getTree());
			menuMgr.addMenuListener(new IMenuListener() {
				public void menuAboutToShow(IMenuManager menuMgr) {
					menuMgr.add(new Separator(IWorkbenchActionConstants.GROUP_FILE));
					menuMgr.add(openAction);
					menuMgr.add(compareWithCurrentAction);
					menuMgr.add(compareWithEachOther);
				}
			});
			menuMgr.setRemoveAllWhenShown(true);
			viewer.getTree().setMenu(menu);
		}
		contributeToActionBars();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#getControl()
	 */
	@Override
	public Control getControl() {
		return control;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.Page#setFocus()
	 */
	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#getDescription()
	 */
	public String getDescription() {
		return getName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#getName()
	 */
	public String getName() {
		IResource resource = getIResource();
		if (resource != null) {
			return resource.getFullPath().toOSString();
		}
		return input.getResource().getName();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.team.ui.history.IHistoryPage#isValidInput(java.lang.Object)
	 */
	public boolean isValidInput(Object object) {
		if (object instanceof BazaarHistoryPageInput) {
			return true;
		}
		return false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.team.ui.history.IHistoryPage#refresh()
	 */
	public void refresh() {
		if (isValidInput(getInput()) && getControl() != null) {
			if (refreshFileHistoryJob == null) {
				refreshFileHistoryJob = new RefreshBazaarHistory();
				refreshFileHistoryJob.addJobChangeListener(new JobChangeAdapter() {
					@Override
					public void done(IJobChangeEvent event) {
						if (event.getResult().isOK()) {
							BazaarHistoryPage.this.getSite().getShell().getDisplay().asyncExec(new Runnable() {
								public void run() {
									viewer.setInput(bazaarFileHistory);
								}
							});
						}
					}
				});
			}

			boolean canceled = true;
			if (refreshFileHistoryJob.getState() != Job.NONE) {
				canceled = refreshFileHistoryJob.cancel();
			}
			if (canceled) {
				refreshFileHistoryJob.setFileHistory(bazaarFileHistory);
				refreshFileHistoryJob.setRule(getIResource() != null ? getIResource().getProject() : ResourcesPlugin
						.getWorkspace().getRoot());
				refreshFileHistoryJob.schedule();
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.core.runtime.IAdaptable#getAdapter(java.lang.Class)
	 */
	@SuppressWarnings("unchecked")
	public Object getAdapter(Class adapter) {
		return null;
	}

	/**
	 * Select the revision in the receiver.
	 */
	public void selectRevision(BazaarRevision revision) {
		if (entries == null) {
			return;
		}

		BazaarFileRevision entry = findIFileRevisionBy(entries, revision);
		if (entry != null) {
			ITreeSelection selection = new TreeSelection(createTreePath(entry));
			viewer.expandToLevel(selection.getPaths(), 0);
			viewer.setSelection(selection, true);
			selectedRevision = revision;
			if (showPaths) {
				pathsViewer.setRevision(selectedRevision);
			}
		}

	}

	private BazaarFileRevision findIFileRevisionBy(IFileRevision[] entries, BazaarRevision revision) {
		for (IFileRevision aEntry : entries) {
			if (((BazaarFileRevision) aEntry).getRevision().equals(revision)) {
				return (BazaarFileRevision) aEntry;
			} else if (logTreeContentProvider.hasChildren(aEntry)) {
				BazaarFileRevision result = findIFileRevisionBy(
						(BazaarFileRevision[]) logTreeContentProvider.getChildren(aEntry), revision);
				if (result != null) {
					return result;
				}
			}
		}
		return null;
	}

	private TreePath createTreePath(BazaarFileRevision entry) {
		List<BazaarFileRevision> list = new ArrayList<BazaarFileRevision>();
		BazaarFileRevision current = entry;
		list.add(0, current);
		while (current.getParent() != null) {
			list.add(0, current.getParent());
			current = current.getParent();
		}
		return new TreePath(list.toArray());
	}

	private void layout() {
		if (showPaths) {
			control.setMaximizedControl(null);
		} else {
			control.setMaximizedControl(viewer.getControl().getParent());
		}
	}

	@Override
	public void dispose() {
		if (refreshFileHistoryJob != null && refreshFileHistoryJob.getState() != Job.NONE) {
			refreshFileHistoryJob.cancel();
		}
		super.dispose();
	}

	private void contributeToActionBars() {
		IHistoryPageSite parentSite = getHistoryPageSite();
		IPageSite pageSite = parentSite.getWorkbenchPageSite();
		IActionBars actionBars = pageSite.getActionBars();
		Action toggleShowAffectedPathsAction = new Action(UITexts.HistoryView_ShowAffectedPaths, UIIcons.AFFECTED_PATHS) {
			@Override
			public void run() {
				showPaths = isChecked();
				store.setValue(BazaarPreferenceConstants.HISTORY_SHOW_AFFETCED_PATHS, showPaths);
				if (showPaths) {
					pathsViewer.setRevision(selectedRevision);
				}
				layout();
			}
		};
		toggleShowAffectedPathsAction.setChecked(showPaths);
		// IMenuManager actionBarsMenu = actionBars.getMenuManager();
		// actionBarsMenu.add(toggleShowAffectedPathsAction);
		IToolBarManager tbm = actionBars.getToolBarManager();
		tbm.add(toggleShowAffectedPathsAction);
		tbm.update(false);
		actionBars.updateActionBars();
	}

}
