/*******************************************************************************
 * Copyright (c) 2007-2013 Guillermo Gonzalez
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.history;

import java.text.DateFormat;

import org.eclipse.jface.layout.TreeColumnLayout;
import org.eclipse.jface.viewers.CellLabelProvider;
import org.eclipse.jface.viewers.ColumnLabelProvider;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DelegatingStyledCellLabelProvider;
import org.eclipse.jface.viewers.ILabelProviderListener;
import org.eclipse.jface.viewers.ITreeContentProvider;
import org.eclipse.jface.viewers.StyledString;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.viewers.TreeViewerColumn;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;
import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeColumn;
import org.eclipse.team.core.history.IFileHistory;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.core.history.BazaarFileRevision;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.UIUtils;

/**
 * @author Guillermo Gonzalez
 * @author Piotr Piastucki
 *
 */
public class HistoryTreeViewer extends TreeViewer {

	public enum Column {
		REVNO, REVID, TAGS, COMMITTER, TIMESTAMP, BRANCH_NICK, MESSAGE;
	}

	private static Column[] defaultColumns = new Column[]{Column.REVNO, Column.MESSAGE, Column.COMMITTER, Column.TIMESTAMP, Column.BRANCH_NICK};
	
	public HistoryTreeViewer(Composite parent, Column...columns) {
		this(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL, columns);
	}

	public HistoryTreeViewer(Composite parent, int style, Column...columns) {
		super(new Tree(new Composite(parent, SWT.NONE), style | SWT.FULL_SELECTION));
		GridData gdata = new GridData(GridData.FILL_BOTH);
		gdata.grabExcessVerticalSpace = true;
		getControl().getParent().setLayoutData(gdata);
		Tree tree = configureTree(getTree().getParent(), getTree(), columns);
		for (TreeColumn column : tree.getColumns()) {
			CellLabelProvider labelProvider = null;
			Object data = column.getData();
			if (data == Column.REVNO || data == Column.REVID) {
				labelProvider = new RevisionColumnLabelProvider();
			} else if (data == Column.MESSAGE) {
				labelProvider = new MessageColumnLabelProvider();
			} else if (data == Column.COMMITTER) {
				labelProvider = new CommitterColumnLabelProvider();
			} else if (data == Column.TIMESTAMP) {
				labelProvider = new DateColumnLabelProvider();
			} else if (data == Column.BRANCH_NICK) {
				labelProvider = new BranchNickColumnLabelProvider();
			}
			if (labelProvider != null) {
				new TreeViewerColumn(this, column).setLabelProvider(labelProvider);
			}
		}
		setSorter(new RevisionSorter());
		setContentProvider(new LogTreeContentProvider());
	}

	private static Tree configureTree(Composite parent, Tree logTree, Column...columns) {
		if(columns == null || columns.length == 0) {
			columns = defaultColumns;
		}
		logTree.setLinesVisible(true);
		logTree.setHeaderVisible(true);

		TreeColumnLayout layout = new TreeColumnLayout();
		parent.setLayout(layout);

		for (int i = 0; i < columns.length; i++) {
			Column col = columns[i];			
			TreeColumn column = null;
			if(col.equals(Column.REVNO)) {
				column = new TreeColumn(logTree, SWT.LEFT);
				column.setText(UITexts.HistoryView_HeaderRevision);
				layout.setColumnData(column, new ColumnWeightData(5, true));
				column.setData(Column.REVNO);
			} else if(col.equals(Column.REVID)) {
				 column = new TreeColumn(logTree, SWT.LEFT);
				 column.setText(UITexts.HistoryView_HeaderRevId);
				 layout.setColumnData(column, new ColumnWeightData(10, true));
				 column.setData(Column.REVID);
			} else if(col.equals(Column.TAGS)) {
				 column = new TreeColumn(logTree, SWT.LEFT);
				 column.setText(UITexts.HistoryView_HeaderTags);
				 layout.setColumnData(column, new ColumnWeightData(10, true));
				 column.setData(Column.TAGS);
			} else if(col.equals(Column.COMMITTER)) {
				column = new TreeColumn(logTree, SWT.LEFT);
				column.setText(UITexts.HistoryView_HeaderCommitter);
				layout.setColumnData(column, new ColumnWeightData(13, true));
				column.setData(Column.COMMITTER);
			} else if(col.equals(Column.TIMESTAMP)) {
				column = new TreeColumn(logTree, SWT.LEFT);
				column.setText(UITexts.HistoryView_HeaderDate);
				column.setData(Column.TIMESTAMP);
				layout.setColumnData(column, new ColumnWeightData(10, true));
			} else if(col.equals(Column.BRANCH_NICK)) {
				column = new TreeColumn(logTree, SWT.LEFT);
				column.setText(UITexts.HistoryView_HeaderBranchNick);
				layout.setColumnData(column, new ColumnWeightData(7, true));
				column.setData(Column.BRANCH_NICK);
			} else if(col.equals(Column.MESSAGE)) {
				column = new TreeColumn(logTree, SWT.LEFT);
				column.setText(UITexts.HistoryView_HeaderMessage);
				layout.setColumnData(column, new ColumnWeightData(30, true));
				column.setData(Column.MESSAGE);
			}
			column.setResizable(i < columns.length - 1);
			UIUtils.addSizeLimiter(column);
		}
		return logTree;
	}

	protected static class LogTreeContentProvider implements ITreeContentProvider {

		public void inputChanged(Viewer v, Object oldInput, Object newInput) {
		}

		public void dispose() {
		}

		public Object[] getElements(Object parent) {
			final IFileHistory fileHistory = (IFileHistory) parent;
			return fileHistory.getFileRevisions();
		}

		public Object[] getChildren(Object parentElement) {
			if (parentElement instanceof BazaarFileRevision) {
                return ((BazaarFileRevision) parentElement).getChildren();
            }
			return null;
		}

		public Object getParent(Object element) {
			if (element instanceof BazaarFileRevision && ((BazaarFileRevision) element).hasParent()) {
				return ((BazaarFileRevision) element).getParent();
			}
			return null;
		}

		public boolean hasChildren(Object element) {
			if (element instanceof BazaarFileRevision) {
				BazaarFileRevision revision = (BazaarFileRevision) element;
				return revision.hasChildren();
			}
			return false;
		}
	}
	
	protected static class RevisionColumnLabelProvider extends ColumnLabelProvider {
		@Override
		public String getText(Object obj) {
			BazaarFileRevision bazaarFileRevision = (BazaarFileRevision) obj;
			IBazaarLogMessage logMessage = bazaarFileRevision.getIBazaarLogMessage();
			String ret = logMessage.getRevision().getValue();
			ret = ret == null || ret.trim().isEmpty() ? logMessage.getRevisionId() : ret;
			return ret;
		}
	}
	
	protected static class MessageColumnLabelProvider extends DelegatingStyledCellLabelProvider {

		public MessageColumnLabelProvider() {
			super(new IStyledLabelProvider() {

				public void addListener(ILabelProviderListener listener) {
				}

				public void dispose() {
				}

				public boolean isLabelProperty(Object element, String property) {
					return false;
				}

				public void removeListener(ILabelProviderListener listener) {
				}

				public StyledString getStyledText(Object element) {
					StyledString string = new StyledString();
					BazaarFileRevision bazaarFileRevision = (BazaarFileRevision) element;
					IBazaarLogMessage logMessage = bazaarFileRevision.getIBazaarLogMessage();
					if (logMessage.getTags() != null && !logMessage.getTags().isEmpty()) {
						StringBuilder tags = new StringBuilder();
						for (String tag : logMessage.getTags()) {
							tags.append("[").append(tag).append("]");
						}
						string.append(tags.toString(), StyledString.DECORATIONS_STYLER);
						string.append(" ");
					}
					string.append(logMessage.getMessage());
					return string;
				}

				public Image getImage(Object element) {
					return null;
				}
			});
		}
	}

	protected static class CommitterColumnLabelProvider extends ColumnLabelProvider {
		@Override
		public String getText(Object obj) {
			BazaarFileRevision bazaarFileRevision = (BazaarFileRevision) obj;
			IBazaarLogMessage logMessage = bazaarFileRevision.getIBazaarLogMessage();
			return logMessage.getCommiter();
		}
	}

	protected static class DateColumnLabelProvider extends ColumnLabelProvider {
		private final static DateFormat df = DateFormat.getInstance();// new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		@Override
		public String getText(Object obj) {
			BazaarFileRevision bazaarFileRevision = (BazaarFileRevision) obj;
			IBazaarLogMessage logMessage = bazaarFileRevision.getIBazaarLogMessage();
			return df.format(logMessage.getDate());
		}
	}

	protected static class BranchNickColumnLabelProvider extends ColumnLabelProvider {
		@Override
		public String getText(Object obj) {
			BazaarFileRevision bazaarFileRevision = (BazaarFileRevision) obj;
			IBazaarLogMessage logMessage = bazaarFileRevision.getIBazaarLogMessage();
			return logMessage.getBranchNick();
		}
	}

	protected static class RevisionSorter extends ViewerSorter {

		@Override
		public int compare(Viewer viewer, Object e1, Object e2) {

			if (!(e1 instanceof BazaarFileRevision) || !(e1 instanceof BazaarFileRevision)) {
				return super.compare(viewer, e1, e2);
			}
			BazaarFileRevision revision1 = (BazaarFileRevision) e1;
			BazaarFileRevision revision2 = (BazaarFileRevision) e2;
			return revision2.compareRevisionTo(revision1);
		}
	}

}
