/*******************************************************************************
 * Copyright (c) 2012 Idereal Limited
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Taler
 *    Piotr Piastucki
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.operations;

import java.io.File;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IProjectDescription;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.operation.IRunnableContext;
import org.eclipse.team.ui.TeamOperation;
import org.vcs.bazaar.eclipse.core.repository.BazaarTeamProviderType;
import org.vcs.bazaar.eclipse.core.utils.ResourceUtil;

/**
 * Operation for creating project(s) from a Bazaar branch.
 * 
 * Influenced by earlier implementation by Guillermo Gonzalez.
 * 
 * @author Alexander Taler alex <@> idereal <.> co <.> nz
 * @author Piotr Piastucki
 */
public class CreateBazaarProjectOperation extends TeamOperation {

	String projectName;
	File projectLocation;
	IProject project;

	/**
	 * 
	 * @param context
	 *            Found with getContainer()
	 * @param projectName
	 *            The project to be created
	 * @param projectLocation
	 *            Where on disk to check out the project
	 * @param project 
	 *            An existing project to be used
	 */
	public CreateBazaarProjectOperation(IRunnableContext context, String projectName, String projectLocation, IProject project) {
		super(context);
		this.projectName = projectName;
		this.projectLocation = new File(projectLocation);
		this.project = project;
	}

	public void run(IProgressMonitor monitor) throws InvocationTargetException {
		try {
			if (project != null) {
				mergeWithExistingProject(monitor);
			} else {
				createProjects(monitor);
			}
			
		} catch (Exception e) {
			throw new InvocationTargetException(e);
		} finally {
			if (project != null) {
				ResourceUtil.deleteRecursively(projectLocation);
			}
		}
	}

	private void createProjects(IProgressMonitor monitor) throws CoreException {
		// Search the resulting checked out directory for existing Eclipse
		// projects.
		List<File> projDirs = ResourceUtil.findProjects(projectLocation);
		if (projDirs.isEmpty()) {
			// If no projects were found, then just create a single project
			// at the root.
			projDirs.add(projectLocation);
		}
		for (File projDir : projDirs) {
			createProject(projDir, monitor);
		}
	}

	/**
	 * Create an Eclipse project at the given location.
	 * 
	 * Eclipse throws an exception if a ProjectDescription is used to create a
	 * project at a location directly in the workspace root. However, if the
	 * project location has the same name as the project itself, then creating
	 * the project without a description will not throw an exception.
	 * 
	 * So, as a special case if the directory is in the workspace root, but
	 * matches the project name, create is used without a description.
	 * 
	 * The other error case is not checked for here, in case this undocumented
	 * Eclipse behaviour changes.
	 */
	private void createProject(File projDir, IProgressMonitor monitor) throws CoreException {
		IWorkspace ws = ResourcesPlugin.getWorkspace();
		IPath projPath = new Path(projDir.toString());
		IPath parentPath = projPath.removeLastSegments(1);
		IProject projectHandle = null;

		if (parentPath.equals(ws.getRoot().getLocation()) && projPath.lastSegment().equals(projectName)) {

			projectHandle = ws.getRoot().getProject(projectName);
			projectHandle.create(monitor);

		} else {

			// Derive a new project name based on the directory
			String newProjectName = projectName + "-" + projPath.lastSegment();
			projectHandle = ws.getRoot().getProject(newProjectName);
			IProjectDescription projDesc = ws.newProjectDescription(newProjectName);
			projDesc.setLocation(projPath);
			projectHandle.create(projDesc, monitor);
		}
		projectHandle.open(monitor);
		BazaarTeamProviderType.registerProject(projectHandle);
	}
	
	private void mergeWithExistingProject(IProgressMonitor monitor) throws CoreException {
		mergeFolders(projectLocation, project.getLocation().toFile());
		project.refreshLocal(IResource.DEPTH_INFINITE, monitor);
	}

	/**
	 * Move a folder and merge it with another one
	 * @return true on success, false on failure
	 */
	private boolean mergeFolders(File source, File destination) {
		boolean result = true;
		for (File f : source.listFiles()) {
			File target = new File(destination, f.getName());
			if (target.exists()) {
				if (target.isDirectory() && f.isDirectory()) {
					result = result && mergeFolders(f, target);
					continue;
				} else {
					target.delete();
				}
			}
			result = result && f.renameTo(target);
		}
		return result;
	}

}
