/*******************************************************************************
* Copyright (c) 2007-2011 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.ui.preferences;

import org.eclipse.core.runtime.preferences.InstanceScope;
import org.eclipse.jface.preference.BooleanFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.jface.preference.FileFieldEditor;
import org.eclipse.jface.preference.IntegerFieldEditor;
import org.eclipse.jface.preference.StringFieldEditor;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;
import org.eclipse.ui.preferences.ScopedPreferenceStore;
import org.vcs.bazaar.client.BazaarClientFactory;
import org.vcs.bazaar.client.BazaarPreference;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.eclipse.ClientProperties;
import org.vcs.bazaar.eclipse.EclipseBazaarCore;
import org.vcs.bazaar.eclipse.IBazaarCoreConstants;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.UITexts;

/**
 * Main Preference page, it allow configure: <br>
 * - the bzr executable path<br>
 * - plugins path <br>
 * - xmlrpc service port<br>
 * 
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 */
public class BazaarPreferencePage extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    private FileFieldEditor executable;
    private BooleanFieldEditor overwriteEmail;
    private StringFieldEditor email;
    private StringFieldEditor clientValue;
	private IntegerFieldEditor xmlrpcPort;

	public BazaarPreferencePage() {
        super(GRID);
//		setPreferenceStore(EclipseBazaarUI.getDefault().getPreferenceStore());
		setPreferenceStore(new ScopedPreferenceStore(new InstanceScope(), EclipseBazaarCore.getPluginId()));
		setDescription(UITexts.BazaarPreferencesPage_description);
	}

	public void init(IWorkbench workbench) {}

	/**
	 * Creates the field editors. Field editors are abstractions of the common
	 * GUI blocks needed to manipulate various types of preferences. Each field
	 * editor knows how to save and restore itself.
	 */
	@Override
    public void createFieldEditors() {
		executable = new FileFieldEditor(BazaarPreference.EXECUTABLE.name(), UITexts.BazaarPreferencesPage_executable, getFieldEditorParent());
		overwriteEmail = new BooleanFieldEditor(IBazaarCoreConstants.OVERWRITE_EMAIL,
                UITexts.BazaarPreferencesPage_overwrite_userid, getFieldEditorParent());
		email = new StringFieldEditor(BazaarPreference.BZR_EMAIL.name(), UITexts.BazaarPreferencesPage_userid, getFieldEditorParent());
		addField(executable);
		addField(overwriteEmail);
		addField(email);
		createClientTypeControl();
		createXmlRpClientConfigControl();
		executable.getTextControl(getFieldEditorParent()).setEditable(false);
		email.getTextControl(getFieldEditorParent()).setEnabled(getPreferenceStore().getBoolean(IBazaarCoreConstants.OVERWRITE_EMAIL));
	}

	@Override
	public void propertyChange(PropertyChangeEvent event) {
		super.propertyChange(event);
		if (event.getSource() == overwriteEmail) {
			email.getTextControl(getFieldEditorParent()).setEnabled((Boolean)event.getNewValue());
		}
	}
	
	private void createClientTypeControl() {
		clientValue = new StringFieldEditor("ClientValue", UITexts.BazaarPreferencesPage_client_type, getFieldEditorParent()); //$NON-NLS-1$
		addField(clientValue);
		clientValue.getTextControl(getFieldEditorParent()).setEnabled(false);
		setClientTypeLabel();
	}

	private void createXmlRpClientConfigControl() {
		xmlrpcPort = new IntegerFieldEditor(BazaarPreference.BZR_XMLRPC_PORT.name(), UITexts.BazaarPreferencesPage_xmlrpc_port, getFieldEditorParent(), 5);
		addField(xmlrpcPort);
	}

	private void setClientTypeLabel() {
		try {
			clientValue.setStringValue(BazaarClientFactory.getPreferredClientType());
		} catch (BazaarClientException e1) {
			clientValue.setStringValue(e1.getMessage());
		}
	}

	@Override
	protected void initialize() {
		super.initialize();
		checkValidClient();
	}

	@Override
	protected void performDefaults() {
		super.performDefaults();
		email.getTextControl(getFieldEditorParent()).setEnabled(overwriteEmail.getBooleanValue());
	}

	@Override
	public boolean performOk() {
		if (super.performOk()) {
			return checkValidClient();
		}
		return false;
	}

	private boolean checkValidClient() {
		ClientProperties properties = EclipseBazaarCore.getDefault().getClientProperties(executable.getStringValue(), 
				overwriteEmail.getBooleanValue() ? email.getStringValue() : null, xmlrpcPort.getStringValue());
		// check the client state
		try {
			EclipseBazaarCore.getDefault().checkClient(properties);
			setMessage(null, ERROR);
			return true;
		} catch (BazaarException e) {
			setMessage(e.getStatus().getMessage(), ERROR);
			return false;
		} finally {
			setClientTypeLabel();
		}
	}

}
