/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.preferences;

import java.util.ArrayList;

import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.preference.FieldEditor;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelectionChangedListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.jface.viewers.ListViewer;
import org.eclipse.jface.viewers.SelectionChangedEvent;
import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerComparator;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.List;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.UIUtils;
import org.vcs.bazaar.eclipse.ui.dialogs.CommitMessageHistory;

/**
 * @author Piotr Piastucki
 */
public class TemplateEditorComponent extends FieldEditor implements ISelectionChangedListener {

	private List list;
	private ListViewer viewer;
	private Composite buttonBox;
	private Button addButton;
	private Button editButton;
	private Button removeButton;
	private Composite previewBox;
	private Text preview;
	private Shell shell;

	private int previewHeight;

	public TemplateEditorComponent(Shell shell, Composite parent, int previewHeight) {
		this.shell = shell;
		this.previewHeight = previewHeight;
		createControl(parent);
		getLabelControl().setText(UITexts.TemplateEditorComponent_label);
	}

	@Override
	protected void adjustForNumColumns(int numColumns) {
		((GridData) getLabelControl().getLayoutData()).horizontalSpan = numColumns;
		((GridData) list.getLayoutData()).horizontalSpan = numColumns - 1;
	}

	@Override
	protected void doFillIntoGrid(Composite parent, int numColumns) {

		Control control = getLabelControl(parent);
		GridData gd = new GridData();
		gd.horizontalSpan = numColumns;
		control.setLayoutData(gd);
		
		List list = getListControl(parent);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = numColumns - 1;
		list.setLayoutData(gd);

		Composite buttonBox = getButtonBoxControl(parent);
		gd = new GridData();
		gd.verticalAlignment = GridData.BEGINNING;
		buttonBox.setLayoutData(gd);

		Composite previewBox = getPreviewBoxControl(parent);
		gd = new GridData(SWT.FILL, SWT.FILL, true, true);
		gd.horizontalSpan = numColumns;
		previewBox.setLayoutData(gd);
	}

	public List getListControl(Composite parent) {
		if (viewer == null) {
			createList(parent);
		}
		return viewer.getList();
	}

	private void createList(Composite parent) {
		viewer = new ListViewer(parent);
		viewer.addSelectionChangedListener(this);
		viewer.setLabelProvider(new LabelProvider() {
			public String getText(Object element) {
				return UIUtils.getFirstLine((String) element);
			}
		});
		viewer.setComparator(new ViewerComparator() {
			public int compare(Viewer viewer, Object e1, Object e2) {
				String template1 = UIUtils.getFirstLine((String) e1);
				String template2 = UIUtils.getFirstLine((String) e2);
				return template1.compareToIgnoreCase(template2);
			}
		});
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				editPressed();
			}
		});
		list = viewer.getList();
		list.setFont(parent.getFont());
		list.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent event) {
				viewer = null;
			}
		});
	}

	private Composite getButtonBoxControl(Composite parent) {
		if (buttonBox == null) {
			createButtonBox(parent);
		}
		return buttonBox;
	}

	private void createButtonBox(Composite parent) {
		buttonBox = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		layout.marginWidth = 0;
		buttonBox.setLayout(layout);
		createButtons(buttonBox);
		buttonBox.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent event) {
				addButton = null;
				editButton = null;
				removeButton = null;
				buttonBox = null;
			}
		});
	}

	private Composite getPreviewBoxControl(Composite parent) {
		if (previewBox == null) {
			createPreviewBox(parent);
		}
		return previewBox;
	}

	private void createPreviewBox(Composite parent) {
		previewBox = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		previewBox.setLayout(layout);
		Label previewLabel = new Label(previewBox, SWT.NONE);
		previewLabel.setText(UITexts.TemplateEditorComponent_preview);
		preview = new Text(previewBox, SWT.MULTI | SWT.READ_ONLY | SWT.BORDER | SWT.V_SCROLL);
		GridData gd = new GridData(SWT.FILL, SWT.FILL, true, false);
		gd.heightHint = previewHeight;
		preview.setLayoutData(gd);
	}

	private void createButtons(Composite box) {
		addButton = createPushButton(box, UITexts.TemplateEditorComponent_new);//$NON-NLS-1$
		addButton.addSelectionListener(new SelectionAdapter() {
			@Override
            public void widgetSelected(SelectionEvent e) {
				addPressed();
			}			
		});
		editButton = createPushButton(box, UITexts.TemplateEditorComponent_edit);//$NON-NLS-1$
		editButton.addSelectionListener(new SelectionAdapter() {
			@Override
            public void widgetSelected(SelectionEvent e) {
				editPressed();
			}			
		});
		removeButton = createPushButton(box, UITexts.TemplateEditorComponent_remove);//$NON-NLS-1$
		removeButton.addSelectionListener(new SelectionAdapter() {
			@Override
            public void widgetSelected(SelectionEvent e) {
				removePressed();
			}			
		});
	}

	private Button createPushButton(Composite parent, String text) {
		Button button = new Button(parent, SWT.PUSH);
		button.setText(text);
		button.setFont(parent.getFont());
		GridData data = new GridData(GridData.FILL_HORIZONTAL);
		int widthHint = convertHorizontalDLUsToPixels(button, IDialogConstants.BUTTON_WIDTH);
		data.widthHint = Math.max(widthHint, button.computeSize(SWT.DEFAULT, SWT.DEFAULT, true).x);
		button.setLayoutData(data);
		return button;
	}

	private void editPressed() {
		IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
		if (selection.size() == 1) {
			String template = (String) selection.getFirstElement();
			editTemplate(template);
		}
	}

	private void addPressed() {
		editTemplate(null);
	}

	private void editTemplate(String template) {
		TemplateEditorDialog dialog = new TemplateEditorDialog(shell, template);
		if (dialog.open() == Window.OK && !dialog.getTemplate().isEmpty()) {
			if (template != null) {
				viewer.remove(template);
			}
			viewer.add(dialog.getTemplate());
		}
	}

	private void removePressed() {
		IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
		viewer.remove(selection.toArray());
	}

	public void selectionChanged(SelectionChangedEvent event) {
		IStructuredSelection selection = (IStructuredSelection) event.getSelection();
		removeButton.setEnabled(selection.size() > 0);
		editButton.setEnabled(selection.size() == 1);
		preview.setText(selection.size() == 1 ? (String) selection.getFirstElement() : "");
	}

	@Override
	public int getNumberOfControls() {
		return 2;
	}

	@Override
	protected void doLoad() {
		for (String template : CommitMessageHistory.getTemplates()) {
			viewer.add(template);
		}
	}

	@Override
	protected void doLoadDefault() {
	}

	@Override
	protected void doStore() {
		int numTemplates = viewer.getList().getItemCount();
		ArrayList<String> templates = new ArrayList<String>();
		for (int i = 0; i < numTemplates; i++) {
			templates.add((String) viewer.getElementAt(i));
		}
		CommitMessageHistory.saveTemplates(templates);
	}

}
