/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.synchronize;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.team.core.mapping.ISynchronizationScopeManager;
import org.eclipse.team.core.mapping.provider.MergeContext;
import org.eclipse.team.ui.TeamUI;
import org.eclipse.team.ui.synchronize.ISynchronizePageConfiguration;
import org.eclipse.team.ui.synchronize.ModelSynchronizeParticipant;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.PartInitException;
import org.vcs.bazaar.client.core.BazaarClientException;
import org.vcs.bazaar.client.core.BranchLocation;
import org.vcs.bazaar.eclipse.core.synchronize.BazaarResourceVariantTreeSubscriber;
import org.vcs.bazaar.eclipse.core.synchronize.BazaarSubscriberMergeContext;
import org.vcs.bazaar.eclipse.core.synchronize.BazaarSynchronizeData;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;

/**
 * @author Piotr Piastucki
 */
public class BazaarModelSynchronizeParticipant extends ModelSynchronizeParticipant {

	public static final String ID = "org.vcs.bazaar.eclipse.ui.synchronize.BazaarModelSynchronizeParticipant"; //$NON-NLS-1$
	
	private static final String DATA_NODE_KEY = "bazaarSynchronizeData"; //$NON-NLS-1$

	private static final String RESOURCE_KEY = "resource";
	
	private static final String BRANCH_KEY = "branch";

	private static final String PATH_KEY = "path";

	private static final String SYNCHRONIZATION_DATA = "BZR_SYNCHRONIZE_DATA";
	
	private BazaarSynchronizeData syncData;
	
	public BazaarModelSynchronizeParticipant() {
	}

	public BazaarModelSynchronizeParticipant(BazaarSubscriberMergeContext context) {
		super(context);
		this.syncData = context.getSyncData();
		try {
			setInitializationData(TeamUI.getSynchronizeManager().getParticipantDescriptor(ID));
		} catch (CoreException e) {
			EclipseBazaarUI.log(e);
		}
		setSecondaryId(Long.toString(System.currentTimeMillis()));
	}

	@Override
	public void init(String secondaryId, IMemento memento) throws PartInitException {
		try {
			IMemento syncDataNode = memento.getChild(DATA_NODE_KEY);
			if (syncDataNode != null) {
				IMemento[] paths = syncDataNode.getChildren(RESOURCE_KEY);
				BranchLocation branchLocation = new BranchLocation(syncDataNode.getString(BRANCH_KEY));
				Set<IResource> resources = new HashSet<IResource>();
				for (IMemento path : paths) {
					IResource resource = ResourcesPlugin.getWorkspace().getRoot().getContainerForLocation(new Path(path.getString(PATH_KEY)));
					resources.add(resource);
				}
				syncData = new BazaarSynchronizeData(resources.toArray(new IResource[resources.size()]), branchLocation);
			}
		} catch (BazaarClientException e) {
			throw new PartInitException(e.getMessage(), e);
		} finally {
			super.init(secondaryId, memento);
		}
	}
	
	@Override
	public void saveState(IMemento memento) {
		super.saveState(memento);
		if (syncData != null) {
			IMemento child = memento.createChild(DATA_NODE_KEY);
			child.putString(BRANCH_KEY, syncData.getRemoteLocation().toString());
			IResource[] resources = syncData.getResources();
			if (resources != null) {
				for (IResource resource : resources) {
					String path = resource.getLocation().toPortableString();
					child.createChild(RESOURCE_KEY).putString(PATH_KEY, path);
				}
			}
		}
	}
	
	@Override
	protected MergeContext restoreContext(ISynchronizationScopeManager manager) throws CoreException {
		if (syncData != null) {
			BazaarResourceVariantTreeSubscriber subscriber = new BazaarResourceVariantTreeSubscriber(syncData);
			subscriber.init(new NullProgressMonitor());
			return new BazaarSubscriberMergeContext(subscriber, manager, syncData);
		}
		return super.restoreContext(manager);
	}

	@Override
	protected void initializeConfiguration(final ISynchronizePageConfiguration configuration) {
		configuration.setProperty(SYNCHRONIZATION_DATA, syncData);
		super.initializeConfiguration(configuration);
		configuration.addActionContribution(new BazaarSynchronizeActionContributor());
	}

}
