/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.views;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.atomic.AtomicReference;

import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.IJobChangeEvent;
import org.eclipse.core.runtime.jobs.Job;
import org.eclipse.core.runtime.jobs.JobChangeAdapter;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.BaseSelectionListenerAction;
import org.eclipse.ui.actions.OpenFileAction;
import org.eclipse.ui.progress.UIJob;
import org.vcs.bazaar.client.IBazaarLogMessage;
import org.vcs.bazaar.eclipse.BzrWorkspaceRoot;
import org.vcs.bazaar.eclipse.core.commands.StatusCommand;
import org.vcs.bazaar.eclipse.core.history.BazaarFileHistory;
import org.vcs.bazaar.eclipse.core.listeners.BzrChangeStateEvent;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.core.resources.IStorageBazaarRevision;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UIIcons;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.UIUtils;
import org.vcs.bazaar.eclipse.ui.actions.ReplaceWithLatestAction;
import org.vcs.bazaar.eclipse.ui.compare.CompareUtils;
import org.vcs.bazaar.eclipse.ui.history.HistoryTreeViewer;
import org.vcs.bazaar.eclipse.ui.history.HistoryTreeViewer.Column;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComparator;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComparator.Order;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionPathLabelProvider;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionStatusLabelProvider;

/**
 * @author Piotr Piastucki
 */
public class StatusView extends BazaarView {

	public static final String VIEW_ID = "org.vcs.bazaar.eclipse.ui.views.StatusView"; //$NON-NLS-1$

	private static Column[] columns = new Column[] { Column.REVID, Column.COMMITTER, Column.TIMESTAMP, Column.MESSAGE };

	private SashForm container;
	private TableViewer viewer;
	private HistoryTreeViewer mergeViewer;
	private IResource[] input;

	@Override
	public void setInput(Object object) {
		if (object instanceof IResource[]) {
			this.input = (IResource[]) object;
			refresh();
		}
	}

	private void refresh() {
		if (input == null || input.length == 0) {
			viewer.setInput(null);
			viewer.refresh();
		} else {
			final AtomicReference<IBzrResourceStatus[]> status = new AtomicReference<IBzrResourceStatus[]>();
			final List<IBazaarLogMessage> pendingMerges = new ArrayList<IBazaarLogMessage>();
			Job job = new Job(UITexts.StatusView_refreshTaskName) {
				@Override
				protected IStatus run(IProgressMonitor monitor) {
					StatusCommand cmd = new StatusCommand(BzrWorkspaceRoot.getBzrResourcesFor(input));
					try {
						cmd.run(monitor);
						status.set(cmd.getStatus());
						pendingMerges.addAll(cmd.getPendingMerges());
						for (IBzrResourceStatus status : cmd.getStatus()) {
							if (!status.isIgnored() && !status.isUnchanged()
									&& status.getResource().getIResource().getType() != IResource.PROJECT)
								EclipseBazaarUI.getDefault().getConsole().logMessage(status.getStatus().toString());
						}
						if (cmd.getPendingMerges().size() > 0) {
							EclipseBazaarUI.getDefault().getConsole().logMessage("pending merges:\n");
							showPendingMerges(cmd.getPendingMerges(), new StringBuilder());
						}
					} catch (BazaarException e) {
					}
					return Status.OK_STATUS;
				}
			};
			job.addJobChangeListener(new JobChangeAdapter() {
				@Override
				public void done(IJobChangeEvent event) {
					if (event.getResult().isOK()) {
						getSite().getShell().getDisplay().asyncExec(new Runnable() {
							public void run() {
								if (pendingMerges.isEmpty()) {
									container.setMaximizedControl(viewer.getControl().getParent());
								} else {
									container.setMaximizedControl(null);
								}
								viewer.setInput(status.get());
								mergeViewer.setInput(new BazaarFileHistory(pendingMerges));
								viewer.refresh();
							}
						});
					}
				}
			});
			job.setUser(true);
			job.setPriority(UIJob.LONG);
			job.schedule();
		}
	}

	protected void showPendingMerges(List<IBazaarLogMessage> pendingMerges, StringBuilder tab) {
		tab.append("    ");
		for (IBazaarLogMessage log : pendingMerges) {
			StringBuilder sb = new StringBuilder(100);
			sb.append(tab.toString()).append(log.getAuthor()).append(" ").append(log.getTimeStamp()).append(" ")
					.append(log.getMessage());
			EclipseBazaarUI.getDefault().getConsole().logMessage(sb.toString());
			if (log.getMerged() != null) {
				showPendingMerges(log.getMerged(), new StringBuilder(tab.toString()));
			}
		}
	}

	@Override
	public void createPartControl(Composite parent) {
		container = new SashForm(parent, SWT.VERTICAL);
		container.setLayoutData(new GridData(GridData.FILL_BOTH));

		Composite tableContainer = new Composite(container, SWT.NONE);
		TableColumnLayout layout = new TableColumnLayout();
		tableContainer .setLayout(layout);

		final Table table = new Table(tableContainer, SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION);
		table.setLayout(new GridLayout());
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		viewer = new TableViewer(table);
		viewer.setContentProvider(new ArrayContentProvider());
		viewer.setComparator(new BzrResourceSelectionComparator(Order.ByStatus.ascending()));
		// Type
		final TableColumn status = new TableColumn(viewer.getTable(), SWT.NONE);
		status.setText(UITexts.CommitDialog_fileStatusHeader);
		status.setMoveable(false);
		status.setResizable(true);
		UIUtils.addSizeLimiter(status);
		layout.setColumnData(status, new ColumnWeightData(1));
		// Path
		final TableColumn file = new TableColumn(viewer.getTable(), SWT.NONE);
		file.setText(UITexts.CommitDialog_fileNameHeader);
		file.setResizable(false);
		file.setMoveable(false);
		UIUtils.addSizeLimiter(file);
		layout.setColumnData(file, new ColumnWeightData(10));

		new TableViewerColumn(viewer, status).setLabelProvider(new BzrResourceSelectionStatusLabelProvider());
		new TableViewerColumn(viewer, file).setLabelProvider(new BzrResourceSelectionPathLabelProvider());
		ColumnViewerToolTipSupport.enableFor(viewer);
		table.setSortDirection(SWT.UP);
		table.setSortColumn(status);
		
		mergeViewer = new HistoryTreeViewer(container, columns);
		container.setMaximizedControl(viewer.getControl().getParent());

		contributeToContextMenu();
		contributeToActionBars();
		addDoubleClickListener();
	}

	private void contributeToActionBars() {
		IActionBars actionBars = getViewSite().getActionBars();
		Action toggleShowAffectedPathsAction = new Action(UITexts.View_Refresh, UIIcons.REFRESH) {
			@Override
			public void run() {
				refresh();
			}
		};
		IToolBarManager tbm = actionBars.getToolBarManager();
		tbm.add(toggleShowAffectedPathsAction);
		tbm.update(false);
		actionBars.updateActionBars();
	}

	private void contributeToContextMenu() {
		final StatusViewDiffAction diffAction = new StatusViewDiffAction(UITexts.ActionCompareWithLatest_label);
		final StatusViewOpenAction openAction = new StatusViewOpenAction(UITexts.ActionOpenInEditor_label);
		final StatusViewRevertAction revertAction = new StatusViewRevertAction(UITexts.ActionRevert_label);
		viewer.getTable().addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				diffAction.selectionChanged(selection);
				openAction.selectionChanged(selection);
				revertAction.selectionChanged(selection);
			}
		});
		MenuManager menuMgr = new MenuManager();
		Menu menu = menuMgr.createContextMenu(viewer.getTable());
		menuMgr.addMenuListener(new IMenuListener() {
			public void menuAboutToShow(IMenuManager menuMgr) {
				menuMgr.add(new Separator(IWorkbenchActionConstants.GROUP_FILE));
				menuMgr.add(diffAction);
				menuMgr.add(openAction);
				menuMgr.add(revertAction);
			}
		});
		menuMgr.setRemoveAllWhenShown(true);
		viewer.getTable().setMenu(menu);
	}

	private void addDoubleClickListener() {
		viewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				IStructuredSelection selection = (IStructuredSelection) viewer.getSelection();
				if (!selection.isEmpty()) {
					IBzrResourceStatus status = (IBzrResourceStatus) selection.getFirstElement();
					if (status.isModified()) {
						compareWithLatest(status);
					} else if (status.getResource().getIResource().getType() == IResource.FILE) {
						openInEditor(status);
					}
				}
			}
		});
	}

	private void openInEditor(IBzrResourceStatus status) {
		OpenFileAction ofa = new OpenFileAction(getSite().getWorkbenchWindow().getActivePage());
		ofa.selectionChanged(new StructuredSelection(status.getResource().getIResource()));
		ofa.run();
	}

	private void compareWithLatest(IBzrResourceStatus status) {
		IResource resource = status.getResource().getIResource();
		CompareUtils.openSaveableEditor(resource, new IStorageBazaarRevision(resource), getSite().getWorkbenchWindow()
				.getActivePage());
	}

	private class StatusViewDiffAction extends BaseSelectionListenerAction {
		protected StatusViewDiffAction(String text) {
			super(text);
		}

		@Override
		public void run() {
			compareWithLatest((IBzrResourceStatus) getStructuredSelection().getFirstElement());
		}

		@Override
		protected boolean updateSelection(IStructuredSelection selection) {
			if (!selection.isEmpty()) {
				IBzrResourceStatus status = (IBzrResourceStatus) selection.getFirstElement();
				return status.isModified();
			}
			return false;
		}
	}

	private class StatusViewOpenAction extends BaseSelectionListenerAction {
		protected StatusViewOpenAction(String text) {
			super(text);
		}

		@Override
		public void run() {
			openInEditor((IBzrResourceStatus) getStructuredSelection().getFirstElement());
		}

		@Override
		protected boolean updateSelection(IStructuredSelection selection) {
			if (!selection.isEmpty()) {
				IBzrResourceStatus status = (IBzrResourceStatus) selection.getFirstElement();
				return status.getResource().getIResource().getType() == IResource.FILE;
			}
			return false;
		}
	}

	private class StatusViewRevertAction extends BaseSelectionListenerAction {
		protected StatusViewRevertAction(String text) {
			super(text);
		}

		@Override
		public void run() {
			ReplaceWithLatestAction rwla = new ReplaceWithLatestAction();
			IBzrResourceStatus status = (IBzrResourceStatus) getStructuredSelection().getFirstElement();
			rwla.selectionChanged(null, new StructuredSelection(status.getResource().getIResource()));
			rwla.run(new Action(UITexts.ActionRevert_label) {
			});
			refresh();
		}

		@Override
		protected boolean updateSelection(IStructuredSelection selection) {
			if (!selection.isEmpty()) {
				IBzrResourceStatus status = (IBzrResourceStatus) selection.getFirstElement();
				return !status.isUnknown();
			}
			return false;
		}
	}

	@Override
	public void setFocus() {
		// TODO Auto-generated method stub
	}

	public void resourceStateChanged(BzrChangeStateEvent event) {
		// TODO Auto-generated method stub
	}

}
