/*******************************************************************************
* Copyright (c) 2007-2011 Guillermo Gonzalez
*
* All rights reserved. This program and the accompanying materials
* are made available under the terms of the Eclipse Public License v1.0
* which accompanies this distribution, and is available at
* http://www.eclipse.org/legal/epl-v10.html
*
*******************************************************************************/
package org.vcs.bazaar.eclipse.ui.widgets;

import java.util.Collections;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.ListenerList;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.jface.layout.TableColumnLayout;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.util.SafeRunnable;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CheckStateChangedEvent;
import org.eclipse.jface.viewers.CheckboxTableViewer;
import org.eclipse.jface.viewers.ColumnViewerToolTipSupport;
import org.eclipse.jface.viewers.ColumnWeightData;
import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.ICheckStateListener;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.viewers.TableViewer;
import org.eclipse.jface.viewers.TableViewerColumn;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.IDE;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UIIcons;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.UIUtils;
import org.vcs.bazaar.eclipse.ui.compare.CompareUtils;
import org.vcs.bazaar.eclipse.ui.dialogs.UntrackedFilesFilter;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComparator.Order;

/**
 * @author Guillermo Gonzalez <guillo.gonzo@gmail.com>
 *
 */
public class BzrResourceSelectionComposite {

	private Composite control;
	private TableViewer tableViewer;
	private List<IBzrResourceStatus> resources;
	private List<IBzrResourceStatus> selection;
	private UntrackedFilesFilter untrackedFilesFilter;
	private boolean filterUntracked = false;
	private boolean checkBox = true;
    private ListenerList checkStateListeners = new ListenerList();
	private boolean showUntrackedButton;
	private ToolItem untrackedFilesButton;

	public BzrResourceSelectionComposite(boolean checkBox, boolean showUntrackedButton) {
		super();
		resources = selection = Collections.emptyList();
		untrackedFilesFilter = new UntrackedFilesFilter();
		this.checkBox = checkBox;
		this.showUntrackedButton = showUntrackedButton;
	}

	public BzrResourceSelectionComposite() {
		this(true, false);
	}

	private void createFilesList(Composite parent) {
		int tableStyle;
		if(checkBox) {
			tableStyle = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI | SWT.CHECK | SWT.BORDER;
		} else {
			tableStyle = SWT.H_SCROLL | SWT.V_SCROLL | SWT.FULL_SELECTION | SWT.MULTI;
		}
		Composite container = new Composite(parent, SWT.NONE);
		TableColumnLayout layout = new TableColumnLayout();
		container.setLayout(layout);

		final Table table = new Table(container, tableStyle);
		table.setLayout(new GridLayout());
		table.setHeaderVisible(true);
		table.setLinesVisible(true);

		// File status
		final TableColumn status = new TableColumn(table, SWT.LEFT);
		status.setText(UITexts.CommitDialog_fileStatusHeader);
		status.setMoveable(false);
		status.setResizable(true);
		UIUtils.addSizeLimiter(status);
		layout.setColumnData(status, new ColumnWeightData(50, false));

		// File name
		final TableColumn file = new TableColumn(table, SWT.LEFT);
		file.setText(UITexts.CommitDialog_fileNameHeader);
		file.setResizable(false);
		file.setMoveable(false);
		UIUtils.addSizeLimiter(file);
		layout.setColumnData(file, new ColumnWeightData(400, false));

		if(checkBox) {
			tableViewer = new CheckboxTableViewer(table);
		} else {
			tableViewer = new TableViewer(table);
		}

		status.addSelectionListener(new BzrResourceHeaderSelectionListener(tableViewer, Order.ByStatus));
		file.addSelectionListener(new BzrResourceHeaderSelectionListener(tableViewer, Order.ByFile));
		tableViewer.setComparator(new BzrResourceSelectionComparator(Order.ByStatus.ascending()));
		new TableViewerColumn(tableViewer, status).setLabelProvider(new BzrResourceSelectionStatusLabelProvider());
		new TableViewerColumn(tableViewer, file).setLabelProvider(new BzrResourceSelectionPathLabelProvider());
		ColumnViewerToolTipSupport.enableFor(tableViewer);
		tableViewer.setContentProvider(new ArrayContentProvider());
		
		/* add comparator and sorting */
		table.setSortDirection(SWT.UP);
		table.setSortColumn(status);

		this.tableViewer.addDoubleClickListener(new IDoubleClickListener() {
			public void doubleClick(DoubleClickEvent event) {
				IStructuredSelection sel = (IStructuredSelection) tableViewer.getSelection();
				if (sel.getFirstElement() instanceof IBzrResourceStatus) {
					IBzrResourceStatus status = (IBzrResourceStatus) sel.getFirstElement();
					if ((status.isAdded() || status.isUnknown()) && status.getResource().getIResource().getType() == IResource.FILE && EclipseBazaarUI.getActivePage() != null) {
						try {
							IDE.openEditor(EclipseBazaarUI.getActivePage(), (IFile) status.getResource().getIResource());
						} catch (PartInitException e) {
							EclipseBazaarUI.log(e);
						}
					} else if(!status.isDeleted() && status.getResource().getIResource().getType() == IResource.FILE) {
						CompareUtils.openDialog(status.getResource().getIResource());
					}
				}
			}
		});
		if (checkBox) {
			((CheckboxTableViewer) tableViewer).addCheckStateListener(new ICheckStateListener() {
				public void checkStateChanged(CheckStateChangedEvent event) {
					fireCheckStateChanged(event);
				}
			});
			final Action selectAllAction = new Action(UITexts.CommitDialog_selectAll) {
				@Override
				public void run() {
					((CheckboxTableViewer) tableViewer).setAllChecked(true);
					fireCheckStateChanged(null);
				}
			};
			final Action deselectAllAction = new Action(UITexts.CommitDialog_deselectAll) {
				@Override
				public void run() {
					((CheckboxTableViewer) tableViewer).setAllChecked(false);
					fireCheckStateChanged(null);
				}
			};
			MenuManager menuMgr = new MenuManager();
			tableViewer.getControl().setMenu(menuMgr.createContextMenu(tableViewer.getControl()));
			menuMgr.add(selectAllAction);
			menuMgr.add(deselectAllAction);
		}
		GridDataFactory gf = GridDataFactory.fillDefaults().align(GridData.FILL, GridData.FILL).grab(true, true);
        gf.applyTo(container);
        status.pack();
	}

	/**
	 * Adds a label just on top of the {@link CheckboxTableViewer}
	 * The default implementation don't add any label
	 * @param container
	 */
	
	protected String getUpperLabel() {
		return "Changes";
	}
	
	protected void createUpperControls(Composite container) {
		Composite upperContainer = new Composite(container, SWT.NONE);
		upperContainer.setLayout(new GridLayout(2, false));
		GridData data2 = new GridData();
        data2.horizontalAlignment = GridData.FILL;
        data2.verticalAlignment = GridData.END;
        data2.grabExcessHorizontalSpace = true;
        data2.grabExcessVerticalSpace = false;
        upperContainer.setLayoutData(data2);

        Label filesLabel = new Label(upperContainer, GridData.FILL_HORIZONTAL);
        filesLabel.setText(getUpperLabel());
		GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.verticalAlignment = GridData.END;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = false;
        filesLabel.setLayoutData(data);

        ToolBar toolbar = new ToolBar(upperContainer, SWT.FLAT);
        createUpperButtons(toolbar);
	}


	public Composite createControls(Composite container) {
		control = new Composite(container, SWT.NONE);
		control.setLayout(new GridLayout());

		createUpperControls(control);
		createMiddleControls(control);
		createBottomControls(control);
		return control;
	}

	protected void createBottomControls(Composite control) {
		Composite optionComposite = createOptionComposite(control);
		createOption(optionComposite);
	}

	protected void createMiddleControls(Composite control) {
		createFilesList(control);
	}

	/**
	 * create the options section.
	 * The default implemention don't add any widget,
	 * it's subclass responsability to implement this as needed.
	 *
	 * @param optionComposite
	 */
	protected void createOption(Composite optionComposite) {
		// TODO Auto-generated method stub
	}

	protected Composite createOptionComposite(Composite container) {
		Composite optionsContainer = new Composite(container, SWT.NONE);
		optionsContainer.setLayout(new GridLayout(1, false));
		GridData data = new GridData();
        data.horizontalAlignment = GridData.FILL;
        data.verticalAlignment = GridData.END;
        data.grabExcessHorizontalSpace = true;
        data.grabExcessVerticalSpace = false;
        optionsContainer.setLayoutData(data);
		return optionsContainer;
	}


	/**
	 * @param tComposite
	 * @return
	 */
	protected void createUpperButtons(ToolBar toolbar) {
		if (showUntrackedButton) {
			untrackedFilesButton = addToolBarItem(toolbar, SWT.CHECK, UIIcons.UNTRACKED_FILE,
					UITexts.CommitDialog_checkboxUntracked, new SelectionAdapter() {
						@Override
						public void widgetSelected(SelectionEvent e) {
							if (untrackedFilesButton.getSelection()) {
								removeUntrackedFilesFilter();
								for (TableColumn column : getTableViewer().getTable().getColumns()) {
									column.pack();
								}
							} else {
								addUntrackedFilesFilter();
							}
							getTableViewer().refresh(true);
						}
					});
			untrackedFilesButton.setSelection(!isFilterUntracked());
		}

		if (checkBox) {
			addToolBarItem(toolbar, SWT.PUSH, UIIcons.CHECK_ALL, UITexts.CommitDialog_selectAll, new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					if (checkBox && tableViewer.getTable().getEnabled()) {
						((CheckboxTableViewer) tableViewer).setAllChecked(true);
						fireCheckStateChanged(null);
					}
				}
			});

			addToolBarItem(toolbar, SWT.PUSH, UIIcons.UNCHECK_ALL, UITexts.CommitDialog_deselectAll, new SelectionAdapter() {
				@Override
				public void widgetSelected(SelectionEvent e) {
					if (checkBox && tableViewer.getTable().getEnabled()) {
						((CheckboxTableViewer) tableViewer).setAllChecked(false);
						fireCheckStateChanged(null);
					}
				}
			});
		}
	}

	protected ToolItem addToolBarItem(ToolBar toolbar, int style, ImageDescriptor imageDescriptor, String tooltip, SelectionAdapter selectionAdapter) {
		ToolItem item = new ToolItem(toolbar, style);
		final Image image = imageDescriptor.createImage();
		item.setImage(image);
		item.setToolTipText(tooltip);
		item.addSelectionListener(selectionAdapter);
		item.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent arg0) {
				image.dispose();
			}
		});
		return item;
	}
	
	public void addUntrackedFilesFilter() {
		if(!filterUntracked) {
			if (untrackedFilesButton != null) {
				untrackedFilesButton.setSelection(false);
			}
			tableViewer.addFilter(untrackedFilesFilter);
			filterUntracked = true;
			fireCheckStateChanged(null);
		}
	}

	public void removeUntrackedFilesFilter() {
		if(filterUntracked) {
			if (untrackedFilesButton != null) {
				untrackedFilesButton.setSelection(true);
			}
			tableViewer.removeFilter(untrackedFilesFilter);
			filterUntracked = false;
			fireCheckStateChanged(null);
		}
	}

	public void addUntrackedFilesFilterIfNoUntrackedSelected() {
		boolean filter = true;
		// Do not filter untracked files if at least one untracked file is selected
		for (IBzrResourceStatus status : selection) {
			if (status.isUnknown() && status.getResource() != null && !status.getResource().isFolder()) {
				filter = false;
				break;
			}
		}
		// or if there are only untracked files available for selection
		if (filter) {
			filter = false;
			for (IBzrResourceStatus status : resources) {
				if (!status.isUnknown()) {
					filter = true;
					break;
				}
			}
		}
		if (filter) {
			addUntrackedFilesFilter();
		}
	}

	public void setResources(List<IBzrResourceStatus> resources) {
		this.resources = resources;
	}

	public List<IBzrResourceStatus> getResources() {
		return resources;
	}

	public List<IBzrResourceStatus> getSelection() {
		if(checkBox) {
			Object[] checkedElements = ((CheckboxTableViewer)tableViewer).getCheckedElements();
			this.selection.clear();
			for (Object checked : checkedElements) {
				this.selection.add((IBzrResourceStatus)checked);
			}
		}
		return selection;
	}

	public void setSelection(List<IBzrResourceStatus> selection) {
		this.selection = selection;
	}

	public boolean isFilterUntracked() {
		return filterUntracked;
	}

	/**
	 * reload the resources, uncheck all, and then select only the resources in selection List
	 */
	public void refreshTableViewer() {
		tableViewer.setInput(resources);
		tableViewer.refresh(true);
		if(checkBox) {
			((CheckboxTableViewer)tableViewer).setAllChecked(false);
			for (IBzrResourceStatus bzrResourceStatus : this.selection) {
				((CheckboxTableViewer)tableViewer).setChecked(bzrResourceStatus, true);
			}
			fireCheckStateChanged(null);
		}
	}

	public TableViewer getTableViewer() {
		return tableViewer;
	}

    public void addCheckStateListener(ICheckStateListener listener) {
        checkStateListeners.add(listener);
    }

    private void fireCheckStateChanged(final CheckStateChangedEvent event) {
        Object[] array = checkStateListeners.getListeners();
        for (int i = 0; i < array.length; i++) {
            final ICheckStateListener l = (ICheckStateListener) array[i];
            SafeRunnable.run(new SafeRunnable() {
                public void run() {
                    l.checkStateChanged(event);
                }
            });
        }
    }
}
