/*******************************************************************************
 * Copyright (c) 2012 Idereal Limited
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *    Alexander Taler
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.createproject;

import org.eclipse.jface.dialogs.IMessageProvider;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Scrollable;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.eclipse.internal.core.BazaarException;

/**
 * A set of widgets and functionality on the project wizard page.
 * 
 * Provides layout: creating a label and single line text box.
 * 
 * Manages page completion by listening for events, providing isComplete()
 * method and calling setPageComplete().
 * 
 * Can "cascade" text to another text entry box. This allows user input to
 * update several entries simultaneously. This is suppressed for entries which
 * the user has edited manually.
 * 
 * @author Alexander Taler alex <@> idereal <.> co <.> nz
 */
public class CreateProjectEntry implements Listener {

    Scrollable textWidget; // The widget holding the text.
    WizardPage page; // The parent page
    CreateProjectWizard wiz; // The parent wizard to act as a reference
    boolean touched = false; // true when the user has directly edited the field
    CreateProjectEntry cascadeTo = null; // An entry object to cascade text to
    String baseValue = ""; // The base value, modifiable but a useful start

    protected Text text; // The text widget which holds user input.  It may be null in sub classes

    /**
     * A new text entry box on a CreateProjectPage.
     * 
     * @param parent The SWT Composite holds this.
     * @param page The CreateProjectPage containing this.
     * @param labelText String to display to user explaining what to enter.
     * @param width Suggested width for the text entry box.
     * @param baseValue Optional base value.
     */
    public CreateProjectEntry(Composite parent,
        WizardPage page,
        CreateProjectWizard wiz,
        String labelText,
        int width,
        String baseValue)
    {
        this.page = page;
        this.wiz = wiz;
        if (baseValue != null) {
            this.baseValue = baseValue;
        }

        Label label = new Label(parent, SWT.LEFT);
        label.setText(labelText);

        textWidget = createWidget(parent);
        final GridData gridData = new GridData(SWT.VERTICAL);
        gridData.horizontalAlignment = width == SWT.DEFAULT ? SWT.FILL : SWT.BEGINNING;
        gridData.grabExcessHorizontalSpace = width == SWT.DEFAULT;
        gridData.widthHint = width;
        textWidget.setLayoutData(gridData);

        textWidget.addListener(SWT.Modify, this);
    }

    protected Scrollable createWidget(Composite parent) {
        text = new Text(parent, SWT.SINGLE | SWT.BORDER);
        text.setText(baseValue);
        return text;
    }

    public void handleEvent(Event event) {
        if (textWidget.isFocusControl()) {
            touched = isModified();
            page.setMessage(null);
        }
        if (cascadeTo != null) {
            cascadeTo.cascade(getValue());
        }
        try {
            isValid();
            page.setPageComplete(true);
        } catch (BazaarException be) {
            page.setPageComplete(false);
            // SetErrorMessage is not displaying. I have not investigated.
            String message = be.getStatus().getMessage();
            if ((message != null) && (message.length() > 0)) {
                page.setMessage(message, IMessageProvider.WARNING);
            }
        }
    }

    public void setCascadeTo(CreateProjectEntry cascadeTo) {
        this.cascadeTo = cascadeTo;
    }

    /**
     * Receive text from another element used to populate this field.
     * 
     * @param cascadeText
     */
    private void cascade(String cascadeText) {
        if (touched) {
            return;
        }
        // Only use the last segment assuming it's a file name or similar
        cascadeText = cascadeText.trim();
        if (cascadeText.startsWith("lp:")) {
        	cascadeText = cascadeText.substring(3);
        }
        String[] parts = cascadeText.split("[\\\\/]");
        if (parts.length > 0) {
            cascadeText = parts[parts.length - 1];
        }
        setValue(baseValue + cascadeText);
        if (cascadeTo != null) {
            cascadeTo.cascade(getValue());
        }
    }

    /**
     * Check if this field is modified from it's original value.
     */
    public boolean isModified() {
        return (getValue().length() > 0) && (!getValue().equals(baseValue));
    }
    
    /**
     * Check if this field is complete and valid.
     * 
     * The default implementation just checks if it's modified. The thrown
     * exception message explains the issue, or is empty if there's nothing
     * interesting to say.
     * 
     * @throws BazaarException including a message.
     */
    public void isValid() throws BazaarException {
        if (!isModified()) {
            throw new BazaarException("");
        }
    }
    
    // This method is used internally in order to allow subclasses to override text
    public String getValue() {
        return text.getText();
    }

    // This method is used internally in order to allow subclasses to override text
    public void setValue(String newValue) {
        text.setText(newValue);
    }

}
