/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.patch;

import java.io.File;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Listener;
import org.eclipse.swt.widgets.Text;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.widgets.BzrResourceSelectionComposite;

/**
 * @author Piotr Piastucki
 */
public class CreatePatchPage extends WizardPage {

	private static final String PATH_KEY = "BazaarCreatePatchWizard.CreatePatchPage.path"; //$NON-NLS-1$
	private static final String LOCATION_KEY = "BazaarCreatePatchWizard.CreatePatchPage.location"; //$NON-NLS-1$
	private static final String LOCATION_VALUE_CLIPBOARD = "clipboard"; //$NON-NLS-1$
	private static final String LOCATION_VALUE_FILE_SYSTEM = "filesystem"; //$NON-NLS-1$
//	private static final String LOCATION_VALUE_WORKSPACE = "workspace"; //$NON-NLS-1$

	private Button clipboardRadio;
	private Button fileRadio;
	private Text filePath;
	private Button fileBrowse;
	private BzrResourceSelectionComposite resourceSelection;
	private boolean pageValid;
	private IProject project;
	/*
	private Button workspaceRadio;
	private Text workspacePath;
	private Button workspaceBrowse;
	*/

	protected CreatePatchPage(String pageName, String title, ImageDescriptor titleImage, List<IBzrResourceStatus> resources, List<IBzrResourceStatus> selectedResources) {
		super(pageName, title, titleImage);
		project = resources.get(0).getResource().getIResource().getProject();
		resourceSelection = new BzrResourceSelectionComposite(true, true);
		resourceSelection.setResources(resources);
		resourceSelection.setSelection(selectedResources);
	}

	public void createControl(Composite parent) {
		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout());
		container.setLayoutData(new GridData(GridData.FILL_BOTH));
		
		Composite composite = new Composite(container, SWT.NONE);
		composite.setLayout(new GridLayout(3, false));
		composite.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		initializeDialogUnits(composite);

		String selectedOption= getDialogSettings().get(LOCATION_KEY);
		if (selectedOption == null) {
			selectedOption = LOCATION_VALUE_CLIPBOARD;
		}
		
		boolean clipboardSelected = selectedOption.equals(LOCATION_VALUE_CLIPBOARD);
		boolean fileSelected = !clipboardSelected;
		GridData gd = new GridData(SWT.BEGINNING, SWT.BEGINNING, false, false);
		gd.horizontalSpan = 3;
		clipboardRadio = new Button(composite, SWT.RADIO);
		clipboardRadio.setText(UITexts.CreatePatchWizard_Clipboard);
		clipboardRadio.setLayoutData(gd);
		clipboardRadio.setSelection(clipboardSelected);

		fileRadio = new Button(composite, SWT.RADIO);
		fileRadio.setText(UITexts.CreatePatchWizard_File);
		fileRadio.setSelection(fileSelected);
		filePath = new Text(composite, SWT.BORDER);
		gd = new GridData(GridData.FILL_HORIZONTAL);
		filePath.setLayoutData(gd);
		filePath.setText(createFileName());
		filePath.setEnabled(fileSelected);
		fileBrowse = new Button(composite, SWT.PUSH);
		fileBrowse.setText(UITexts.CreatePatchWizard_Browse);
		/*
		workspaceRadio = new Button(composite, SWT.RADIO);
		workspaceRadio.setText(UITexts.CreatePatchWizard_Workspace);
		workspaceRadio.setSelection(fileSelected);
		workspacePath = new Text(composite, SWT.BORDER);
		workspacePath.setLayoutData(new GridData(GridData.FILL_HORIZONTAL));
		workspacePath.setEnabled(fileSelected);
		workspaceBrowse = new Button(composite, SWT.PUSH);
		workspaceBrowse.setText(UITexts.CreatePatchWizard_Browse);
		workspaceBrowse.setEnabled(fileSelected);
*/
		
		Composite control = resourceSelection.createControls(container);
		control.setLayoutData(new GridData(GridData.FILL_BOTH));
		resourceSelection.addUntrackedFilesFilterIfNoUntrackedSelected();
		resourceSelection.refreshTableViewer();
		
		clipboardRadio.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (((Button) event.widget).getSelection()) {
					filePath.setEnabled(false);
					fileBrowse.setEnabled(false);
					//workspacePath.setEnabled(false);
					//workspaceBrowse.setEnabled(false);
					getDialogSettings().put(LOCATION_KEY, LOCATION_VALUE_CLIPBOARD);
					validatePage();
				}
			}
		});
		
		fileRadio.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (((Button) event.widget).getSelection()) {
					filePath.setEnabled(true);
					fileBrowse.setEnabled(true);
					//workspacePath.setEnabled(false);
					//workspaceBrowse.setEnabled(false);
					filePath.setFocus();
					getDialogSettings().put(LOCATION_KEY, LOCATION_VALUE_FILE_SYSTEM);
					validatePage();
				}
			}
		});

		filePath.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (validatePage()) {
					IPath path = Path.fromOSString(filePath.getText()).removeLastSegments(1);
					getDialogSettings().put(PATH_KEY, path.toPortableString());
				}
			}
		});

		fileBrowse.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				final FileDialog dialog = new FileDialog(getShell(), SWT.PRIMARY_MODAL | SWT.SAVE);
				if (pageValid) {
					final File file = new File(filePath.getText());
					dialog.setFilterPath(file.getParent());
					dialog.setFileName(file.getName());
				} else {
					dialog.setFileName(""); //$NON-NLS-1$
				}
				dialog.setText(""); //$NON-NLS-1$
				final String path = dialog.open();
				if (path != null) {
					filePath.setText(new Path(path).toOSString());
				}
				validatePage();
			}
		});
/*		
		workspaceRadio.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
				if (((Button) event.widget).getSelection()) {
					filePath.setEnabled(false);
					fileBrowse.setEnabled(false);
					workspacePath.setEnabled(true);
					workspaceBrowse.setEnabled(true);
					workspacePath.setFocus();
					getDialogSettings().put(LOCATION_KEY, LOCATION_VALUE_WORKSPACE);
					validatePage();
				}
			}
		});

		workspacePath.addModifyListener(new ModifyListener() {
			public void modifyText(ModifyEvent e) {
				if (validatePage()) {
					IPath path= Path.fromOSString(workspacePath.getText()).removeLastSegments(1);
					getDialogSettings().put(PATH_KEY, path.toPortableString());
				}
			}
		});

		workspaceBrowse.addListener(SWT.Selection, new Listener() {
			public void handleEvent(Event event) {
			}
		});
*/
		Dialog.applyDialogFont(composite);
		setControl(container);
		validatePage();
	}

	private boolean validatePage() {
		if (fileRadio.getSelection()) {
			pageValid = validateFile();
		} else {
			pageValid = true;
		}
		if (pageValid) {
			setMessage(null);
			setErrorMessage(null);
		}
		setPageComplete(pageValid);
		return pageValid;
	}
	
	private boolean validateFile() {
		final String pathString = filePath.getText().trim();
		if (pathString.length() == 0 || !new Path("").isValidPath(pathString)) { //$NON-NLS-1$
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemInvalid);
			return false;
		}
	
		final File file = new File(pathString);
		if (!file.isAbsolute()) {
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemInvalid);
			return false;
		}
	
		if (file.isDirectory()) {
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemInvalid);
			return false;
		}
	
		if (pathString.endsWith("/") || pathString.endsWith("\\")) { //$NON-NLS-1$//$NON-NLS-2$
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemInvalid);
			return false;
		}
	
		final File parent = file.getParentFile();
		if (!(parent.exists() && parent.isDirectory())) {
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemInvalid);
			return false;
		}
		if (file.exists() && !file.canWrite()) {
			setErrorMessage(UITexts.CreatePatchWizard_FilesystemReadOnly);
			return false;
		}
		
		return true;
	}
	
	
	private String createFileName() {
		String suggestedFileName = project.getName().concat(".patch"); //$NON-NLS-1$
		String path = getDialogSettings().get(PATH_KEY);
		if (path != null) {
			return Path.fromPortableString(path).append(suggestedFileName).toOSString();
		}
		return (new File(System.getProperty("user.dir", ""), suggestedFileName)).getPath(); //$NON-NLS-1$ //$NON-NLS-2$
	}

	public File getFile() {
		if (pageValid && fileRadio.getSelection()) {
			return new File(filePath.getText().trim());
		}
		return null;
	}

	public List<IBzrResourceStatus> getSelectedResources() {
		return resourceSelection.getSelection();
	}

}
