/*******************************************************************************
 * Copyright (c) 2013 Piotr Piastucki
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *******************************************************************************/
package org.vcs.bazaar.eclipse.ui.wizards.patch;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.IDialogConstants;
import org.eclipse.jface.dialogs.IDialogSettings;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.dnd.Clipboard;
import org.eclipse.swt.dnd.TextTransfer;
import org.eclipse.swt.dnd.Transfer;
import org.vcs.bazaar.eclipse.core.model.IBzrResourceStatus;
import org.vcs.bazaar.eclipse.ui.EclipseBazaarUI;
import org.vcs.bazaar.eclipse.ui.UIIcons;
import org.vcs.bazaar.eclipse.ui.UITexts;
import org.vcs.bazaar.eclipse.ui.operations.CreatePatchOperation;

/**
 * @author Piotr Piastucki
 */
public class CreatePatchWizard extends Wizard {

	private CreatePatchPage page;
	private List<IBzrResourceStatus> resources;
	private List<IBzrResourceStatus> selectedResources;
	private static final String DIALOG_SETTINGS_SECTION = "CreatePatchWizard";
	
	public CreatePatchWizard(List<IBzrResourceStatus> resources, List<IBzrResourceStatus> selectedResources) {
		this.resources = resources;
		this.selectedResources = selectedResources;
		setWindowTitle(UITexts.CreatePatchWizard_WindowTitle);
		IDialogSettings section = EclipseBazaarUI.getDefault().getDialogSettings().getSection(DIALOG_SETTINGS_SECTION);
		if (section == null) {
			section = EclipseBazaarUI.getDefault().getDialogSettings().addNewSection(DIALOG_SETTINGS_SECTION);
		}
		setDialogSettings(section);
	}

	@Override
	public void addPages() {
		String pageTitle = UITexts.CreatePatchWizard_Title;
		String pageDescription = UITexts.CreatePatchWizard_Description;
		page = new CreatePatchPage(pageTitle, pageTitle, UIIcons.WIZBAN_CREATE_PATCH, resources, selectedResources);
		page.setDescription(pageDescription);
		addPage(page);
	}
	
	@Override
	public boolean performFinish() {
		final File file = page.getFile();
		if (file != null && !validateFile(file)) {
			return false;
		}
		final CreatePatchOperation operation = new CreatePatchOperation(getContainer(), page.getSelectedResources(), null, null);
		try {
			getContainer().run(true, true, new IRunnableWithProgress() {
				public void run(IProgressMonitor monitor) throws InvocationTargetException {
					try {
						operation.run(monitor);
						String content = operation.getPatchContent();
						if (file != null) {
							writeToFile(file, content);
							IFile[] files = ResourcesPlugin.getWorkspace().getRoot().findFilesForLocationURI(file.toURI());
							for (int i = 0; i < files.length; i++)
								files[i].refreshLocal(IResource.DEPTH_ZERO,monitor);
						} else {
							copyToClipboard(content);
						}
					} catch (Exception e) {
						throw new InvocationTargetException(e);
					}
				}
			});
		} catch (Exception e) {
			((WizardPage)getContainer().getCurrentPage()).setErrorMessage(e.getMessage() == null ? e.getMessage() : UITexts.CreatePatchWizard_InternalError);
		}
		return true;
	}

	private void copyToClipboard(final String content) {
		getShell().getDisplay().syncExec(new Runnable() {
			public void run() {
				TextTransfer plainTextTransfer = TextTransfer.getInstance();
				Clipboard clipboard = new Clipboard(getShell().getDisplay());
				clipboard.setContents(new String[] { content }, new Transfer[] { plainTextTransfer });
				clipboard.dispose();
			}
		});
	}
	
	private boolean validateFile(File file) {
		if (file.exists()) {
			final MessageDialog dialog = new MessageDialog(getShell(), 
															UITexts.CreatePatchWizard_OverwriteTitle, 
															null, 
															UITexts.CreatePatchWizard_OverwriteMsg, 
															MessageDialog.QUESTION, 
															new String[] { IDialogConstants.YES_LABEL, IDialogConstants.CANCEL_LABEL }, 0);
			dialog.open();
			if (dialog.getReturnCode() != 0) {
				return false;
			}
		}
		return true;
	}

	private void writeToFile(final File file, String content) throws IOException {
		Writer output = new BufferedWriter(new FileWriter(file));
		try {
			output.write(content);
		} finally {
			output.close();
		}
	}
	
}
